SystemOrganization addCategory: #CodeCityTest!

TestCase subclass: #AbstractGlyphTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!AbstractGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAddMapperOnGlyphProperty
	| glyph colorMapper alphaMapper newAlpha newColor |
	newColor := ColorValue red.
	newAlpha := 0.3.
	colorMapper := ConstantMapper new constantValue: newColor.
	alphaMapper := ConstantMapper new constantValue: newAlpha.
	glyph := AbstractGlyph new.
	glyph addMapper: colorMapper onGlyphProperty: #color.
	self assert: ((glyph mapperForProperty: #color) computeValueOn: nil) = newColor.
	self assert: ((glyph mapperForProperty: #alpha) computeValueOn: nil) = AbstractNodeGlyph defaultAlpha.
	glyph addMapper: alphaMapper onGlyphProperty: #alpha.
	self assert: ((glyph mapperForProperty: #alpha) computeValueOn: nil) = newAlpha! !

!AbstractGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testDefaultValueForProperty
	self assert: (AbstractGlyph defaultValueForProperty: #color) 
				= AbstractGlyph defaultColor.
	self assert: (AbstractGlyph defaultValueForProperty: #alpha) 
				= AbstractGlyph defaultAlpha! !

!AbstractGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInitialize
	|glyph expectedProperties|
	glyph := AbstractGlyph new.
	self assert: glyph color = glyph class defaultColor.
	self assert: glyph alpha = glyph class defaultAlpha.
	self assert: glyph highlighted = glyph class defaultHighlighted.
	expectedProperties := Set with: #color with: #alpha.
	self assert: glyph properties size = expectedProperties size.
	glyph properties do: [:each|self assert: (expectedProperties includes: each)]! !

!AbstractGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInitializeMappers
	| glyph |
	glyph := AbstractGlyph new.
	glyph properties do: 
			[:each | 
			| mapper |
			mapper := glyph mapperForProperty: each.
			self assert: mapper notNil.
			self assert: (mapper isKindOf: ConstantMapper).
			self assert: (mapper computeValueOn: nil) 
						= (AbstractGlyph defaultValueForProperty: each)]! !

!AbstractGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testMapperForProperty
	| glyph |
	glyph := AbstractGlyph new.
	self assert: ((glyph mapperForProperty: #color) computeValueOn: nil) 
				= AbstractGlyph defaultColor.
	self assert: ((glyph mapperForProperty: #alpha) computeValueOn: nil) 
				= AbstractGlyph defaultAlpha! !

!AbstractGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testProperties
	| glyph |
	glyph := AbstractGlyph new.
	self assert: glyph properties = (OrderedCollection with: #color with: #alpha)! !

!AbstractGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testRenderFigure
	| glyph |
	glyph := AbstractGlyph new.
	self should: [glyph renderFigure] raise: SubclassResponsibilityError! !

!AbstractGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testRenderingColor
	| glyph color highlightColor |
	glyph := AbstractGlyph new.
	color := ColorValue red.
	highlightColor := glyph class highlightColor.
	glyph color: color.
	self assert: glyph renderingColor = color.
	glyph highlighted: true.
	self assert: glyph renderingColor = highlightColor! !

TestCase subclass: #AbstractLayoutTest
	instanceVariableNames: 'epsilon'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!AbstractLayoutTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	epsilon := 0.0001! !

!AbstractLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAsScript
	|sourceLayout scriptedLayout|
	sourceLayout := SpiralLayout 
							newHorizontalMargin: 0.5
							horizontalGap: 1
							verticalMargin: 0.2
							verticalGap: 1.
	scriptedLayout := Compiler evaluate: sourceLayout asScript.
	self assert: sourceLayout class = scriptedLayout class.
	self assert: sourceLayout horizontalMargin = scriptedLayout horizontalMargin.
	self assert: sourceLayout horizontalGap = scriptedLayout horizontalGap.
	self assert: sourceLayout verticalMargin = scriptedLayout verticalMargin.
	self assert: sourceLayout verticalGap = scriptedLayout verticalGap.! !

!AbstractLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBumperElevate
	|bumperSize glyph startPosition endPosition layout|
	layout := AbstractLayout new.
	bumperSize := layout verticalMargin.
	startPosition := 3, 4, 5.
	endPosition := 3, 4, (5 + bumperSize).
	glyph := CuboidGlyph new basicPosition: startPosition.
	self assert: glyph notNil.
	self assert: glyph position = startPosition.
	layout bumperElevate: glyph.
	self assert: (glyph position - endPosition) < epsilon.! !

!AbstractLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testComputeLargestDimensionForGlyphs
	| glyphs dimension1 dimension2 dimension3 layout largestDimension|
	layout := AbstractLayout new.
	dimension1 := 5, 5, 5.
	dimension2 := 2, 8, 1.
	dimension3 := 7, 3, 4.
	glyphs := OrderedCollection 
					with: (CuboidGlyph new dimension: dimension1)
					with: (CuboidGlyph new dimension: dimension2)
					with: (CuboidGlyph new dimension: dimension3).
	self assert: (glyphs notNil and: [glyphs size = 3]).
	largestDimension := layout computeLargestDimensionForGlyphs: glyphs.	
	self assert: largestDimension =  (7, 8, 5)! !

!AbstractLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testDiameterOfCircumcircleForBoxSized
	|width length layout diameter|
	width := 3.
	length := 4.
	layout := AbstractLayout new.
	diameter := layout diameterOfCircumcircleForBoxSized: width, length.
	self assert: diameter = 5! !

!AbstractLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testEquals
	|l1 l2otherType|
	l1 := ChessboardLayout new.
	self assert: l1 = l1 copy.
	l2otherType := QuadraticLayout new.
	self deny: l1 = l2otherType.
	#(horizontalMargin horizontalGap verticalMargin verticalGap) do: [:each| | l3mod setter|
		setter := (each,':') asSymbol.
		l3mod := l1 copy perform: setter with: (l1 perform: each) * 2; yourself.
		self deny: l1 = l3mod.
	].! !

!AbstractLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testMoveToOriginSceneWithRadiusContainingFigures
	| glyphs position1 position2 position3 radius layout |
	layout := AbstractLayout new.
	radius := 11.
	position1 := 5, 5, 5.
	position2 := 2, 8, 1.
	position3 := 7, 3, 4.
	glyphs := OrderedCollection 
					with: (CuboidGlyph new basicPosition: position1)
					with: (CuboidGlyph new basicPosition: position2)
					with: (CuboidGlyph new basicPosition: position3).
	self assert: (glyphs notNil and: [glyphs size = 3]).
	layout moveToOriginSceneWithRadius: radius containingFigures: glyphs.
	self assert: (glyphs at: 1) position = (16, 16, 5).
	self assert: (glyphs at: 2) position = (13, 19, 1).
	self assert: (glyphs at: 3) position = (18, 14, 4)! !

TestCase subclass: #AbstractNodeGlyphTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAddIncomingEdge
	| source destination edge |
	source := AbstractNodeGlyph new.
	destination := AbstractNodeGlyph new.
	edge := AbstractEdgeGlyph from: source to: destination.
	self assert: destination incomingEdges isEmpty.
	destination addIncomingEdge: edge.
	self assert: destination incomingEdges size = 1.
	self assert: destination incomingEdges first = edge! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAddNode
	| parent child |
	parent := AbstractNodeGlyph new.
	child := AbstractNodeGlyph new.
	self assert: parent nodes isEmpty.
	parent addNode: child.
	self assert: parent nodes size = 1.
	self assert: parent nodes first = child.
	self assert: child parent = parent! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAddNodes
	| parent children |
	parent := AbstractNodeGlyph new.
	children := OrderedCollection 
				with: AbstractNodeGlyph new
				with: AbstractNodeGlyph new.
	self assert: parent nodes isEmpty.
	parent addNodes: children.
	self assert: parent nodes size = 2.
	self assert: parent nodes = children.
	self assert: children first parent = parent.
	self assert: children second parent = parent! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAddOutgoingEdge
	| source destination edge |
	source := AbstractNodeGlyph new.
	destination := AbstractNodeGlyph new.
	edge := AbstractEdgeGlyph from: source to: destination.
	self assert: source outgoingEdges isEmpty.
	source addOutgoingEdge: edge.
	self assert: source outgoingEdges size = 1.
	self assert: source outgoingEdges first = edge! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAerialConnectionPoint
	self deny: true! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testApplyLayout
	| parent children |
	parent := CuboidGlyph new.
	children := OrderedCollection 
				with: AbstractNodeGlyph new
				with: AbstractNodeGlyph new
				with: AbstractNodeGlyph new
				with: AbstractNodeGlyph new.
	parent addNodes: children.
	self assert: parent position = AbstractNodeGlyph defaultPosition.
	self assert: parent dimension = AbstractNodeGlyph defaultDimension.
	children do: [:each | self assert: each position = AbstractNodeGlyph defaultPosition].
	parent applyLayout.
	self assert: parent position = AbstractNodeGlyph defaultPosition.
	self deny: parent dimension = AbstractNodeGlyph defaultDimension.
	children do: [:each | self deny: each position = AbstractNodeGlyph defaultPosition].! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBasicPosition
	| parent child position |
	parent := AbstractNodeGlyph new.
	child := AbstractNodeGlyph new.
	parent addNode: child.
	position := 3,4,5.
	parent basicPosition: position.
	self assert: parent position = position.
	self assert: child position = AbstractNodeGlyph defaultPosition! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCenter
	| glyph |
	glyph := (AbstractNodeGlyph new)
				basicPosition: 3 , 4 , 5;
				dimension: 8 , 4 , 2;
				yourself.
	self assert: glyph center = (7 , 6 , 6)! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCompositePosition
	| parent child position |
	parent := AbstractNodeGlyph new.
	child := AbstractNodeGlyph new.
	parent addNode: child.
	position := 3,4,5.
	parent compositePosition: position.
	self assert: parent position = position.
	self assert: child position = position! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConnectionHeightIncrement
	self deny: true! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testDeepApplyLayout
	| parent children |
	parent := CuboidGlyph new.
	children := OrderedCollection 
				with: CuboidGlyph new
				with: CuboidGlyph new
				with: CuboidGlyph new
				with: CuboidGlyph new.
	parent addNodes: children.
	self assert: parent position = AbstractNodeGlyph defaultPosition.
	self assert: parent dimension = AbstractNodeGlyph defaultDimension.
	children do: [:each | self assert: each position = AbstractNodeGlyph defaultPosition].
	parent deepApplyLayout.
	self assert: parent position = AbstractNodeGlyph defaultPosition.
	self deny: parent dimension = AbstractNodeGlyph defaultDimension.
	children do: [:each | self deny: each position = AbstractNodeGlyph defaultPosition].! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testDimensionUndersizing
	| glyph width height length dimension defaultDimension |
	glyph := AbstractNodeGlyph new.
	dimension := 3, 4, 5.
	glyph dimension: dimension.
	self assert: glyph dimension = dimension.
	defaultDimension := AbstractNodeGlyph defaultDimension.
	width :=defaultDimension x / 2.
	length := defaultDimension y / 2.
	height := defaultDimension z / 2.
	glyph width: width.
	self deny: glyph width = width.
	self assert: glyph width = defaultDimension x.
	glyph length: length.
	self assert: glyph length = defaultDimension y.
	glyph height: height.
	self assert: glyph height = defaultDimension z! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testEastWallBottomCenter
	| glyph |
	glyph := (AbstractNodeGlyph new)
				basicPosition: 3 , 4 , 5;
				dimension: 8 , 4 , 2;
				yourself.
	self assert: glyph eastWallBottomCenter = (11 , 6 , 5)! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testHeightOfTallestChildConnector
	self deny: true! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testIsLeafNode
	| parent child1 child2 child11 |
	parent := AbstractNodeGlyph new.
	child1 := AbstractNodeGlyph new.
	child2 := AbstractNodeGlyph new.
	child11 := AbstractNodeGlyph new.
	child1 addNode: child11.
	parent
		addNode: child1;
		addNode: child2.
	self deny: parent isLeafNode.
	self deny: child1 isLeafNode.
	self assert: child2 isLeafNode.
	self assert: child11 isLeafNode! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testIsRootNode
	| parent child1 child2 child11 |
	parent := AbstractNodeGlyph new.
	child1 := AbstractNodeGlyph new.
	child2 := AbstractNodeGlyph new.
	child11 := AbstractNodeGlyph new.
	child1 addNode: child11.
	parent
		addNode: child1;
		addNode: child2.
	(OrderedCollection 
		with: child1
		with: child2
		with: child11) do: [:each | self deny: each isRootNode].
	self assert: parent isRootNode! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testNodesRecursive
	| parent children firstChildChildren secondChild firstChild secondChildChildren |
	parent := AbstractNodeGlyph new.
	firstChild := AbstractNodeGlyph new.
	firstChildChildren := OrderedCollection 
				with: AbstractNodeGlyph new with: AbstractNodeGlyph new.
	firstChild addNodes: firstChildChildren.
	secondChild := AbstractNodeGlyph new.
	secondChildChildren := OrderedCollection 
				with: AbstractNodeGlyph new with: AbstractNodeGlyph new with: AbstractNodeGlyph new.
	secondChild addNodes: secondChildChildren.
	children := OrderedCollection with: firstChild with: secondChild.
	parent addNodes: children.
	self assert: parent nodes size = 2.
	self assert: parent nodesRecursive size = 7.
	self assert: parent nodes size = 2! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testNorthWallBottomCenter
	| glyph |
	glyph := (AbstractNodeGlyph new)
				basicPosition: 3 , 4 , 5;
				dimension: 8 , 4 , 2;
				yourself.
	self assert: glyph northWallBottomCenter = (7 , 4 , 5)! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testProperties
	| glyph |
	glyph := AbstractNodeGlyph new.
	self assert: glyph properties = (OrderedCollection with: #color with: #alpha with: #width with: #length)! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testRootNode
	| parent child1 child2 child11 |
	parent := AbstractNodeGlyph new.
	child1 := AbstractNodeGlyph new.
	child2 := AbstractNodeGlyph new.
	child11 := AbstractNodeGlyph new.
	child1 addNode: child11.
	parent
		addNode: child1;
		addNode: child2.
	self assert: parent rootNode = parent.
	self assert: child1 rootNode = parent.
	self assert: child2 rootNode = parent.
	self assert: child11 rootNode = parent! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testSouthWallBottomCenter
	| glyph |
	glyph := (AbstractNodeGlyph new)
				basicPosition: 3 , 4 , 5;
				dimension: 8 , 4 , 2;
				yourself.
	self assert: glyph southWallBottomCenter = (7 , 8 , 5)! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testSubtreeHeight
	| parent child1 child2 child11 |
	parent := AbstractNodeGlyph new.
	child1 := AbstractNodeGlyph new.
	child2 := AbstractNodeGlyph new.
	child11 := AbstractNodeGlyph new.
	child1 addNode: child11.
	parent
		addNode: child1;
		addNode: child2.
	self assert: parent subtreeHeight = 2.
	self assert: child1 subtreeHeight = 1.
	self assert: child2 subtreeHeight = 0.
	self assert: child11 subtreeHeight = 0! !

!AbstractNodeGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testWestWallBottomCenter
	| glyph |
	glyph := (AbstractNodeGlyph new)
				basicPosition: 3 , 4 , 5;
				dimension: 8 , 4 , 2;
				yourself.
	self assert: glyph westWallBottomCenter = (3 , 6 , 5)! !

TestCase subclass: #ArchGlyphTest
	instanceVariableNames: 'mooseModel glyphs expectedJunctions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!ArchGlyphTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
modelSpec
	^
#(Moose.Model (entity
	(FAMIX.Package (id: 1) (name P1))
		(FAMIX.Package (id: 2) (name P2) (packagedIn (idref: 1)))
			(FAMIX.Class (id: 5) (name C5) (packagedIn (idref: 2)))
			(FAMIX.Class (id: 6) (name C6) (packagedIn (idref: 2)))
			(FAMIX.Class (id: 7) (name C7) (packagedIn (idref: 2)))
		(FAMIX.Package (id: 3) (name P3) (packagedIn (idref: 1)))
			(FAMIX.Class (id: 8) (name C8) (packagedIn (idref: 3)))
			(FAMIX.Class (id: 9) (name C9) (packagedIn (idref: 3)))
		(FAMIX.Class (id: 4) (name C4) (packagedIn (idref: 1)))
))! !

!ArchGlyphTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	mooseModel := (Importer withActiveMetamodel importArray: self modelSpec) 
				name: #TestModel.
	glyphs := self setUpGlyphs.
	expectedJunctions := OrderedCollection new
		add: 8.5, 10, 45;
		add: 5.5, 7, 22.5;
		add: 13.5, 5.5, 30;
		add: 2.5, 16.5, 8;
		add: 4, 5, 10;
		add: 8, 3, 15;
		add: 5, 10.5, 5;
		add: 13.5, 4, 3;
		add: 13, 8, 20;
		yourself! !

!ArchGlyphTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUpGlyphs
	|glyphCollection|
	glyphCollection := OrderedCollection new.
	1 to: 9 do: [:each | |glyphClass initial glyph model|
		each <= 3 ifTrue: [glyphClass := TerrainGlyph. initial := 'P'] ifFalse: [glyphClass := CuboidGlyph. initial := 'C'].
		model := mooseModel entityNamed: (initial, each printString) asSymbol.
		glyph := glyphClass new model: model.
		glyphCollection add: glyph].
	(glyphCollection at: 5) basicPosition: 2, 2, 2; dimension: 4, 6, 8.
	(glyphCollection at: 6) basicPosition: 7, 2, 2; dimension: 2, 2, 13.
	(glyphCollection at: 7) basicPosition: 2, 9, 2; dimension: 6, 3, 3.
	(glyphCollection at: 2) terrainThickness: 1; basicPosition: 1, 1, 1; dimension: 9, 12, 14.
	5 to: 7 do: [:each | (glyphCollection at: 2) addNode: (glyphCollection at: each)].
	(glyphCollection at: 8) basicPosition: 12, 2, 2; dimension: 3, 4, 1.
	(glyphCollection at: 9) basicPosition: 12, 7, 2; dimension: 2, 2, 18.
	(glyphCollection at: 3) terrainThickness: 1; basicPosition: 11, 1, 1; dimension: 5, 9, 19.
	8 to: 9 do: [:each | (glyphCollection at: 3) addNode: (glyphCollection at: each)].
	(glyphCollection at: 4) basicPosition: 1, 14, 1; dimension: 3, 5, 7.
	(glyphCollection at: 1) terrainThickness: 1; dimension: 17 , 20 , 20; basicPosition: 0 , 0 , 0.
	2 to: 3 do: [:each | (glyphCollection at: 1) addNode: (glyphCollection at: each)].
	(glyphCollection at: 1) addNode: (glyphCollection at: 4).
	^glyphCollection! !

!ArchGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testControlPoints
	|C5toC8 C4toC9 C6toC6 P3toC7 edge|
	C5toC8 := (#(5 2 1 3 8) collect: [:each | expectedJunctions at: each]) asOrderedCollection.
	C4toC9 := (OrderedCollection with: 4 with: 1 with: 3 with: 9) collect: [:each | expectedJunctions at: each].
	P3toC7 := (OrderedCollection with: 3 with: 1 with: 2 with: 7) collect: [:each | expectedJunctions at: each].
	C6toC6 := (OrderedCollection with: 6) collect: [:each | expectedJunctions at: each].
	edge := ArchGlyph new.
	edge fromNode: (glyphs at: 5); toNode: (glyphs at: 8).
	self assert: (edge controlPoints = C5toC8).
	"self assert: (tree junctionSequenceFromGlyph: (glyphs at: 5) to: (glyphs at: 8)) = C5toC8.
	self assert: (tree junctionSequenceFromGlyph: (glyphs at: 4) to: (glyphs at: 9)) = C4toC9.
	self assert: (tree junctionSequenceFromGlyph: (glyphs at: 3) to: (glyphs at: 7)) = P3toC7.
	self assert: (tree junctionSequenceFromGlyph: (glyphs at: 6) to: (glyphs at: 6)) = C6toC6."! !

!ArchGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testNodeSequenceFromRoot
	|rootToC5 rootToC8 rootToC4 rootToRoot nodes|
	"nodes := glyphs collect: [:each | tree nodeForModel: each model]."
	rootToC5 := (OrderedCollection with: 1 with: 2 with: 5) collect: [:each | nodes at: each].
	rootToC8 := (OrderedCollection with: 1 with: 3 with: 8) collect: [:each | nodes at: each].
	rootToC4 := (OrderedCollection with: 1 with: 4) collect: [:each | nodes at: each].
	rootToRoot := (OrderedCollection with: 1) collect: [:each | nodes at: each].
	"self assert: (tree nodeSequenceFromRoot: (nodes at: 5)) = rootToC5.
	self assert: (tree nodeSequenceFromRoot: (nodes at: 8)) = rootToC8.
	self assert: (tree nodeSequenceFromRoot: (nodes at: 4)) = rootToC4.
	self assert: (tree nodeSequenceFromRoot: (nodes at: 1)) = rootToRoot."

	self deny: true! !

!ArchGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testNodeSequenceToRoot
	|fromC5 fromC8 fromC4 rootToRoot nodes|
	"nodes := glyphs collect: [:each | tree nodeForModel: each model]."
	fromC5 := (OrderedCollection with: 5 with: 2 with: 1) collect: [:each | nodes at: each].
	fromC8 := (OrderedCollection with: 8 with: 3 with: 1) collect: [:each | nodes at: each].
	fromC4 := (OrderedCollection with: 4 with: 1) collect: [:each | nodes at: each].
	rootToRoot := (OrderedCollection with: 1) collect: [:each | nodes at: each].
	
	"self assert: (tree nodeSequenceToRoot: (nodes at: 5)) = fromC5.
	self assert: (tree nodeSequenceToRoot: (nodes at: 8)) = fromC8.
	self assert: (tree nodeSequenceToRoot: (nodes at: 4)) = fromC4.
	self assert: (tree nodeSequenceToRoot: (nodes at: 1)) = rootToRoot."

	self deny: true! !

TestCase subclass: #BlockMapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!BlockMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAsScript
	| aBlock instantiatedBlockMapper uninstantiatedBlockMapper |
	aBlock := [:each | each class].
	instantiatedBlockMapper := BlockMapper new block: aBlock.
	self assert: instantiatedBlockMapper 
				= (Compiler evaluate: instantiatedBlockMapper asScript).
	uninstantiatedBlockMapper := BlockMapper new.
	self assert: uninstantiatedBlockMapper 
				= (Compiler evaluate: uninstantiatedBlockMapper asScript)! !

!BlockMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testComputeValueOn
	| mapper model1 value1 model2  |
	mapper := BlockMapper new block: [:each | each count: [:e|e > 1]].
	model1 := #(1 2 3 4 5).
	value1 := mapper computeValueOn: model1.
	self assert: value1 = 4.
	model2 := nil.
	self should: [mapper computeValueOn: model2] raise: MessageNotUnderstood! !

!BlockMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testEquality
	| i1 i2 notI aBlock |
	aBlock := [:each | each size printString].
	i1 := BlockMapper new
			block:  aBlock.
	i2 := i1 copy.
	self assert: i1 = i2.
	#(#block:) do: 
			[:each | 
			notI := i1 copy perform: each with: nil.
			self deny: notI = i1]! !

TestCase subclass: #BlueToRedColorSchemeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!BlueToRedColorSchemeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testColorAtIndex
	| colorScheme middleColor |
	colorScheme := BlueToRedColorScheme newResolution: 5.
	middleColor := ColorValue 
			red: (ColorValue blue red + ColorValue red red) / 2
			green: (ColorValue blue green + ColorValue red green) / 2
			blue: (ColorValue blue blue + ColorValue red blue) / 2.
	self assert: (colorScheme colorAtIndex: 1) = ColorValue blue.
	self assert: (colorScheme colorAtIndex: 5) = ColorValue red.
	self assert: (colorScheme colorAtIndex: 3) = middleColor! !

TestCase subclass: #BoxPlotConverterTest
	instanceVariableNames: 'epsilon valuesWithExtremes valuesWithoutExtremes boxPlotWithExtremes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!BoxPlotConverterTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	epsilon := 0.0001.
	valuesWithExtremes := #(8.25 18.25 11.5 11.6 11.9 14.5 12.2 12.8 12.9 18.26 13.1 13.3 13.4 13.8 13.9 14.2 12.0 14.5 14.6 15.3 12.9 8.24).
	valuesWithoutExtremes := #(8.25 18.25 11.5 11.6 11.9 14.5 12.2 12.8 12.9 18.24 13.1 13.3 13.4 13.8 13.9 14.2 12.0 14.5 14.6 15.3 12.9 8.26).
	boxPlotWithExtremes := BoxPlotConverter newValues: valuesWithExtremes! !

!BoxPlotConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConvertForValuesWithOutlier
	| discretizer inputValues numberOfInterquartileValues |
	inputValues := #(-55 -20 -12 -11 -10 -7 -4  0  1  1  1  1  1  1  1  1  1   
						  1   1   2   2   2  2  2  2  2  3  3  3  3  3  3  3  4
						  4   4   4   4   5  5  6  7  7  7  7  7  8  8  9  9  9  
						  9   9   9  10  11 11 12 16 17 19 21 21 23 25 26 30 49).
	discretizer := BoxPlotConverter newValues: inputValues.
	self assert: (discretizer convert: -56) isNil.
	self assert: (discretizer convert: -55) = 1.
	self assert: (discretizer convert: -12) = 1.
	self assert: (discretizer convert: -11) = 2.
	self assert: (discretizer convert: -10) = 2.
	self assert: (discretizer convert: 0) = 2.
	self assert: (discretizer convert: 1) = 3.
	self assert: (discretizer convert: 3) = 3.
	self assert: (discretizer convert: 4) = 4.
	self assert: (discretizer convert: 5) = 4.
	self assert: (discretizer convert: 9) = 4.
	self assert: (discretizer convert: 10) = 5.
	self assert: (discretizer convert: 21) = 5.
	self assert: (discretizer convert: 23) = 6.
	self assert: (discretizer convert: 49) = 6.
	self assert: (discretizer convert: 50) isNil.
	numberOfInterquartileValues := inputValues count: 
			[:each | discretizer isInInterquartileRange: each].
	self assert: numberOfInterquartileValues > (inputValues size / 2)! !

!BoxPlotConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConvertForValuesWithVoidIntervals
	| discretizer inputValues numberOfInterquartileValues |
	inputValues := #(1 1 1  1   1  1  1  1   1  1  1  1   1  1  1  1   1
				       1 1 2  2   2  2  2  2   2  3  3  3   3  3  3  3   4
				       4 4 4  4   5  5  6  7   7  7  7  7   8  8  9  9   9
				       9 9 9 10  11 11 12 16  17 19 21 21  23 25 26 30  49).
	discretizer := BoxPlotConverter newValues: inputValues.
	self assert: (discretizer convert: 0) isNil.
	self assert: (discretizer convert: 1) = 3.
	self assert: (discretizer convert: 3) = 3.

	self assert: (discretizer convert: 4) = 4.
	self assert: (discretizer convert: 9) = 4.
	self assert: (discretizer convert: 10) = 5.
	self assert: (discretizer convert: 21) = 5.
	self assert: (discretizer convert: 22) = 6.
	self assert: (discretizer convert: 49) = 6.
	self assert: (discretizer convert: 50) isNil.
	numberOfInterquartileValues := inputValues count: 
			[:each | discretizer isInInterquartileRange: each].
	self assert: numberOfInterquartileValues > (inputValues size / 2)! !

!BoxPlotConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConvertWithVoidRange
	| converter |
	converter := BoxPlotConverter newValues: #(3 3 3 3).
	self assert: (converter convert: 3) = 4 "interquartile (somewhere in the middle)"! !

!BoxPlotConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCovertForNaughtyValues
	|values discretizer numberOfInterquartileValues |
	values := #(0 0 0 0 0 0 0 0 0 0 0 0 1 5 9).
	discretizer := BoxPlotConverter newValues: values.
	self assert: (discretizer convert: -1) isNil.
	self assert: (discretizer convert: 0) = 4.
	self assert: (discretizer convert: 1) = 6.
	self assert: (discretizer convert: 5) = 6.
	self assert: (discretizer convert: 9) = 6.
	self assert: (discretizer convert: 10) isNil.
	numberOfInterquartileValues := values count: 
			[:each | (discretizer convert: each) = 3 or: [(discretizer convert: each) = 4]].
	self assert: numberOfInterquartileValues > (values size / 2)! !

!BoxPlotConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testIsAboveIntertquartileRange
	self assert: (boxPlotWithExtremes isAboveInterquartileRange: 14.5) not.
	self assert: (boxPlotWithExtremes isAboveInterquartileRange: 14.6).! !

!BoxPlotConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testIsBelowIntertquartileRange
	self assert: (boxPlotWithExtremes isBelowInterquartileRange: 11.9).
	self assert: (boxPlotWithExtremes isBelowInterquartileRange: 12.0) not.! !

!BoxPlotConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testIsInIntertquartileRange
	self assert: (boxPlotWithExtremes isInInterquartileRange: 11.9) not.
	self assert: (boxPlotWithExtremes isInInterquartileRange: 12.0).
	self assert: (boxPlotWithExtremes isInInterquartileRange: 14.5).
	self assert: (boxPlotWithExtremes isInInterquartileRange: 14.6) not.! !

!BoxPlotConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testIsLowerOutlier
	self assert: (boxPlotWithExtremes isLowerExtreme: 8.24).
	self assert: (boxPlotWithExtremes isLowerExtreme: 8.25) not.! !

!BoxPlotConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testIsUpperOutlier
	self assert: (boxPlotWithExtremes isUpperExtreme: 18.25) not.
	self assert: (boxPlotWithExtremes isUpperExtreme: 18.26).! !

!BoxPlotConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testNumberSummary
	self assert: (boxPlotWithExtremes lowerExtremeValueLimit - 8.25) abs < epsilon.
	self assert: (boxPlotWithExtremes lowerQuartile - 12.0) abs < epsilon.
	self assert: (boxPlotWithExtremes median - 13.2) abs < epsilon.
	self assert: (boxPlotWithExtremes upperQuartile - 14.5) abs < epsilon.
	self assert: (boxPlotWithExtremes upperExtremeValueLimit - 18.25) abs < epsilon! !

TestCase subclass: #BoxplotMapperModelTest
	instanceVariableNames: 'glyphProperty modelClass initialMapper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!BoxplotMapperModelTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	glyphProperty := #color.
	modelClass := FAMIXMethod.
	initialMapper := BoxplotMapper new
		function: #numberOfLinesOfCode;
		codomain: #(2 4 6 8 10).! !

!BoxplotMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCodomain
	| mapperModel |
	mapperModel := BoxplotMapperModel newMapper: nil forModelClass: modelClass.
	self assert: mapperModel codomain value isNil.
	mapperModel := BoxplotMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel codomain value = initialMapper codomain! !

!BoxplotMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCreation
	| mapperModel |
	mapperModel := BoxplotMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel mapper = initialMapper! !

!BoxplotMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInitializeCodomain
	| mapperModel |
	mapperModel := BoxplotMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel initializeCodomain = initialMapper codomain! !

!BoxplotMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInitializeModelProperty
	| mapperModel |
	mapperModel := BoxplotMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel initializeModelProperty = initialMapper function! !

!BoxplotMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testModelProperty
	| mapperModel |
	mapperModel := BoxplotMapperModel newMapper: nil forModelClass: modelClass.
	self assert: mapperModel modelProperty value isNil.
	mapperModel := BoxplotMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel modelProperty value = initialMapper function! !

!BoxplotMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testModelPropertyList
	| mapperModel |
	mapperModel := BoxplotMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel modelPropertyList
		equals: (modelClass new allDeclaredProperties 
				collect: [:e | e definingMethod selector])! !

!BoxplotMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testOutputMapper
	| mapperModel newCodomain newModelProperty resultingSpecModel |
	mapperModel := BoxplotMapperModel newMapper: initialMapper forModelClass: modelClass.
	newCodomain := #(10 9 8 7 5).
	newModelProperty := #numberOfStatements.
	mapperModel modelProperty value: newModelProperty.
	mapperModel codomain value: newCodomain.
	resultingSpecModel := (initialMapper copy)
				function: newModelProperty;
				codomain: newCodomain;
				yourself.
	self assert: mapperModel outputMapper = resultingSpecModel! !

TestCase subclass: #BoxplotMapperTest
	instanceVariableNames: 'mooseModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!BoxplotMapperTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
modelSpec
	^
#(Moose.Model (entity
	(FAMIX.Namespace (id: 1) (name TestNamespace))

	(FAMIX.Class (id: 2) (name TestClass1) (belongsTo (idref: 1)))
	(FAMIX.Method (name method1TC1) (signature 'method1TC1()') (belongsTo (idref: 2)) (LOC 2))

	(FAMIX.Class (id: 3) (name TestClass3) (belongsTo (idref: 1)))
	(FAMIX.Method (name method1TC3) (signature 'method1TC3()') (belongsTo (idref: 3)) (LOC 2))
	(FAMIX.Method (name method2TC3) (signature 'method2TC3()') (belongsTo (idref: 3)) (LOC 2))
	(FAMIX.Method (name method3TC3) (signature 'method3TC3()') (belongsTo (idref: 3)) (LOC 2))

	(FAMIX.Class (id: 4) (name TestClass5) (belongsTo (idref: 1)))
	(FAMIX.Method (name method1TC5) (signature 'method1TC5()') (belongsTo (idref: 4)) (LOC 2))
	(FAMIX.Method (name method2TC5) (signature 'method2TC5()') (belongsTo (idref: 4)) (LOC 2))
	(FAMIX.Method (name method3TC5) (signature 'method3TC5()') (belongsTo (idref: 4)) (LOC 2))
	(FAMIX.Method (name method4TC5) (signature 'method4TC5()') (belongsTo (idref: 4)) (LOC 2))
	(FAMIX.Method (name method5TC5) (signature 'method5TC5()') (belongsTo (idref: 4)) (LOC 2))

	(FAMIX.Class (id: 5) (name TestClass7) (belongsTo (idref: 1)))
	(FAMIX.Method (name method1TC7) (signature 'method1TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method2TC7) (signature 'method2TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method3TC7) (signature 'method3TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method4TC7) (signature 'method4TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method5TC7) (signature 'method5TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method6TC7) (signature 'method6TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method7TC7) (signature 'method7TC7()') (belongsTo (idref: 5)) (LOC 2))

))! !

!BoxplotMapperTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	mooseModel := (Importer withActiveMetamodel importArray: self modelSpec) name: #'TestModel'! !

!BoxplotMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAsScript
	| aFunction instantiatedBlockMapper uninstantiatedBlockMapper population codomain scripted |
	aFunction := #size.
	population := OrderedCollection with: #(1 2 3) with: #($a $b $c $d) with: 'defgh'.
	codomain := #(1 4 5 6 8 11).
	instantiatedBlockMapper := BoxplotMapper new 
										function: aFunction;
										population: population;
										codomain: codomain.
	self assert: instantiatedBlockMapper 
				= (Compiler evaluate: instantiatedBlockMapper asScript).
	uninstantiatedBlockMapper := BoxplotMapper new.
	self assert: uninstantiatedBlockMapper 
				= (Compiler evaluate: uninstantiatedBlockMapper asScript).
	scripted := Compiler evaluate: instantiatedBlockMapper asScript.
	self assert: scripted population isEmpty! !

!BoxplotMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildConverter
	| mapper converter |
	mapper := (BoxplotMapper new)
				population: mooseModel allClasses;
				function: #numberOfMethods;
				yourself.
	converter := mapper buildConverter.
	self assert: converter class = BoxPlotConverter.	
	self assert: converter lowerExtremeValueLimit = -4.
	self assert: converter lowerQuartile = 2.
	self assert: converter median = 4.
	self assert: converter upperQuartile = 6.
	self assert: converter upperExtremeValueLimit = 12! !

!BoxplotMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testComputeValueOn
	| mapper mappedValues |
	mapper := (BoxplotMapper new)
				population: mooseModel allClasses;
				function: #numberOfMethods;
				codomain: #(2 5 11 23 44 65);
				yourself.
	mappedValues := mooseModel allClasses 
				collect: [:each | mapper computeValueOn: each].
	self assert: mappedValues equals: #(5 11 23 44)! !

!BoxplotMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testEquality
	| i1 i2 notI stillI1 |
	i1 := BoxplotMapper new
			function:  #size;
			codomain: #(1 2 3 4).
	i2 := i1 copy.
	self assert: i1 = i2.
	#(#function: #codomain:) do: 
			[:each | 
			notI := i1 copy perform: each with: nil.
			self deny: notI = i1].
	"population is not taken into account for the equality test"
	stillI1 := i1 copy perform: #population: with: #(1 2).
	self assert: stillI1 = i1! !

TestCase subclass: #BricksLayoutTest
	instanceVariableNames: 'glyphs epsilon mooseModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!BricksLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testLayoutGlyphsOriginSameSizes
	"self run: #testLayoutGlyphsOriginSameSizes"
	| layout origin hGap vMargin box expectedBox relativePositions biasedPositions hMargin vGap actualPositions|
	epsilon := 0.00001.
	origin := 3 , 4 , 5.
	glyphs := (1 to: 7) 
				collect: [:each | CuboidGlyph new dimension: 7 , 11 , 13].
	layout := BricksLayout 
				newHorizontalMargin: 0.7
				horizontalGap: 1
				verticalMargin: 0.2
				verticalGap: 2.
	self assert: (glyphs notNil and: [glyphs size = 7]).
	box := layout layoutGlyphs: glyphs origin: origin.
	hMargin := layout horizontalMargin.
	hGap := layout horizontalGap.
	vMargin := layout verticalMargin.
	vGap := layout verticalGap.
	expectedBox := 
			(hMargin + 7 + hGap + 7 + hMargin), 
			(hMargin + 11 + hGap + 11 + hMargin), 
			(vMargin + 13 + vGap + 13 + vMargin).

	self assert: (box x - expectedBox x) abs < epsilon.
	self assert: (box y - expectedBox y) abs < epsilon.
	self assert: (box z - expectedBox z) abs < epsilon.

	relativePositions := (OrderedCollection new)
				add: hMargin , hMargin , vMargin;
				add: (hMargin + 7 + hGap) , hMargin , vMargin;
				add: hMargin , (hMargin + 11 + hGap) , vMargin;
				add: (hMargin + 7 + hGap) , (hMargin + 11 + hGap) , vMargin;
				add: hMargin , hMargin , (vMargin + 13 + vGap);
				add: (hMargin + 7 + hGap) , hMargin , (vMargin + 13 + vGap);
				add: hMargin , (hMargin + 11 + hGap) , (vMargin + 13 + vGap);
				yourself.
	biasedPositions := relativePositions collect: [:each | each + origin].
	actualPositions := glyphs collect: [:each | each position].
	1 to: actualPositions size do: [:each| |expected actual|
		expected := biasedPositions at: each.
		actual := actualPositions at: each.
		self assert: (expected - actual) abs < epsilon
	]! !

!BricksLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testLayoutGlyphsOriginVariousSizes
	"self run: #testLayoutGlyphsOriginVariousSizes"
	| layout origin hGap vMargin box expectedBox relativePositions biasedPositions hMargin vGap actualPositions dimensions cellSize|
	epsilon := 0.00001.
	origin := 3 , 4 , 5.
	dimensions := (OrderedCollection new)
				add: 3 , 1 , 1; add: 5 , 8 , 2; 
				add: 7 , 2 , 3; add: 6 , 6 , 4; 
				
				add: 4 , 3 , 5; add: 1 , 2 , 6;
				add: 4 , 9 , 7;
				yourself.
	glyphs := dimensions collect: [:each | CuboidGlyph new dimension: each].
	layout := BricksLayout newHorizontalMargin: 3 horizontalGap: 1 
				verticalMargin: 4 verticalGap: 2.
	cellSize := layout computeLargestDimensionForGlyphs: glyphs.
	self assert: cellSize = (7 , 9 , 7).
	self assert: (glyphs notNil and: [glyphs size = 7]).
	box := layout layoutGlyphs: glyphs origin: origin.
	hMargin := layout horizontalMargin.
	hGap := layout horizontalGap.
	vMargin := layout verticalMargin.
	vGap := layout verticalGap.
	expectedBox := (hMargin + 7 + hGap + 7 + hMargin), (hMargin + 9 + hGap + 9 + hMargin), (vMargin + 7 + vGap + 7 + vMargin).
	self assert: (box x - expectedBox x) abs < epsilon.
	self assert: (box y - expectedBox y) abs < epsilon.
	self assert: (box z - expectedBox z) abs < epsilon.
	relativePositions := (OrderedCollection new)
				add: (hMargin , hMargin , vMargin) + (2,4,3);
				add: ((hMargin + 7 + hGap) , hMargin , vMargin) + (1,0.5,2.5);
				add: (hMargin , (hMargin + 9 + hGap) , vMargin) + (0,3.5,2);
				add: ((hMargin + 7 + hGap) , (hMargin + 9 + hGap) , vMargin) + (0.5,1.5,1.5);
				add: (hMargin , hMargin , (vMargin + 7 + vGap)) + (1.5,3,1);
				add: ((hMargin + 7 + hGap) , hMargin , (vMargin + 7 + vGap)) + (3,3.5,0.5);
				add: (hMargin , (hMargin + 9 + hGap) , (vMargin + 7 + vGap)) + (1.5,0,0);
				yourself.
	biasedPositions := relativePositions collect: [:each | each + origin].
	actualPositions := glyphs collect: [:each | each position].
	1 to: actualPositions size do: [:each| |expected actual|
		expected := biasedPositions at: each.
		actual := actualPositions at: each.
		self assert: (expected - actual) abs < epsilon
	]! !

TestCase subclass: #ChessboardLayoutTest
	instanceVariableNames: 'glyphs epsilon'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!ChessboardLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testLayoutGlyphsOrigin
	| dimensions layout origin hGap hMargin vMargin largestSize box relativePositions biasedPositions actualPositions|
	epsilon := 0.000001.
	origin := 3 , 4 , 5.
	dimensions := (OrderedCollection new)
				add: 3 , 1 , 1; add: 5 , 8 , 2; add: 7 , 2 , 3;
				add: 6 , 6 , 4; add: 4 , 3 , 5; add: 1 , 2 , 6;
				add: 4 , 9 , 7;
				yourself.
	glyphs := dimensions collect: [:each | CuboidGlyph new dimension: each].
	layout := ChessboardLayout newHorizontalMargin: 0.7 horizontalGap: 1 verticalMargin: 0.2 verticalGap: 2.
	self assert: (glyphs notNil and: [glyphs size = dimensions size]).
	self assert: (glyphs collect: [:each | each dimension]) = dimensions.
	box := layout layoutGlyphs: glyphs origin: origin.
	hGap := layout horizontalGap.
	hMargin := layout horizontalMargin.
	vMargin := layout verticalMargin.
	largestSize := layout computeLargestDimensionForGlyphs: glyphs.
	self assert: largestSize = (7 , 9 , 7).
	self assert: (box x - (3 * largestSize x + (2 * hGap) + (2 * hMargin))) abs < epsilon.
	self assert: (box y - (3 * largestSize y + (2 * hGap) + (2 * hMargin))) abs < epsilon.
	self assert: (box z - (largestSize z + (2 * vMargin))) abs < epsilon.
	"it arranges them column-wise (on y axis) and get to next column increasing x"
	relativePositions := OrderedCollection new
		add: (hMargin + 2), (hMargin + 4), vMargin;
		add: (hMargin + 1), (hMargin + 9 + hGap + 0.5), vMargin;		
		add: (hMargin + 0), (hMargin + 9 + hGap + 9 + hGap + 3.5), vMargin;
		add: (hMargin + 7 + hGap + 0.5), (hMargin + 1.5), vMargin;
		add: (hMargin + 7 + hGap + 1.5), (hMargin + 9 + hGap + 3), vMargin;
		add: (hMargin + 7 + hGap + 3), (hMargin + 9 + hGap + 9 + hGap + 3.5), vMargin;
		add: (hMargin + 7 + hGap + 7 + hGap + 1.5), (hMargin + 0), vMargin;
		yourself.
	biasedPositions := relativePositions collect: [:each|each + origin].
	actualPositions := glyphs collect: [:each|each position].
	self assert: actualPositions size = biasedPositions size.
	1 to: actualPositions size do: [:eachIndex|
		self assert: 
		((actualPositions at: eachIndex) - (biasedPositions at: eachIndex)) abs < epsilon]! !

TestCase subclass: #CityEdgeShapeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!CityEdgeShapeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testGlyphForFromTo
	| shape edge node1 node2 |
	shape := CityEdgeShape new.
	node1 := CuboidGlyph new model: 1.
	node2 := CuboidGlyph new model: 2.
	edge := shape glyphFor: 3 from: node1 to: node2.
	self assert: edge class = LineGlyph.
	self assert: edge model = 3! !

TestCase subclass: #CityNodeShapeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!CityNodeShapeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testGlyphFor
	| shape model glyph |
	shape := CityNodeShape new cuboid.
	model := 1.
	glyph := shape glyphFor: model.
	self assert: glyph class = CuboidGlyph.
	self assert: glyph model = 1! !

TestCase subclass: #CityShapeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!CityShapeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testDoesNotUnderstand
	| shape block |
	shape := CityShape new.
	block := [:each | ColorValue blue].
	self assert: shape mappers isEmpty.
	shape color: block.
	self assert: shape mappers size = 1.
	self assert: (shape mappers at: #color) = (BlockMapper new block: block)! !

TestCase subclass: #ColorLinearMapperModelTest
	instanceVariableNames: 'initialMapper glyphProperty modelClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!ColorLinearMapperModelTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	glyphProperty := #height.
	modelClass := FAMIXMethod.
	initialMapper := ColorLinearMapper new 
		function: #numberOfLinesOfCode;
		startColor: ColorValue blue;
		endColor: ColorValue red;
		spectrumwise: true! !

!ColorLinearMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCreation
	| mapperModel |
	mapperModel := ColorLinearMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel mapper = initialMapper! !

!ColorLinearMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInitializeEndColor
	| nilHandlingModel modelWithNilAttributes mapperWithNilAttributes |
	nilHandlingModel := ColorLinearMapperModel newMapper: nil forModelClass: modelClass.
	self assert: nilHandlingModel endColor value 
				= nilHandlingModel defaultEndColor value.
	mapperWithNilAttributes := initialMapper
				endColor: nil;
				yourself.
	modelWithNilAttributes := ColorLinearMapperModel newMapper: mapperWithNilAttributes forModelClass: modelClass.
	self assert: modelWithNilAttributes endColor value 
				= nilHandlingModel defaultEndColor value! !

!ColorLinearMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInitializeModelProperty
	| mapperModel |
	mapperModel := ColorLinearMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel initializeModelProperty = initialMapper function! !

!ColorLinearMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInitializeSpectrumwise
	| nilHandlingModel lackingSpecModel mapperWithNilAttributes |
	nilHandlingModel := ColorLinearMapperModel newMapper: nil forModelClass: modelClass.
	self assert: nilHandlingModel spectrumwise value 
				= nilHandlingModel defaultSpectrumwise value.
	mapperWithNilAttributes := initialMapper
				spectrumwise: nil;
				yourself.
	lackingSpecModel := ColorLinearMapperModel newMapper: mapperWithNilAttributes forModelClass: modelClass.
	self assert: lackingSpecModel spectrumwise value  
				= nilHandlingModel defaultSpectrumwise value! !

!ColorLinearMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInitializeStartColor
	| nilHandlingModel  modelWithNilAttributes mapperWithNilAttributes |
	nilHandlingModel := ColorLinearMapperModel newMapper: nil forModelClass: modelClass.
	self assert: nilHandlingModel startColor value 
				= nilHandlingModel defaultStartColor value.
	mapperWithNilAttributes := initialMapper
				startColor: nil;
				yourself.
	modelWithNilAttributes := ColorLinearMapperModel newMapper: mapperWithNilAttributes forModelClass: modelClass.
	self assert: modelWithNilAttributes startColor value 
				= nilHandlingModel defaultStartColor value! !

!ColorLinearMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testModelProperty
	| mapperModel |
	mapperModel := ColorLinearMapperModel newMapper: nil forModelClass: modelClass.
	self assert: mapperModel modelProperty value = nil.
	mapperModel := ColorLinearMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel modelProperty value = initialMapper function! !

!ColorLinearMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testModelPropertyList
	| mapperModel |
	mapperModel := ColorLinearMapperModel newMapper: initialMapper
				forModelClass: modelClass.
	self assert: mapperModel modelPropertyList
		equals: (modelClass new allDeclaredProperties 
				collect: [:e | e definingMethod selector])! !

!ColorLinearMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testOutputMapper
	| mapperModel newProperty newStartColor newEndColor resultingMapperModel |
	mapperModel := ColorLinearMapperModel newMapper: initialMapper forModelClass: modelClass.
	newProperty := #numberOfInvocations.
	newEndColor := ColorValue green.
	newStartColor := ColorValue yellow.
	mapperModel modelProperty value: newProperty.	
	mapperModel endColor value: newEndColor.
	mapperModel startColor value: newStartColor.
	mapperModel spectrumwise value: false.
	resultingMapperModel := (initialMapper copy)
				function: newProperty;
				startColor: newStartColor;
				endColor: newEndColor;
				spectrumwise: false.
	self assert: mapperModel outputMapper = resultingMapperModel! !

TestCase subclass: #ColorLinearMapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!ColorLinearMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAsScript
	| instantiatedBlockMapper uninstantiatedBlockMapper scripted |
	instantiatedBlockMapper := ColorLinearMapper new 
										function: #numberOfMethods;
										startColor: ColorValue white;
										endColor:  ColorValue black;
										spectrumwise: true.
	self assert: instantiatedBlockMapper 
				= (Compiler evaluate: instantiatedBlockMapper asScript).
	uninstantiatedBlockMapper := ColorLinearMapper new.
	self assert: uninstantiatedBlockMapper 
				= (Compiler evaluate: uninstantiatedBlockMapper asScript).
	scripted := Compiler evaluate: instantiatedBlockMapper asScript.
	self assert: scripted population isEmpty! !

!ColorLinearMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildColorScheme
	| mapper colorScheme1 colorScheme2 |
	mapper := ColorLinearMapper new.
	mapper population: #(-10 30 10).
	colorScheme1 := mapper buildColorScheme.
	self assert: colorScheme1 class = RGBColorScheme.
	self assert: colorScheme1 firstColor = ColorValue blue.
	self assert: colorScheme1 lastColor = ColorValue red.
	self assert: colorScheme1 resolution = 41.
	mapper spectrumwise: true.
	colorScheme2 := mapper buildColorScheme.
	self assert: colorScheme2 class = HSBColorScheme! !

!ColorLinearMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildConverter
	| mapper converter |
	mapper := ColorLinearMapper new.
	mapper population: #(-10 30 10).
	converter := mapper buildConverter.
	self assert: converter class = LinearConverter.
	self assert: converter minimumInput = -10.
	self assert: converter maximumInput = 30.
	self assert: converter minimumOutput = 1.
	self assert: converter maximumOutput = 41! !

!ColorLinearMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testComputeValueOn
	| mapper1 mapper2 |
	mapper1 := ColorLinearMapper new.
	mapper1 population: #(-10 30 10).
	self assert: (mapper1 computeValueOn: -10) = ColorValue blue.
	self assert: (mapper1 computeValueOn: 10) = ColorValue darkMagenta.
	self assert: (mapper1 computeValueOn: 30) = ColorValue red.
	mapper2 := ColorLinearMapper new.
	mapper2 population: #(-10 30 10); spectrumwise: true.
	self assert: (mapper2 computeValueOn: -10) = ColorValue blue.
	self assert: (mapper2 computeValueOn: 10) = ColorValue green.
	self assert: (mapper2 computeValueOn: 30) = ColorValue red! !

!ColorLinearMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testEquality
	| i1 i2 notI stillI1 |
	i1 := (ColorLinearMapper new)
				function: #numberOfMethods;
				startColor: ColorValue white;
				endColor: ColorValue black;
				spectrumwise: true.
	i2 := i1 copy.
	self assert: i1 = i2.
	#(#function: #startColor: #endColor: #spectrumwise:) do: 
			[:each | 
			notI := i1 copy perform: each with: nil.
			self deny: notI = i1].
	"population is not taken into account for the equality test"
	stillI1 := i1 copy perform: #population: with: #(1 2).
	self assert: stillI1 = i1! !

TestCase subclass: #ColorSchemeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!ColorSchemeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testScaleColorPropertyForIndex
	|colorScheme blue red darkMagenta first middle last|
	blue := ColorValue blue.
	red := ColorValue red.
	darkMagenta := ColorValue darkMagenta.
	colorScheme := ColorScheme from: blue to: red resolution: 5.
	first := ColorValue 
				red:  (colorScheme scaleColorProperty: #red forIndex: 1) 
				green: (colorScheme scaleColorProperty: #green forIndex: 1)
				blue: (colorScheme scaleColorProperty: #blue forIndex: 1).
	middle := ColorValue 
				red:  (colorScheme scaleColorProperty: #red forIndex: 3) 
				green: (colorScheme scaleColorProperty: #green forIndex: 3)
				blue: (colorScheme scaleColorProperty: #blue forIndex: 3).
	last := ColorValue 
				red:  (colorScheme scaleColorProperty: #red forIndex: 5) 
				green: (colorScheme scaleColorProperty: #green forIndex: 5)
				blue: (colorScheme scaleColorProperty: #blue forIndex: 5).
	self assert: first = blue.
	self assert: middle = darkMagenta.
	self assert: last = red! !

!ColorSchemeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testScaleColorPropertyForIndexWithResolution1
	
	| colorScheme outputColor |
	colorScheme := ColorScheme
		from: ColorValue blue
		to: ColorValue red
		resolution: 1.
	outputColor := ColorValue
		red: (colorScheme scaleColorProperty: #red forIndex: 1)
		green: (colorScheme scaleColorProperty: #green forIndex: 1)
		blue: (colorScheme scaleColorProperty: #blue forIndex: 1).
	self assert: outputColor = ColorValue darkMagenta! !

TestCase subclass: #ConstantMapperModelTest
	instanceVariableNames: 'initialMapper glyphProperty modelClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!ConstantMapperModelTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	glyphProperty := #color.
	modelClass := FAMIXMethod.
	initialMapper := ConstantMapper new 
		constantValue: ColorValue blue! !

!ConstantMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConstantValue
	| mapperModel |
	mapperModel := ConstantMapperModel newMapper: nil forModelClass: modelClass.
	self assert: mapperModel constantValue value = nil.
	mapperModel := ConstantMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel constantValue value = initialMapper constantValue! !

!ConstantMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCreation
	| mapperModel |
	mapperModel := ConstantMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel mapper = initialMapper! !

!ConstantMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInitializeConstantValue
	| mapperModel |
	mapperModel := ConstantMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel initializeConstantValue = initialMapper constantValue! !

!ConstantMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testOutputMapper
	| mapperModel newValue resultingMapperModel |
	mapperModel := ConstantMapperModel newMapper: initialMapper forModelClass: modelClass.
	newValue := ColorValue red.
	mapperModel constantValue value: newValue.
	resultingMapperModel := (initialMapper copy)
				constantValue: newValue.
	self assert: mapperModel outputMapper = resultingMapperModel! !

TestCase subclass: #ConstantMapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!ConstantMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAsScript
	| value instantiatedBlockMapper uninstantiatedBlockMapper |
	value := true.
	instantiatedBlockMapper := ConstantMapper new constantValue: value.
	self assert: instantiatedBlockMapper 
				= (Compiler evaluate: instantiatedBlockMapper asScript).
	uninstantiatedBlockMapper := ConstantMapper new.
	self assert: uninstantiatedBlockMapper 
				= (Compiler evaluate: uninstantiatedBlockMapper asScript)! !

!ConstantMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testComputeValueOn
	| model1 mapper value1 value2 model2 |
	mapper := ConstantMapper new constantValue: ColorValue red; yourself.
	model1 := #(1 2 3 4 5).
	value1 := mapper computeValueOn: model1.
	self assert: value1 = ColorValue red.
	model2 := nil.	
	value2 := mapper computeValueOn: model2.
	self assert: value2 = ColorValue red! !

!ConstantMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testEquality
	| i1 i2 notI value |
	value := ColorValue blue.
	i1 := ConstantMapper new
			constantValue:  value.
	i2 := i1 copy.
	self assert: i1 = i2.
	#(#constantValue:) do: 
			[:each | 
			notI := i1 copy perform: each with: nil.
			self deny: notI = i1]! !

TestCase subclass: #CuboidGlyphTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!CuboidGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildVolume
	| cuboidGlyph dimension position volume boundingBox epsilon epsilon3d |
	epsilon := 0.00000001.
	epsilon3d := epsilon , epsilon , epsilon.
	dimension := 3 , 4 , 5.
	position := 11 , 23 , 47.
	cuboidGlyph := CuboidGlyph new
				basicPosition: position;
				dimension: dimension;
				yourself.
	volume := cuboidGlyph buildVolume.
	self assert: volume notNil.
	self assert: (volume isKindOf: JunOpenGL3dObject).
	boundingBox := volume boundingBox.
	self assert: (boundingBox origin - position) abs < epsilon3d.
	self assert: (boundingBox corner - dimension - position) abs < epsilon3d! !

!CuboidGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInnerDimension
	| cuboidGlyph dimension requestedInnerDimension |
	dimension := 3 , 4 , 5.
	cuboidGlyph := CuboidGlyph new
				dimension: dimension;
				yourself.
	requestedInnerDimension := 4 , 7 , 9.
	self assert: cuboidGlyph dimension = dimension.
	"setter"
	cuboidGlyph innerDimension: requestedInnerDimension.
	self assert: cuboidGlyph dimension = requestedInnerDimension.
	"getter"
	self assert: cuboidGlyph innerDimension = requestedInnerDimension.! !

!CuboidGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInnerOrigin
	| cuboidGlyph position |
	position := 11 , 23 , 47.
	cuboidGlyph := CuboidGlyph new
				basicPosition: position;
				yourself.
	self assert: cuboidGlyph position = position.
	"getter"
	self assert: cuboidGlyph innerOrigin = position.
	"setter"
	cuboidGlyph basicInnerOrigin: position.
	self assert: cuboidGlyph innerOrigin = position! !

TestCase subclass: #HSBColorSchemeTest
	instanceVariableNames: 'epsilon'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!HSBColorSchemeTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	epsilon := 0.001! !

!HSBColorSchemeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testColorAtIndex
	|scheme color1 color2 color3 color4 color5 resolution expectedColors actualColors colorProperties|
	color1 := ColorValue hue: 0.17 saturation: 0.2 brightness: 1.
	color2 := (ColorValue hue: 0.2875 saturation: 0.3375 brightness: 0.895).
	color3 := (ColorValue hue: 0.405 saturation: 0.475 brightness: 0.79).
	color4 := (ColorValue hue: 0.5225 saturation: 0.6125 brightness: 0.685).
	color5 := ColorValue hue: 0.64 saturation: 0.75 brightness: 0.58.
	resolution := 5.
	expectedColors := OrderedCollection new add: color1; add: color2; add: color3; add: color4; add: color5; yourself.
	scheme := HSBColorScheme from: color1 to: color5 resolution: resolution.
	actualColors := (1 to: resolution) inject: OrderedCollection new into: [:colors :eachIndex| |color|
		color := scheme colorAtIndex: eachIndex.
		colors add: color; yourself
	].
	colorProperties := #(#hue #saturation #brightness). 
	1 to: resolution do: [:each| 
		colorProperties do: [:eachProperty| |actual expected|
			actual := (actualColors at: each) perform: eachProperty.
			expected := (expectedColors at: each) perform: eachProperty.
			self assert: (actual - expected) abs < epsilon
		]		
	].! !

!HSBColorSchemeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCreation
	|firstColor lastColor resolution scheme|
	firstColor := ColorValue red.
	lastColor := ColorValue blue.
	resolution := 7.
	scheme := ColorScheme from: firstColor to: lastColor resolution: resolution.
	self assert: scheme firstColor = firstColor.
	self assert: scheme lastColor = lastColor.
	self assert: scheme resolution = resolution! !

!HSBColorSchemeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testScaleColorPropertyForIndex
	|scheme firstColor lastColor resolution |
	firstColor := ColorValue hue: 0.17 saturation: 0.2 brightness: 1.
	lastColor := ColorValue hue: 0.64 saturation: 0.75 brightness: 0.58.
	resolution := 5.
	scheme := ColorScheme from: firstColor to: lastColor resolution: resolution.
	"ascending property"
	self assert: ((scheme scaleColorProperty: #hue forIndex: 1) - 0.17) abs < epsilon.
	self assert: ((scheme scaleColorProperty: #hue forIndex: 2) - 0.2875) abs < epsilon.
	self assert: ((scheme scaleColorProperty: #hue forIndex: 3) - 0.405) abs < epsilon.
	self assert: ((scheme scaleColorProperty: #hue forIndex: 4) - 0.5225) abs < epsilon.
	self assert: ((scheme scaleColorProperty: #hue forIndex: 5) - 0.64) abs < epsilon.
	"descending property"
	self assert: ((scheme scaleColorProperty: #brightness forIndex: 1) - 1) abs < epsilon.
	self assert: ((scheme scaleColorProperty: #brightness forIndex: 2) - 0.895) abs < epsilon.
	self assert: ((scheme scaleColorProperty: #brightness forIndex: 3) - 0.79) abs < epsilon.
	self assert: ((scheme scaleColorProperty: #brightness forIndex: 4) - 0.685) abs < epsilon.
	self assert: ((scheme scaleColorProperty: #brightness forIndex: 5) - 0.58) abs < epsilon.! !

TestCase subclass: #HistoryInfrastructureBuilderTest
	instanceVariableNames: 'builder oldConfiguration history'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!HistoryInfrastructureBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
defaultViewConfiguration
	
	| config layout |
	layout := RectanglePackingLayout new horizontalMargin: 1; verticalMargin: 0.
	config := ConfigurationRepository defaultConfiguration.
	config
		addMapper: (ConstantMapper new constantValue: 1)
		onGlyphProperty: #terrainThickness
		forModelClass: MooseModel.
	config setLayout: layout forModelClass: FAMIXPackage.
	config
		addMapper: (ConstantMapper new constantValue: 1)
		onGlyphProperty: #terrainThickness
		forModelClass: FAMIXPackage.
	config setLayout: layout forModelClass: MooseModel.
	^config! !

!HistoryInfrastructureBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
history
	| v1 v2 v3 v4 aHhistory |
	aHhistory := ModelHistory new.
	v1 := (Importer withActiveMetamodel importArray: self version1) 
				name: #Version1.
	v2 := (Importer withActiveMetamodel importArray: self version2) 
				name: #Version2.
	v3 := (Importer withActiveMetamodel importArray: self version3) 
				name: #Version3.
	v4 := (Importer withActiveMetamodel importArray: self version4) 
				name: #Version4.
	aHhistory addLastVersionForMooseModel: v1.
	aHhistory addLastVersionForMooseModel: v2.
	aHhistory addLastVersionForMooseModel: v3.
	aHhistory addLastVersionForMooseModel: v4.
	aHhistory name: #History.
	^aHhistory! !

!HistoryInfrastructureBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	
	oldConfiguration := Preferences uniqueInstance viewConfiguration.
	Preferences uniqueInstance viewConfiguration: self defaultViewConfiguration.
	history := self history.
	builder := HistoryInfrastructureBuilder withModelHistory: history! !

!HistoryInfrastructureBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
tearDown
	Preferences uniqueInstance viewConfiguration: oldConfiguration! !

!HistoryInfrastructureBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildEvolutionaryGlyphDictionary
	| dictionary package1 package2 package3 classA classB classC classD |
	dictionary := builder buildEvolutionaryGlyphDictionary.
	package1 := history allPackageHistories detect: [:each | each name = #package1].
	package2 := history allPackageHistories detect: [:each | each name = #package2].
	package3 := history allPackageHistories detect: [:each | each name = #package3].
	classA := history allClassHistories detect: [:each | each name = #ClassA].
	classB := history allClassHistories detect: [:each | each name = #ClassB].
	classC := history allClassHistories detect: [:each | each name = #ClassC].
	classD := history allClassHistories detect: [:each | each name = #ClassD].
	self assert: (dictionary at: package1) position = (1,1,1).
	self assert: (dictionary at: package1) dimension = (12,8,4).
	self assert: (dictionary at: package2) position = (1,10,1).
	self assert: (dictionary at: package2) dimension = (6,6,6).
	self assert: (dictionary at: package3) position = (2,11,2).
	self assert: (dictionary at: package3) dimension = (4,4,5).
	self assert: (dictionary at: classA) position = (2,2,2).
	self assert: (dictionary at: classA) dimension = (6,6,1).
	self assert: (dictionary at: classB) position = (9,2,2).
	self assert: (dictionary at: classB) dimension = (3,3,2).
	self assert: (dictionary at: classC) position = (9,6,2).
	self assert: (dictionary at: classC) dimension = (2,2,3).
	self assert: (dictionary at: classD) position = (3,12,3).
	self assert: (dictionary at: classD) dimension = (2,2,4)! !

!HistoryInfrastructureBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
version1
	^
#(Moose.Model (entity
	(FAMIX.Namespace (id: 1) (name aNamespace))
	(FAMIX.Package (id: 201) (name package1))
	  (FAMIX.Class (id: 2002) (name ClassB) (belongsTo (idref: 1)) (packagedIn (idref: 201)))
	    (FAMIX.Attribute (name a1B) (belongsTo (idref: 2002)))
	    (FAMIX.Method (name m1B) (signature 'm1B()') (belongsTo (idref: 2002)) (LOC 2))
	  (FAMIX.Class (id: 2003) (name ClassC) (belongsTo (idref: 1)) (packagedIn (idref: 201)))
	    (FAMIX.Attribute (name a1C) (belongsTo (idref: 2003)))
	    (FAMIX.Attribute (name a2C) (belongsTo (idref: 2003)))
	    (FAMIX.Method (name m3C) (signature 'm3C()') (belongsTo (idref: 2003)) (LOC 6))
))! !

!HistoryInfrastructureBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
version2
	^
#(Moose.Model (entity
	(FAMIX.Namespace (id: 1) (name aNamespace))
	(FAMIX.Package (id: 201) (name package1))
	  (FAMIX.Class (id: 2001) (name ClassA) (belongsTo (idref: 1)) (packagedIn (idref: 201)))
	    (FAMIX.Attribute (name a1A) (belongsTo (idref: 2001)))
	    (FAMIX.Method (name m1A) (signature 'm1A()') (belongsTo (idref: 2001)) (LOC 1))
	  (FAMIX.Class (id: 2002) (name ClassB) (belongsTo (idref: 1)) (packagedIn (idref: 201)))
	    (FAMIX.Attribute (name a1B) (belongsTo (idref: 2002)))
	    (FAMIX.Attribute (name a2B) (belongsTo (idref: 2002)))
	    (FAMIX.Attribute (name a3B) (belongsTo (idref: 2002)))
	    (FAMIX.Method (name m1B) (signature 'm1B()') (belongsTo (idref: 2002)) (LOC 2))
	    (FAMIX.Method (name m2B) (signature 'm2B()') (belongsTo (idref: 2002)) (LOC 3))
	  (FAMIX.Class (id: 2003) (name ClassC) (belongsTo (idref: 1)) (packagedIn (idref: 201)))
	    (FAMIX.Attribute (name a1C) (belongsTo (idref: 2003)))
	    (FAMIX.Attribute (name a2C) (belongsTo (idref: 2003)))
	    (FAMIX.Method (name m1C) (signature 'm1C()') (belongsTo (idref: 2003)) (LOC 4))
	    (FAMIX.Method (name m2C) (signature 'm2C()') (belongsTo (idref: 2003)) (LOC 5))
))! !

!HistoryInfrastructureBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
version3
	^
#(Moose.Model (entity
	(FAMIX.Namespace (id: 1) (name aNamespace))
	(FAMIX.Package (id: 201) (name package1))
	  (FAMIX.Class (id: 2001) (name ClassA) (belongsTo (idref: 1)) (packagedIn (idref: 201)))
	    (FAMIX.Attribute (name a1A) (belongsTo (idref: 2001)))
	    (FAMIX.Attribute (name a2A) (belongsTo (idref: 2001)))
	    (FAMIX.Attribute (name a3A) (belongsTo (idref: 2001)))
	    (FAMIX.Attribute (name a4A) (belongsTo (idref: 2001)))
	    (FAMIX.Attribute (name a5A) (belongsTo (idref: 2001)))
	    (FAMIX.Attribute (name a6A) (belongsTo (idref: 2001)))
	    (FAMIX.Method (name m1A) (signature 'm1A()') (belongsTo (idref: 2001)) (LOC 1))
	  (FAMIX.Class (id: 2002) (name ClassB) (belongsTo (idref: 1)) (packagedIn (idref: 201)))
	    (FAMIX.Attribute (name a1B) (belongsTo (idref: 2002)))
	    (FAMIX.Attribute (name a2B) (belongsTo (idref: 2002)))
	    (FAMIX.Attribute (name a3B) (belongsTo (idref: 2002)))
	    (FAMIX.Method (name m1B) (signature 'm1B()') (belongsTo (idref: 2002)) (LOC 2))
	    (FAMIX.Method (name m2B) (signature 'm2B()') (belongsTo (idref: 2002)) (LOC 3))
	  (FAMIX.Class (id: 2003) (name ClassC) (belongsTo (idref: 1)) (packagedIn (idref: 201)))
	    (FAMIX.Attribute (name a1C) (belongsTo (idref: 2003)))
	    (FAMIX.Attribute (name a2C) (belongsTo (idref: 2003)))
	    (FAMIX.Method (name m1C) (signature 'm1C()') (belongsTo (idref: 2003)) (LOC 4))
	    (FAMIX.Method (name m2C) (signature 'm2C()') (belongsTo (idref: 2003)) (LOC 5))
	    (FAMIX.Method (name m3C) (signature 'm3C()') (belongsTo (idref: 2003)) (LOC 6))
	(FAMIX.Package (id: 202) (name package2))
	(FAMIX.Package (id: 203) (name package3) (packagedIn (idref: 202)))
	  (FAMIX.Class (id: 2004) (name ClassD) (belongsTo (idref: 1)) (packagedIn (idref: 203)))
	    (FAMIX.Attribute (name a1D) (belongsTo (idref: 2004)))
	    (FAMIX.Attribute (name a2D) (belongsTo (idref: 2004)))
	    (FAMIX.Method (name m1D) (signature 'm1D()') (belongsTo (idref: 2004)) (LOC 7))
	    (FAMIX.Method (name m2D) (signature 'm2D()') (belongsTo (idref: 2004)) (LOC 8))
	    (FAMIX.Method (name m3D) (signature 'm3D()') (belongsTo (idref: 2004)) (LOC 9))
	    (FAMIX.Method (name m4D) (signature 'm4D()') (belongsTo (idref: 2004)) (LOC 10))

))! !

!HistoryInfrastructureBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
version4
	^
#(Moose.Model (entity
	(FAMIX.Namespace (id: 1) (name aNamespace))
	(FAMIX.Package (id: 202) (name package2))
	(FAMIX.Package (id: 203) (name package3) (packagedIn (idref: 202)))
	  (FAMIX.Class (id: 2004) (name ClassD) (belongsTo (idref: 1)) (packagedIn (idref: 203)))
	    (FAMIX.Attribute (name a1D) (belongsTo (idref: 2004)))
	    (FAMIX.Method (name m1D) (signature 'm1D()') (belongsTo (idref: 2004)) (LOC 7))
	    (FAMIX.Method (name m3D) (signature 'm3D()') (belongsTo (idref: 2004)) (LOC 9))
))! !

TestCase subclass: #IdentityMapperModelTest
	instanceVariableNames: 'initialMapper glyphProperty modelClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!IdentityMapperModelTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	glyphProperty := #height.
	modelClass := FAMIXMethod.
	initialMapper := IdentityMapper new
		function: #numberOfLinesOfCode! !

!IdentityMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCreation
	| mapperModel |
	mapperModel := IdentityMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel mapper = initialMapper! !

!IdentityMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInitializeModelProperty
	| mapperModel |
	mapperModel := IdentityMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel initializeModelProperty = initialMapper function! !

!IdentityMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testModelProperty
	| mapperModel |
	mapperModel := IdentityMapperModel newMapper: nil forModelClass: modelClass.
	self assert: mapperModel modelProperty value isNil.
	mapperModel := IdentityMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel modelProperty value = initialMapper function! !

!IdentityMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testModelPropertyList
	| mapperModel |
	mapperModel := IdentityMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel modelPropertyList size = modelClass new allDeclaredProperties size! !

!IdentityMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testOutputMapper
	| mapperModel newFunction resultingMapperModel |
	mapperModel := IdentityMapperModel newMapper: initialMapper forModelClass: modelClass.
	newFunction := #numberOfInvocations.
	mapperModel modelProperty value: newFunction.
	resultingMapperModel := (initialMapper copy)
				function: newFunction;
				yourself.
	self assert: mapperModel outputMapper = resultingMapperModel! !

TestCase subclass: #IdentityMapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!IdentityMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAsScript
	| aFunction instantiatedBlockMapper uninstantiatedBlockMapper |
	aFunction := #size.
	instantiatedBlockMapper := IdentityMapper new function: aFunction.
	self assert: instantiatedBlockMapper 
				= (Compiler evaluate: instantiatedBlockMapper asScript).
	uninstantiatedBlockMapper := IdentityMapper new.
	self assert: uninstantiatedBlockMapper 
				= (Compiler evaluate: uninstantiatedBlockMapper asScript)! !

!IdentityMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testComputeValueOn
	| mapper model1 value1 |
	mapper := IdentityMapper new function: #size; yourself.
	model1 := #(1 2 3 4 5).
	value1 := mapper computeValueOn: model1.
	self assert: value1 = 5! !

!IdentityMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testEquality
	| i1 i2 notI aFunction |
	aFunction := #size.
	i1 := IdentityMapper new
			function:  aFunction.
	i2 := i1 copy.
	self assert: i1 = i2.
	#(#function:) do: 
			[:each | 
			notI := i1 copy perform: each with: nil.
			self deny: notI = i1]! !

TestCase subclass: #LinearConverterTest
	instanceVariableNames: 'epsilon'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!LinearConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConvert
	| converter |
	converter := LinearConverter newInputFrom: 2 to: 5 outputFrom: -25 to: 125.
	self assert: (converter convert: 2) = -25.
	self assert: (converter convert: 2.5) = 0.
	self assert: (converter convert: 3) = 25.
	self assert: (converter convert: 4) = 75.
	self assert: (converter convert: 5) = 125.! !

!LinearConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConvertWithVoidRange
	| converter |
	converter := (LinearConverter new)
				minimumInput: 1;
				maximumInput: 1;
				minimumOutput: 2;
				maximumOutput: 6.
	self assert: (converter convert: 1) = 4! !

TestCase subclass: #LinearMapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!LinearMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInputRangeResolution
	| mapper |
	mapper := LinearMapper new.
	mapper population: #(1 -8 100 3 -4 12).
	self assert: mapper inputRangeResolution = 109! !

TestCase subclass: #ModelCompleterTest
	instanceVariableNames: 'mooseModel buggyMooseModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!ModelCompleterTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
buggyModelSpec
	^
#(Moose.Model (entity
	(FAMIX.Namespace (id: 1) (name 'org::argouml::uml::notation::uml'))
	(FAMIX.Class (id: 2) (name MockClass) (belongsTo (idref: 1)))
))! !

!ModelCompleterTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
modelSpec
	^
#(Moose.Model (entity
	(FAMIX.Namespace (id: 1) (name 'package1::package11::package111'))
	(FAMIX.Namespace (id: 6) (name 'package1::package11::package112::package1121'))

	(FAMIX.Class (id: 2) (name TestClass1) (belongsTo (idref: 6)))
	(FAMIX.Method (name method1TC1) (signature 'method1TC1()') (belongsTo (idref: 2)) (LOC 2))

	(FAMIX.Class (id: 3) (name TestClass3) (belongsTo (idref: 6)))
	(FAMIX.Method (name method1TC3) (signature 'method1TC3()') (belongsTo (idref: 3)) (LOC 2))
	(FAMIX.Method (name method2TC3) (signature 'method2TC3()') (belongsTo (idref: 3)) (LOC 2))
	(FAMIX.Method (name method3TC3) (signature 'method3TC3()') (belongsTo (idref: 3)) (LOC 2))

	(FAMIX.Class (id: 4) (name TestClass5) (belongsTo (idref: 6)))
	(FAMIX.Method (name method1TC5) (signature 'method1TC5()') (belongsTo (idref: 4)) (LOC 2))
	(FAMIX.Method (name method2TC5) (signature 'method2TC5()') (belongsTo (idref: 4)) (LOC 2))
	(FAMIX.Method (name method3TC5) (signature 'method3TC5()') (belongsTo (idref: 4)) (LOC 2))
	(FAMIX.Method (name method4TC5) (signature 'method4TC5()') (belongsTo (idref: 4)) (LOC 2))
	(FAMIX.Method (name method5TC5) (signature 'method5TC5()') (belongsTo (idref: 4)) (LOC 2))

	(FAMIX.Class (id: 5) (name TestClass7) (belongsTo (idref: 1)))
	(FAMIX.Method (name method1TC7) (signature 'method1TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method2TC7) (signature 'method2TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method3TC7) (signature 'method3TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method4TC7) (signature 'method4TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method5TC7) (signature 'method5TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method6TC7) (signature 'method6TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method7TC7) (signature 'method7TC7()') (belongsTo (idref: 5)) (LOC 2))

))! !

!ModelCompleterTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	mooseModel := (Importer withActiveMetamodel 
				importArray: self modelSpec) name: #TestModel.
	buggyMooseModel :=  (Importer withActiveMetamodel
				importArray: self buggyModelSpec) name: #BuggyTestModel! !

!ModelCompleterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCreatePackagesFromNamespaces
	| completer createdPackages expectedPackageNames actualPackageNames expectedClasses expectedClassNames |
	completer := ModelCompleter forMooseModel: mooseModel.
	completer createPackagesFromNamespaces.
	createdPackages := mooseModel allPackages.
	expectedPackageNames := #(#package1 
							#'package1::package11' 
							#'package1::package11::package111' 
							#'package1::package11::package112'
							#'package1::package11::package112::package1121').	
	actualPackageNames := createdPackages collect: [:each|each name].
	self assert: (expectedPackageNames equalsTo: actualPackageNames).
	self assert: (createdPackages select: [:each|each isNil]) isEmpty.
	self assert: (createdPackages at: 5) packagedIn == (createdPackages at: 4).
	self assert: (createdPackages at: 4) packagedIn == (createdPackages at: 2).
	self assert: (createdPackages at: 3) packagedIn == (createdPackages at: 2).
	self assert: (createdPackages at: 2) packagedIn == (createdPackages at: 1).
	self assert: (createdPackages at: 1) packagedIn isNil.
	#(1 2 4) do: [:eachIndex| self assert: (createdPackages at: eachIndex) definedClasses isEmpty].
	self assert: (createdPackages at: 3) definedClasses size = 1.
	self assert: (createdPackages at: 5) definedClasses size = 3.

	expectedClasses := mooseModel allClasses.
	expectedClassNames := #(#TestClass1 #TestClass3 #TestClass5 #TestClass7). 
	self assert: ((expectedClasses collect: [:each|each name]) equalsTo: expectedClassNames).
	self assert: (expectedClasses at: 1) packagedIn == (createdPackages at: 5).
	self assert: (expectedClasses at: 2) packagedIn == (createdPackages at: 5).
	self assert: (expectedClasses at: 3) packagedIn == (createdPackages at: 5).
	self assert: (expectedClasses at: 4) packagedIn == (createdPackages at: 3).! !

!ModelCompleterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCreatePackagesFromNamespacesOnBugCase
	| completer createdPackages expectedPackageNames actualPackageNames |
	completer := ModelCompleter forMooseModel: buggyMooseModel.
	completer createPackagesFromNamespaces.
	createdPackages := buggyMooseModel allPackages.
	expectedPackageNames := #(
		#'org' 						
		#'org::argouml' 						
		#'org::argouml::uml' 						
		#'org::argouml::uml::notation' 						
		#'org::argouml::uml::notation::uml' 
	).
	actualPackageNames := createdPackages collect: [:each|each name].
	self assert: (expectedPackageNames equalsTo: actualPackageNames).
	self assert: buggyMooseModel allClasses size = 1.
	self assert: buggyMooseModel allClasses any belongsTo name = (expectedPackageNames at: 5).! !

!ModelCompleterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testTokenizeBySeparator
	|completer computedTokens namespaces separator |
	separator := '::'.
	completer := ModelCompleter forMooseModel: mooseModel.
	namespaces := mooseModel allNamespaces.
	self assert: namespaces size = 2.
	self assert: mooseModel allPackages isEmpty.

	computedTokens := completer tokenize: namespaces last name bySeparator: separator.
	self assert: computedTokens size = 4.
	self assert: (computedTokens at: 1) = 'package1'.
	self assert: (computedTokens at: 2) = 'package11'.
	self assert: (computedTokens at: 3) = 'package112'.
	self assert: (computedTokens at: 4) = 'package1121'! !

!ModelCompleterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testTokenizeBySeparatorOnBugCase
	|completer computedTokens namespaces separator |
	separator := '::'.
	completer := ModelCompleter forMooseModel: buggyMooseModel.
	namespaces := buggyMooseModel allNamespaces.
	self assert: namespaces size = 1.
	self assert: buggyMooseModel allPackages isEmpty.

	computedTokens := completer tokenize: namespaces last name bySeparator: separator.
	self assert: computedTokens size = 5.
	self assert: (computedTokens at: 1) = 'org'.
	self assert: (computedTokens at: 2) = 'argouml'.
	self assert: (computedTokens at: 3) = 'uml'.
	self assert: (computedTokens at: 4) = 'notation'.
	self assert: (computedTokens at: 5) = 'uml'! !

TestCase subclass: #ModelElementSpecificationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!ModelElementSpecificationTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAddMapperOnGlyphProperty
	| spec mapper1v1 mapper1v2 mapper2|
	spec := (ModelElementSpecification new)
				visible: true;
				glyphClass: CuboidGlyph;
				layout: SpiralLayout new.
	self assert: spec mapperDictionary isEmpty.
	mapper1v1 := ConstantMapper new constantValue: 13.
	spec addMapper: mapper1v1 onGlyphProperty: #height.
	self assert: spec mapperDictionary size = 1.
	self assert: (spec mapperOnGlyphProperty: #height) = mapper1v1.
	mapper1v2 := IdentityMapper new function: #numberOfMethods.
	spec addMapper: mapper1v2 onGlyphProperty: #height.
	self assert: spec mapperDictionary size = 1.
	self assert: (spec mapperOnGlyphProperty: #height) = mapper1v2.
	mapper2 := BlockMapper new block: [:each | each printString].
	spec addMapper: mapper2 onGlyphProperty: #alpha.
	self assert: spec mapperDictionary size = 2.
	self assert: (spec mapperOnGlyphProperty: #alpha) = mapper2.
	self assert: (spec mapperOnGlyphProperty: #height) = mapper1v2.
	self assert: (spec mapperOnGlyphProperty: #color) isNil! !

!ModelElementSpecificationTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAsScript
	| scriptedSpec sourceSpec |
	sourceSpec := (ModelElementSpecification new)
				visible: true;
				glyphClass: CuboidGlyph;
				layout: SpiralLayout new;
				addMapper: (ConstantMapper new 
								constantValue: 13) 
					onGlyphProperty: #height;
				addMapper: (ThresholdMapper new
								function: #depthInHierarchy;
								codomain: #(0.1 0.4 0.8 1.0);
								thresholds: #(2 4 6))
					onGlyphProperty: #alpha.
	scriptedSpec := Compiler evaluate: sourceSpec asScript.
	self assert: sourceSpec = scriptedSpec! !

TestCase subclass: #MooseModelTest
	instanceVariableNames: 'mooseModel buggyMooseModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!MooseModelTest commentStamp: 'stephane.ducasse 6/13/2008 13:41' prior: 0!
Just a test about mooseModel root and so on.!

!MooseModelTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
buggyModelSpec
	^
#(Moose.Model (entity
	(FAMIX.Namespace (id: 1) (name 'org::argouml::uml::notation::uml'))
	(FAMIX.Class (id: 2) (name MockClass) (belongsTo (idref: 1)))
))! !

!MooseModelTest methodsFor: 'util' stamp: ''!
model
	" to debug and avoid caching behavior: LANPackageTestResource reset."
	^ LANPackageTestResource current model ! !

!MooseModelTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
modelSpec
	^
#(Moose.Model (entity
	(FAMIX.Namespace (id: 1) (name 'package1::package11::package111'))
	(FAMIX.Namespace (id: 6) (name 'package1::package11::package112::package1121'))

	(FAMIX.Class (id: 2) (name TestClass1) (belongsTo (idref: 6)))
	(FAMIX.Method (name method1TC1) (signature 'method1TC1()') (belongsTo (idref: 2)) (LOC 2))

	(FAMIX.Class (id: 3) (name TestClass3) (belongsTo (idref: 6)))
	(FAMIX.Method (name method1TC3) (signature 'method1TC3()') (belongsTo (idref: 3)) (LOC 2))
	(FAMIX.Method (name method2TC3) (signature 'method2TC3()') (belongsTo (idref: 3)) (LOC 2))
	(FAMIX.Method (name method3TC3) (signature 'method3TC3()') (belongsTo (idref: 3)) (LOC 2))

	(FAMIX.Class (id: 4) (name TestClass5) (belongsTo (idref: 6)))
	(FAMIX.Method (name method1TC5) (signature 'method1TC5()') (belongsTo (idref: 4)) (LOC 2))
	(FAMIX.Method (name method2TC5) (signature 'method2TC5()') (belongsTo (idref: 4)) (LOC 2))
	(FAMIX.Method (name method3TC5) (signature 'method3TC5()') (belongsTo (idref: 4)) (LOC 2))
	(FAMIX.Method (name method4TC5) (signature 'method4TC5()') (belongsTo (idref: 4)) (LOC 2))
	(FAMIX.Method (name method5TC5) (signature 'method5TC5()') (belongsTo (idref: 4)) (LOC 2))

	(FAMIX.Class (id: 5) (name TestClass7) (belongsTo (idref: 1)))
	(FAMIX.Method (name method1TC7) (signature 'method1TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method2TC7) (signature 'method2TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method3TC7) (signature 'method3TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method4TC7) (signature 'method4TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method5TC7) (signature 'method5TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method6TC7) (signature 'method6TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method7TC7) (signature 'method7TC7()') (belongsTo (idref: 5)) (LOC 2))

))! !

!MooseModelTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	mooseModel := (Importer withActiveMetamodel 
				importArray: self modelSpec) name: #TestModel.
	buggyMooseModel :=  (Importer withActiveMetamodel
				importArray: self buggyModelSpec) name: #BuggyTestModel! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testAdd
	| entity |
	entity := MooseEntity new.
	model add: entity.
	self assert: model entities size = 1.
	self assert: entity mooseModel == model! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testAddAnnouncement
	| entity announcedEntity |
	entity := MooseEntity new.
	announcedEntity := nil.
	model announcer 
		when: MooseEntityAdded do: [:a | announcedEntity := a entity ].

	model add: entity.
	self assert: announcedEntity == entity.! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testAddingAndRemoving
	| nbOfInstancesOriginal newModel nbOfInstancesBefore nbOfInstancesAfter |
	10 timesRepeat: [Smalltalk garbageCollect].
	Object new.
	
	self assert: (MooseModel root ~= nil).
	nbOfInstancesOriginal := MooseModel allInstances size.
	newModel := MooseModel new.
	newModel install.

	self assert: (MooseModel root includes: newModel).
	
	nbOfInstancesBefore := MooseModel allInstances size.
	self assert: (nbOfInstancesBefore = (nbOfInstancesOriginal + 1)).
	
	newModel removeFromModel.
	newModel := nil.
	10 timesRepeat: [Smalltalk garbageCollect].
	Object new.
	
	nbOfInstancesAfter := MooseModel allInstances size.
	self assert: nbOfInstancesAfter equals: nbOfInstancesOriginal! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testAllBookmarks
	| entity1 entity2 entity3 |
	entity1 := MooseEntity new.
	entity2 := MooseEntity new.
	entity3 := MooseEntity new.
	model add: entity1.
	model add: entity2.
	model add: entity3.
	
	entity1 bookmarkAs: 'one'.
	entity2 bookmarkAs: 'two'.

	self assert: model allBookmarks size = 2! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testAsMSEString

	self assert: MooseModel new asMSEString = '()'.
	
	self assert: (MooseModel new add: MooseEntity new; asMSEString) =  '(
	(Moose.Entity (id: 1)))'! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testBookmarkAsRaisesAnnouncement
	| entity1 announcedEntity |
	entity1 := MooseEntity new.
	model add: entity1.
	announcedEntity := nil.
	model announcer 
		when: MooseEntityAdded do: [:a | announcedEntity := a entity ].
	
	entity1 bookmarkAs: 'one'.
	self assert: announcedEntity notNil.! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testCacheInvalidationAfterAdd
	| entity |
	self assert: model allClasses isEmpty. 
	model add: FAMIXClass new.
	self assert: model allClasses size = 1.! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testCacheInvalidationAfterMultipleAddRemove
	| classA classB |
	classA := FAMIXClass new name: 'ClassA'.
	classB := FAMIXClass new name: 'ClassB'.
	model add: classA.
	self assert: model allClasses size = 1.
	model add: classB.
	self assert: model allClasses size = 2.
	model remove: classA.
	self assert: model allClasses size = 1.
	model add: classA.
	self assert: model allClasses size = 2.
	model remove: classA.
	self assert: model allClasses size = 1.
	model remove: classB.
	self assert: model allClasses isEmpty
! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testCacheInvalidationAfterRemove
	| class |
	class := FAMIXClass new name: 'AClass'.
	model add: class.
	self assert: model allClasses size = 1.
	model remove: class.
	self assert: model allClasses isEmpty! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testCollect

	| t group el1 el2 el3 v |
	t := OrderedCollection new.
	group := MooseModel new.
	group add: (el1 := MooseEntity new).
	group add: (el2 := MooseEntity new).
	group add: (el3 := MooseEntity new).

	self assert: (group collect: #name) asArray =  #(#noname #noname #noname)! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testDelete

	"Raise an error since it is not in the root"
	self should: [[MooseModel new delete] valueSupplyingAnswer: true ] raise: Error.
	
	model := MooseModel new.
	self deny: (MooseModel root includes: model).
	MooseModel root add: model.
	self assert: (MooseModel root includes: model).

	[model delete] valueSupplyingAnswer: {'Are you sure to want to delete ', model mooseName printString, '?' . true}.
	self deny: (MooseModel root includes: model).
! !

!MooseModelTest methodsFor: 'tests' stamp: 'TudorGirba 3/21/2012 21:37'!
testDifference

	| t model1 model2 model3 el1 el2 el3 el4 |
	t := OrderedCollection new.
	model1 := MooseModel new.
	model1 add: (el1 := MooseEntity new).
	model1 add: (el2 := MooseEntity new).
	model1 add: (el3 := MooseEntity new).

	model2 := MooseModel new.
	model2 add: (el4 := MooseEntity new).
								
	self assert: (model1 difference: model2) entities asSet = model1 entities asSet.
	self assert: (model1 difference: MooseModel new) entities asSet = model1 entities asSet.
	self assert: (model1 difference: model1) entities isEmpty.
	
	model2 add: el1.
	self assert: (model1 difference: model2) entities size = 2.
	self assert: ((model1 difference: model2) entities includesAll: {el2 . el3}).  

	model1 add: el1.
	self assert: (model1 difference: model2) entities size = 2.
	self assert: ((model1 difference: model2) entities includesAll: {el2 . el3}).  
! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testDoSeparatedBy

	| t group el1 el2 el3 v |
	t := OrderedCollection new.
	group := MooseModel new.
	group add: (el1 := MooseEntity new).
	group add: (el2 := MooseEntity new).
	group add: (el3 := MooseEntity new).

	group do: [:x | t add: x] separatedBy:  [t add: 10].
	self assert: t asArray = {el1 . 10 . el2 . 10 . el3}! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testEnsureClassesAndNamespaces
	| aMethod |

	model add: (aMethod := FAMIXMethod new).
	self assert: (aMethod parentType isNil).
	self should: [(aMethod parentScope isNil)] raise: Error.


	model ensureClassesAndNamespaces.
	self deny: (aMethod parentType isNil).
	self deny: (aMethod parentScope isNil).

	self assert: (aMethod parentType == model unknownFAMIXClass).
	self assert: (aMethod parentScope == model unknownFAMIXNamespace).
! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testExport

	| stream |
	stream := WriteStream on: String new.
	MooseModel export: model to: stream.
	self assert: stream contents = '()'.
	
	model add: (FAMIXClass new name: 'Foo').
	stream := WriteStream on: String new.
	MooseModel export: model to: stream.
	self assert: stream contents =  '(
	(FAMIX.Class (id: 1)
		(name ''Foo'')))'! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testExportMetamodelTo

	| stream |
	stream := WriteStream on: String new.
	self shouldnt: [MooseModel exportMetamodelTo: stream]  raise: Error.
	self deny: stream isEmpty! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testIncludeElementOfType
	self assert: (self model includesElementsOfType: FAMIXClass).
	self deny: (self model includesElementsOfType: SmallInteger)! !

!MooseModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInferPackageHierarchyBasedOnNamespaces
	
	| createdPackages expectedPackageNames actualPackageNames expectedClasses expectedClassNames |
	mooseModel inferPackageHierarchyBasedOnNamespaces.
	createdPackages := mooseModel allPackages.
	expectedPackageNames := #(#package1 
							#'package1::package11' 
							#'package1::package11::package111' 
							#'package1::package11::package112'
							#'package1::package11::package112::package1121').	
	actualPackageNames := createdPackages collect: [:each|each name].
	self assert: (expectedPackageNames equalsTo: actualPackageNames).
	self assert: (createdPackages select: [:each|each isNil]) isEmpty.
	self assert: (createdPackages at: 5) packagedIn == (createdPackages at: 4).
	self assert: (createdPackages at: 4) packagedIn == (createdPackages at: 2).
	self assert: (createdPackages at: 3) packagedIn == (createdPackages at: 2).
	self assert: (createdPackages at: 2) packagedIn == (createdPackages at: 1).
	self assert: (createdPackages at: 1) packagedIn isNil.
	#(1 2 4) do: [:eachIndex| self assert: (createdPackages at: eachIndex) definedClasses isEmpty].
	self assert: (createdPackages at: 3) definedClasses size = 1.
	self assert: (createdPackages at: 5) definedClasses size = 3.

	expectedClasses := mooseModel allClasses.
	expectedClassNames := #(#TestClass1 #TestClass3 #TestClass5 #TestClass7). 
	self assert: ((expectedClasses collect: [:each|each name]) equalsTo: expectedClassNames).
	self assert: (expectedClasses at: 1) packagedIn == (createdPackages at: 5).
	self assert: (expectedClasses at: 2) packagedIn == (createdPackages at: 5).
	self assert: (expectedClasses at: 3) packagedIn == (createdPackages at: 5).
	self assert: (expectedClasses at: 4) packagedIn == (createdPackages at: 3).! !

!MooseModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInferPackageHierarchyBasedOnNamespacesOnBugCase
	| createdPackages expectedPackageNames actualPackageNames |
	buggyMooseModel inferPackageHierarchyBasedOnNamespaces.
	createdPackages := buggyMooseModel allPackages.
	expectedPackageNames := #(
		#'org' 						
		#'org::argouml' 						
		#'org::argouml::uml' 						
		#'org::argouml::uml::notation' 						
		#'org::argouml::uml::notation::uml' 
	).
	actualPackageNames := createdPackages collect: [:each|each name].
	self assert: (expectedPackageNames equalsTo: actualPackageNames).
	self assert: buggyMooseModel allClasses size = 1.
	self assert: buggyMooseModel allClasses any belongsTo name = (expectedPackageNames at: 5).! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testIntersect

	| model1 model2 model3 el1 el2 el3 el4 |
	model1 := MooseModel new.
	model1 add: (el1 := MooseEntity new).
	model1 add: (el2 := MooseEntity new).
	model1 add: (el3 := MooseEntity new).

	model2 := MooseModel new.
	model2 add: (el4 := MooseEntity new).
	model2 add: el1.
								
	self assert: (model1 intersect: MooseModel new) entities isEmpty.
	self assert: (model1 intersect: model2) entities asArray = {el1}.
	self assert: (MooseModel new intersect: MooseModel new) entities isEmpty.
! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testMenuCommandOn
	| builder |
	builder := PragmaMenuBuilder new.
	self assert: builder itemList isNil.
	MooseModel menuCommandOn: builder.
	self assert: builder itemList isCollection.
	self assert: builder itemList notEmpty.
! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testPrintOn
	
	| model |
	model := MooseModel new.
	self assert: (model printString = 'a MooseModel #noname(0)').

	model := MooseModel new.
	model name: 'hello'.
	self assert: (model printString = 'a MooseModel #hello(0)').

	model := MooseModel new.
	model name: 'hello2'.
	model add: (MooseEntity new).
	self assert: (model printString = 'a MooseModel #hello2(1)').
	
	 self assert: (MooseModel meta printString = 'a FMMetaRepository')
! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testReferenceModel
	
	| model parentModel |
"	self assert: MooseModel root mooseModel isNil."
	model := MooseModel new.
	parentModel := MooseModel new.
	parentModel add: model.
	self assert: model localMooseModel == model.
	self assert: model mooseModel == parentModel! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testReject

	| t group el1 el2 el3 v |
	t := OrderedCollection new.
	group := MooseModel new.
	group add: (el1 := MooseEntity new).
	group add: (el2 := MooseEntity new).
	group add: (el3 := MooseEntity new).

	v := group reject: [ :el | el == el1 ].
	self assert: (v size = 2).
	self assert: (v includesAll: {el2 . el3})! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testRemoveAll

	| t group el1 el2 el3 v |
	group := MooseModel new.
	group add: (el1 := MooseEntity new).
	group add: (el2 := MooseEntity new).
	group add: (el3 := MooseEntity new).

	group removeAll: {el1 . el2 . el3}.
	self assert: group isEmpty! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testRemoveAll2

	| t group el1 el2 el3 v |
	group := MooseModel new.
	group add: (el1 := MooseEntity new).
	group add: el1.
	group add: (el2 := MooseEntity new).
	group add: (el3 := MooseEntity new).

	group removeAll: {el1 . el2 . el3}.
	self deny: group isEmpty.
	self assert: group entities asArray = {el1}! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testRemoveAnnouncement
	| entity announcedEntity |
	entity := MooseEntity new.
	announcedEntity := nil.
	model announcer when: MooseEntityRemoved do: [:a | 
		announcedEntity := a entity ].
	model add: entity.
	self assert: announcedEntity isNil.
	model remove: entity.
	self assert: announcedEntity == entity.! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testRemoveFromModel
	
	| model parentModel |
	model := MooseModel new.
	parentModel := MooseModel new.
	parentModel add: model.
	self assert: model mooseModel == parentModel.
	self assert: model removeFromModel == model.
	self assert: parentModel entities isEmpty.
! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testRemoveModelNamedFromRoot
	"self debug: #testRemoveModelNamedFromRoot"
	
	| model1 model2 parentModel res |
	model1 := MooseModel new.
	model1 name: 'zork'.
	model2 := MooseModel new.
	model2 name: 'baz'.
	parentModel := MooseModel new.
	parentModel add: model1.
	parentModel add: model2.
	self assert: model1 mooseModel == parentModel.
	self assert: model2 mooseModel == parentModel.
	self assert: parentModel size = 2.
	res := (parentModel removeModelNamed: 'zork').
	self assert: res name = #zork.
	self assert: res mooseModel isNil.
	self deny: (parentModel includes: res).
	self assert: parentModel size = 1.
	! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testRenamedAnnouncement
	| oldName |
	model announcer when: MooseEntityRenamed do: [ :a | 
		oldName := a oldName ].
	model name: #somename.
	self assert: oldName = #noname.
	model name: #anothername.
	self assert: oldName = #somename.! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testRootFolder

	model := MooseModel new.
	self assert: model rootFolder = (FileDirectory default fullName,
											FileDirectory slash,
											'src',
											FileDirectory slash,
											model name,
											FileDirectory slash ).
	! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testRootModel
	
	self assert: MooseModel new mooseID  > 0! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testRootUniqueness
	self assert: MooseModel root == MooseModel root! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testSelect

	| t group el1 el2 el3 v |
	t := OrderedCollection new.
	group := MooseModel new.
	group add: (el1 := MooseEntity new).
	group add: (el2 := MooseEntity new).
	group add: (el3 := MooseEntity new).

	v := group select: [ :el | el == el1 ].
	self assert: (v size = 1).
	self assert: (v first == el1)! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testUnion

	| model1 model2 model3 el1 el2 el3 el4 |
	model1 := MooseModel new.
	model1 add: (el1 := MooseEntity new).
	model1 add: (el2 := MooseEntity new).
	model1 add: (el3 := MooseEntity new).

	model2 := MooseModel new.
	model2 add: (el4 := MooseEntity new).

	model3 := model1 union: model2.
	self assert: (model3 entities includesAll: {el1 . el2 . el3 . el4}).
	self assert: (model3 entities size = 4).

	self assert: ((model2 union: model1) entities includesAll: {el1 . el2 . el3 . el4}).
	self assert: ((model2 union: model1) ~= (model1 union: model2)).! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testUnion2

	| model1 group2 group3 el1 el2 el3 el4 |
	model1 := MooseModel new.
	model1 add: (el1 := MooseEntity new).
	model1 add: (el2 := MooseEntity new).
	model1 add: (el3 := MooseEntity new).

	group2 := MooseGroup new.
	group2 add: (el4 := MooseEntity new).

	group3 := model1 union: group2.
	self assert: (group3 entities includesAll: {el1 . el2 . el3 . el4}).
	self assert: (group3 entities size = 4).

	self assert: ((group2 union: model1) entities includesAll: {el1 . el2 . el3 . el4}).
	self assert: ((group2 union: model1) ~= (model1 union: group2)).! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testUnknownFAMIXClass
	self assert: (model unknownFAMIXClass isKindOf: FAMIXClass).
	self assert: (model unknownFAMIXClass == model unknownFAMIXClass).
	self assert: (model unknownFAMIXClass isStub)! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testUnknownFAMIXNamespace
	self assert: (model unknownFAMIXNamespace isKindOf: FAMIXNamespace).
	self assert: (model unknownFAMIXNamespace == model unknownFAMIXNamespace).
	self assert: (model unknownFAMIXNamespace isStub)! !

!MooseModelTest methodsFor: 'tests' stamp: ''!
testUnknownProperty
	
	| model |
	model := MooseModel new.
	self assert: (model propertyNamed: #UNKNOWN) isNil.
	model propertyNamed: 'UNKNOWN' put: 10.
	self assert: (model propertyNamed: #UNKNOWN) = 10! !

TestCase subclass: #NumericalLinearMapperModelTest
	instanceVariableNames: 'initialMapper glyphProperty modelClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!NumericalLinearMapperModelTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	glyphProperty := #height.
	modelClass := FAMIXMethod.
	initialMapper := NumericalLinearMapper new
		function: #numberOfLinesOfCode;
		minimumOutput: 13;
		maximumOutput: 1780! !

!NumericalLinearMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCreation
	| mapperModel |
	mapperModel := NumericalLinearMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel mapper = initialMapper! !

!NumericalLinearMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInitializeMaximumOutput
	| mapperModel |
	mapperModel := NumericalLinearMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel initializeMaximumOutput = initialMapper maximumOutput! !

!NumericalLinearMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInitializeMinimumOutput
	| mapperModel |
	mapperModel := NumericalLinearMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel initializeMinimumOutput = initialMapper minimumOutput! !

!NumericalLinearMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInitializeModelProperty
	| mapperModel |
	mapperModel := NumericalLinearMapperModel newMapper: initialMapper
				forModelClass: modelClass.
	self assert: mapperModel initializeModelProperty = initialMapper function! !

!NumericalLinearMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testModelProperty
	| mapperModel |
	mapperModel := NumericalLinearMapperModel newMapper: nil
				forModelClass: modelClass.
	self assert: mapperModel modelProperty value isNil.
	mapperModel := NumericalLinearMapperModel newMapper: initialMapper
				forModelClass: modelClass.
	self assert: mapperModel modelProperty value = initialMapper function! !

!NumericalLinearMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testModelPropertyList
	| mapperModel |
	mapperModel := NumericalLinearMapperModel newMapper: initialMapper
				forModelClass: modelClass.
	self assert: mapperModel modelPropertyList
		equals: (modelClass new allDeclaredProperties 
				collect: [:e | e definingMethod selector])! !

!NumericalLinearMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testOutputMapper
	| mapperModel newProperty newMaxOutput newMinOutput resultingSpecModel |
	mapperModel := NumericalLinearMapperModel newMapper: initialMapper forModelClass: modelClass.
	newProperty := #numberOfInvocations.
	newMinOutput := -7.
	newMaxOutput := 2198.
	mapperModel modelProperty value: newProperty.	
	mapperModel minimumOutput value: newMinOutput.
	mapperModel maximumOutput value: newMaxOutput.
	resultingSpecModel := (initialMapper copy)
				function: newProperty;
				minimumOutput: newMinOutput;
				maximumOutput: newMaxOutput.
	self assert: mapperModel outputMapper = resultingSpecModel! !

TestCase subclass: #NumericalLinearMapperTest
	instanceVariableNames: 'mapper values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!NumericalLinearMapperTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	mapper := NumericalLinearMapper new.
	values := #(0 1 2 5 3).
	mapper population: values; minimumOutput: 100; maximumOutput: 150.! !

!NumericalLinearMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAsScript
	| instantiatedBlockMapper uninstantiatedBlockMapper scripted |
	instantiatedBlockMapper := NumericalLinearMapper new 
				function: #numberOfMethods;
				minimumOutput: -13;
				maximumOutput: 999.
	self assert: instantiatedBlockMapper 
				= (Compiler evaluate: instantiatedBlockMapper asScript).
	uninstantiatedBlockMapper := NumericalLinearMapper new.
	self assert: uninstantiatedBlockMapper 
				= (Compiler evaluate: uninstantiatedBlockMapper asScript).
	scripted := Compiler evaluate: instantiatedBlockMapper asScript.
	self assert: scripted population isEmpty! !

!NumericalLinearMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildConverter
	| converter |
	converter := mapper converter.
	self assert: converter class = LinearConverter.
	self assert: converter minimumInput = 0.
	self assert: converter maximumInput = 5.
	self assert: converter minimumOutput = 100.
	self assert: converter maximumOutput = 150! !

!NumericalLinearMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testComputeValueOn
	self assert: (mapper computeValueOn: 0) = 100.
	self assert: (mapper computeValueOn: 5) = 150.
	self assert: (mapper computeValueOn: 3) = 130.
	self assert: (mapper computeValueOn: 1.5) = 115! !

!NumericalLinearMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testEquality
	| i1 i2 notI stillI1 |
	i1 := (NumericalLinearMapper new)
				function: #numberOfMethods;
				minimumOutput: -13;
				maximumOutput: 999.
	i2 := i1 copy.
	self assert: i1 = i2.
	#(#function: #minimumOutput: #maximumOutput:) do: 
			[:each | 
			notI := i1 copy perform: each with: nil.
			self deny: notI = i1].
	"population is not taken into account for the equality test"
	stillI1 := i1 copy perform: #population: with: #(1 2).
	self assert: stillI1 = i1! !

TestCase subclass: #PercentileConverterTest
	instanceVariableNames: 'epsilon'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!PercentileConverterTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	epsilon := 0.0001! !

!PercentileConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConvertForPercentiles
	| discretizer |
	discretizer := PercentileConverter newFrom: 0 to: 100 percentiles: #(30 50 100).
	self assert: (discretizer convert: -1) isNil.
	self assert: (discretizer convert: 0) = 1.
	self assert: (discretizer convert: 30) = 1.
	self assert: (discretizer convert: 31) = 2.
	self assert: (discretizer convert: 50) = 2.
	self assert: (discretizer convert: 51) = 3.
	self assert: (discretizer convert: 100) = 3.
	self assert: (discretizer convert: 101) isNil! !

!PercentileConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConvertForResolution
	|adapter|
	adapter := PercentileConverter newFrom: 0 to: 10 resolution: 5.
	self assert: (adapter convert: -0.01) isNil.
	self assert: (adapter convert: 0) = 1.
	self assert: (adapter convert: 2) = 1.
	self assert: (adapter convert: 2.01) = 2.
	self assert: (adapter convert: 4) = 2.
	self assert: (adapter convert: 4.01) = 3.
	self assert: (adapter convert: 6) = 3.
	self assert: (adapter convert: 6.01) = 4.
	self assert: (adapter convert: 8) = 4.
	self assert: (adapter convert: 8.01) = 5.
	self assert: (adapter convert: 10) = 5.
	self assert: (adapter convert: 10.01) isNil.

	adapter := PercentileConverter newFrom: -3 to: 2 resolution: 2.
	self assert: (adapter convert: -3.01) isNil.
	self assert: (adapter convert: -3) = 1.
	self assert: (adapter convert: -2.5) = 1.
	self assert: (adapter convert: -1) = 1.
	self assert: (adapter convert: -0.5) = 1.
	self assert: (adapter convert: -0.49) = 2.
	self assert: (adapter convert: 0) = 2.
	self assert: (adapter convert: 1) = 2.
	self assert: (adapter convert: 2) = 2.
	self assert: (adapter convert: 3) isNil.! !

!PercentileConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConvertForThresholds
	| discretizer |
	discretizer := PercentileConverter newFrom: 0 to: 100 thresholds: #(30 50 100).
	self assert: (discretizer convert: -1) isNil.
	self assert: (discretizer convert: 0) = 1.
	self assert: (discretizer convert: 30) = 1.
	self assert: (discretizer convert: 31) = 2.
	self assert: (discretizer convert: 50) = 2.
	self assert: (discretizer convert: 51) = 3.
	self assert: (discretizer convert: 100) = 3.
	self assert: (discretizer convert: 101) isNil! !

!PercentileConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConvertForThresholdsWithBothExtremes
	| discretizer |
	discretizer := PercentileConverter newFrom: 0 to: 10 thresholds: #(3 5 7).
	self assert: (discretizer convert: -1) isNil.
	self assert: (discretizer convert: 0) = 1.
	self assert: (discretizer convert: 3) = 1.
	self assert: (discretizer convert: 3.1) = 2.
	self assert: (discretizer convert: 5) = 2.
	self assert: (discretizer convert: 5.1) = 3.
	self assert: (discretizer convert: 7) = 3.
	self assert: (discretizer convert: 7.1) = 4.
	self assert: (discretizer convert: 10) = 4.
	self assert: (discretizer convert: 10.1) isNil! !

!PercentileConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConvertForThresholdsWithLowerExtreme
	| discretizer |
	discretizer := PercentileConverter newFrom: 4 to: 10 thresholds: #(3 5 7).
	self assert: (discretizer convert: 3) isNil.
	self assert: (discretizer convert: 4) = 2.
	self assert: (discretizer convert: 5) = 2.
	self assert: (discretizer convert: 5.1) = 3.
	self assert: (discretizer convert: 7) = 3.
	self assert: (discretizer convert: 7.1) = 4.
	self assert: (discretizer convert: 10) = 4.
	self assert: (discretizer convert: 10.1) isNil! !

!PercentileConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConvertForThresholdsWithNoExtremes
	| discretizer |
	discretizer := PercentileConverter newFrom: 4 to: 6 thresholds: #(3 5 7).
	self assert: (discretizer convert: 3) isNil.
	self assert: (discretizer convert: 4) = 2.
	self assert: (discretizer convert: 5) = 2.
	self assert: (discretizer convert: 5.1) = 3.
	self assert: (discretizer convert: 6) = 3.
	self assert: (discretizer convert: 6.1) isNil! !

!PercentileConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConvertForThresholdsWithUpperExtreme
	| discretizer |
	discretizer := PercentileConverter newFrom: 1 to: 6 thresholds: #(3 5 7).
	self assert: (discretizer convert: 0) isNil.
	self assert: (discretizer convert: 1) = 1.
	self assert: (discretizer convert: 3) = 1.
	self assert: (discretizer convert: 3.1) = 2.
	self assert: (discretizer convert: 5) = 2.
	self assert: (discretizer convert: 5.1) = 3.
	self assert: (discretizer convert: 6) = 3.
	self assert: (discretizer convert: 6.1) isNil! !

!PercentileConverterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConvertWithVoidRange
	| converter |
	converter := PercentileConverter newFrom: 1 to: 1 thresholds: #(3 5 7).
	self assert: (converter convert: 1) = 1! !

TestCase subclass: #PopulationtMapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!PopulationtMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testComputeMaximumInput
	| mapper values |
	mapper := PopulationMapper new.
	self assert: mapper function = #yourself.
	values := #(1 -8 100 3 -4 12).
	self assert: mapper computeMaximumInput isNil.
	mapper population: values.
	self assert: mapper computeMaximumInput = 100! !

!PopulationtMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testComputeMinimumInput
	| mapper values |
	mapper := PopulationMapper new.
	self assert: mapper function = #yourself.
	values := #(1 -8 100 3 -4 12).
	self assert: mapper computeMinimumInput isNil.
	mapper population: values.
	self assert: mapper computeMinimumInput = -8! !

!PopulationtMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConverter
	| mapper |
	mapper := PopulationMapper new.
	self assert: mapper population isEmpty.
	mapper population: #(1 2 3 4 5).
	self assert: mapper population size = 5.
	self should: [mapper converter] raise: SubclassResponsibilityError! !

!PopulationtMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testSortedFunctionValues
	| mapper values |
	mapper := PopulationMapper new.
	values := #(1 -8 100 3 -4 12).
	mapper population: values.
	self assert: mapper sortedFunctionValues equals: #(-8 -4 1 3 12 100)! !

TestCase subclass: #ProgressiveBricksLayoutTest
	instanceVariableNames: 'glyphs epsilon mooseModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!ProgressiveBricksLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBrickIndexWithinLayerForBrickIndexBricksPerSide
	|layout|
	layout := ProgressiveBricksLayout new.
	self assert: (layout brickIndexWithinLevelOfBrickWithIndex: 0 forSideCapacity: 2) = 0.
	self assert: (layout brickIndexWithinLevelOfBrickWithIndex: 3 forSideCapacity: 2) = 3.
	self assert: (layout brickIndexWithinLevelOfBrickWithIndex: 5 forSideCapacity: 2) = 1.
	self assert: (layout brickIndexWithinLevelOfBrickWithIndex: 1 forSideCapacity: 5) = 1.
	self assert: (layout brickIndexWithinLevelOfBrickWithIndex: 16 forSideCapacity: 5) = 0.
	self assert: (layout brickIndexWithinLevelOfBrickWithIndex: 17 forSideCapacity: 5) = 1.
	self assert: (layout brickIndexWithinLevelOfBrickWithIndex: 24 forSideCapacity: 5) = 8.
	self assert: (layout brickIndexWithinLevelOfBrickWithIndex: 32 forSideCapacity: 5) = 0.
	self assert: (layout brickIndexWithinLevelOfBrickWithIndex: 33 forSideCapacity: 5) = 1! !

!ProgressiveBricksLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBrickIndexWithinSideForBrickIndexBricksPerSide
	|layout|
	layout := ProgressiveBricksLayout new.
	self assert: (layout brickIndexWithinSideOfBrickWithIndex: 1 forSideCapacity: 1) = 0.
	self assert: (layout brickIndexWithinSideOfBrickWithIndex: 1 forSideCapacity: 2) = 0.
	self assert: (layout brickIndexWithinSideOfBrickWithIndex: 2 forSideCapacity: 2) = 0.
	self assert: (layout brickIndexWithinSideOfBrickWithIndex: 3 forSideCapacity: 2) = 0.
	self assert: (layout brickIndexWithinSideOfBrickWithIndex: 4 forSideCapacity: 2) = 0.
	self assert: (layout brickIndexWithinSideOfBrickWithIndex: 1 forSideCapacity: 5) = 1.
	self assert: (layout brickIndexWithinSideOfBrickWithIndex: 4 forSideCapacity: 5) = 0.
	self assert: (layout brickIndexWithinSideOfBrickWithIndex: 5 forSideCapacity: 5) = 1.
	self assert: (layout brickIndexWithinSideOfBrickWithIndex: 6 forSideCapacity: 5) = 2.
	self assert: (layout brickIndexWithinSideOfBrickWithIndex: 7 forSideCapacity: 5) = 3.
	self assert: (layout brickIndexWithinSideOfBrickWithIndex: 8 forSideCapacity: 5) = 0.
	self assert: (layout brickIndexWithinSideOfBrickWithIndex: 9 forSideCapacity: 5) = 1.
	self assert: (layout brickIndexWithinSideOfBrickWithIndex: 12 forSideCapacity: 5) = 0.
	self assert: (layout brickIndexWithinSideOfBrickWithIndex: 13 forSideCapacity: 5) = 1.
	self assert: (layout brickIndexWithinSideOfBrickWithIndex: 16 forSideCapacity: 5) = 0.
	self assert: (layout brickIndexWithinSideOfBrickWithIndex: 17 forSideCapacity: 5) = 1! !

!ProgressiveBricksLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBricksPerSideForElementSetSize
	|layout|
	layout := ProgressiveBricksLayout new.
	self assert: (layout sideCapacityForCardinality: 0) = 1.
	self assert: (layout sideCapacityForCardinality: 7) = 1.
	self assert: (layout sideCapacityForCardinality: 8) = 2.
	self assert: (layout sideCapacityForCardinality: 31) = 2.
	self assert: (layout sideCapacityForCardinality: 32) = 3.
	self assert: (layout sideCapacityForCardinality: 71) = 3.
	self assert: (layout sideCapacityForCardinality: 72) = 4.
	self assert: (layout sideCapacityForCardinality: 127) = 4.
	self assert: (layout sideCapacityForCardinality: 128) = 5! !

!ProgressiveBricksLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCoordinateFactorForBrickIndexBricksPerSide
	|layout|
	layout := ProgressiveBricksLayout new.
	self assert: (layout coordinateFactorOfBrickIndex: 1 forSideCapacity: 1) = (0,0,0).
	self assert: (layout coordinateFactorOfBrickIndex: 1 forSideCapacity: 2) = (0,0,0).
	self assert: (layout coordinateFactorOfBrickIndex: 2 forSideCapacity: 2) = (1,0,0).
	self assert: (layout coordinateFactorOfBrickIndex: 3 forSideCapacity: 2) = (1,1,0).
	self assert: (layout coordinateFactorOfBrickIndex: 4 forSideCapacity: 2) = (0,1,0).
	self assert: (layout coordinateFactorOfBrickIndex: 1 forSideCapacity: 5) = (0,0,0).
	self assert: (layout coordinateFactorOfBrickIndex: 2 forSideCapacity: 5) = (1,0,0).
	self assert: (layout coordinateFactorOfBrickIndex: 3 forSideCapacity: 5) = (2,0,0).
	self assert: (layout coordinateFactorOfBrickIndex: 4 forSideCapacity: 5) = (3,0,0).
	self assert: (layout coordinateFactorOfBrickIndex: 5 forSideCapacity: 5) = (4,0,0).
	self assert: (layout coordinateFactorOfBrickIndex: 6 forSideCapacity: 5) = (4,1,0).
	self assert: (layout coordinateFactorOfBrickIndex: 7 forSideCapacity: 5) = (4,2,0).
	self assert: (layout coordinateFactorOfBrickIndex: 8 forSideCapacity: 5) = (4,3,0).
	self assert: (layout coordinateFactorOfBrickIndex: 9 forSideCapacity: 5) = (4,4,0).
	self assert: (layout coordinateFactorOfBrickIndex: 10 forSideCapacity: 5) = (3,4,0).
	self assert: (layout coordinateFactorOfBrickIndex: 11 forSideCapacity: 5) = (2,4,0).
	self assert: (layout coordinateFactorOfBrickIndex: 12 forSideCapacity: 5) = (1,4,0).
	self assert: (layout coordinateFactorOfBrickIndex: 13 forSideCapacity: 5) = (0,4,0).
	self assert: (layout coordinateFactorOfBrickIndex: 14 forSideCapacity: 5) = (0,3,0).
	self assert: (layout coordinateFactorOfBrickIndex: 15 forSideCapacity: 5) = (0,2,0).
	self assert: (layout coordinateFactorOfBrickIndex: 16 forSideCapacity: 5) = (0,1,0).
	self assert: (layout coordinateFactorOfBrickIndex: 17 forSideCapacity: 5) = (0,0,1).! !

!ProgressiveBricksLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testLayerIndexForBrickIndexBricksPerSide
	|layout|
	layout := ProgressiveBricksLayout new.
	self assert: (layout levelIndexOfBrickIndex: 1 forSideCapacity: 2) = 0.
	self assert: (layout levelIndexOfBrickIndex: 4 forSideCapacity: 2) = 1.
	self assert: (layout levelIndexOfBrickIndex: 5 forSideCapacity: 2) = 1.
	self assert: (layout levelIndexOfBrickIndex: 1 forSideCapacity: 5) = 0.
	self assert: (layout levelIndexOfBrickIndex: 16 forSideCapacity: 5) = 1.
	self assert: (layout levelIndexOfBrickIndex: 17 forSideCapacity: 5) = 1.
	self assert: (layout levelIndexOfBrickIndex: 32 forSideCapacity: 5) = 2.
	self assert: (layout levelIndexOfBrickIndex: 33 forSideCapacity: 5) = 2.! !

!ProgressiveBricksLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testMaximumNumberOfBricksForBricksPerSide
	|layout|
	layout := ProgressiveBricksLayout new.
	self assert: (layout maximumBuildingCapacityForSideCapacity: 1) = 7.
	self assert: (layout maximumBuildingCapacityForSideCapacity: 2) = 31.
	self assert: (layout maximumBuildingCapacityForSideCapacity: 3) = 71.
	self assert: (layout maximumBuildingCapacityForSideCapacity: 4) = 127! !

!ProgressiveBricksLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testMinimumNumberOfBricksForBricksPerSide
	|layout|
	layout := ProgressiveBricksLayout new.
	self assert: (layout minimumBuildingCapacityForSideCapacity: 1) = 0.
	self assert: (layout minimumBuildingCapacityForSideCapacity: 2) = 8.
	self assert: (layout minimumBuildingCapacityForSideCapacity: 3) = 32.
	self assert: (layout minimumBuildingCapacityForSideCapacity: 4) = 72.
	self assert: (layout minimumBuildingCapacityForSideCapacity: 5) = 128! !

!ProgressiveBricksLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testMinimumNumberOfLayersForBricksPerSide
	|layout|
	layout := ProgressiveBricksLayout new.
	self assert: (layout minimumNumberOfLevelsForSideCapacity: 1) = 0.
	self assert: (layout minimumNumberOfLevelsForSideCapacity: 2) = 2.
	self assert: (layout minimumNumberOfLevelsForSideCapacity: 3) = 4.
	self assert: (layout minimumNumberOfLevelsForSideCapacity: 4) = 6! !

!ProgressiveBricksLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testNumberOfBricksPerLayerForBricksPerSide
	|layout|
	layout := ProgressiveBricksLayout new.
	self assert: (layout levelCapacityForSideCapacity: 1) = 1.
	self assert: (layout levelCapacityForSideCapacity: 2) = 4.
	self assert: (layout levelCapacityForSideCapacity: 3) = 8.
	self assert: (layout levelCapacityForSideCapacity: 4) = 12.
	self assert: (layout levelCapacityForSideCapacity: 5) = 16! !

!ProgressiveBricksLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testSideIndexForBrickIndexBricksPerSide
	|layout|
	layout := ProgressiveBricksLayout new.
	self assert: (layout sideIndexOfBrickIndex: 1 forSideCapacity: 1) = 0.
	self assert: (layout sideIndexOfBrickIndex: 1 forSideCapacity: 2) = 1.
	self assert: (layout sideIndexOfBrickIndex: 2 forSideCapacity: 2) = 2.
	self assert: (layout sideIndexOfBrickIndex: 3 forSideCapacity: 2) = 3.
	self assert: (layout sideIndexOfBrickIndex: 4 forSideCapacity: 2) = 0.
	self assert: (layout sideIndexOfBrickIndex: 5 forSideCapacity: 2) = 1.
	self assert: (layout sideIndexOfBrickIndex: 1 forSideCapacity: 5) = 0.
	self assert: (layout sideIndexOfBrickIndex: 4 forSideCapacity: 5) = 1.
	self assert: (layout sideIndexOfBrickIndex: 5 forSideCapacity: 5) = 1.
	self assert: (layout sideIndexOfBrickIndex: 8 forSideCapacity: 5) = 2.
	self assert: (layout sideIndexOfBrickIndex: 9 forSideCapacity: 5) = 2.
	self assert: (layout sideIndexOfBrickIndex: 13 forSideCapacity: 5) = 3.
	self assert: (layout sideIndexOfBrickIndex: 16 forSideCapacity: 5) = 0.
	self assert: (layout sideIndexOfBrickIndex: 17 forSideCapacity: 5) = 0! !

TestCase subclass: #QuadraticLayoutTest
	instanceVariableNames: 'glyphs epsilon dimensions relativePositions layout expectedBox'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!QuadraticLayoutTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	|hMargin vMargin hGap sizeX sizeY sizeZ|
	epsilon := 0.0001.
	dimensions := (OrderedCollection new)
				add: 3 , 1 , 1; add: 5 , 8 , 2; add: 7 , 2 , 3;
				add: 6 , 6 , 4; add: 4 , 3 , 5; add: 1 , 2 , 6;
				add: 4 , 9 , 7;
				yourself.
	glyphs := dimensions collect: [:each | CuboidGlyph new dimension: each].
	layout := QuadraticLayout 
				newHorizontalMargin: 0.7 horizontalGap: 1
				verticalMargin: 0.2 verticalGap: 2.
	hMargin := layout horizontalMargin.
	hGap := layout horizontalGap.
	vMargin := layout verticalMargin.
	relativePositions := (OrderedCollection new)
				add: hMargin , hMargin , vMargin;
				add: (hMargin + 3 + hGap) , hMargin , vMargin;
				add: (hMargin + 3 + hGap + 5 + hGap) , hMargin , vMargin;
				add: hMargin , (hMargin + 8 + hGap) , vMargin;
				add: (hMargin + 6 + hGap) , (hMargin + 8 + hGap) , vMargin;
				add: (hMargin + 6 + hGap + 4 + hGap) , (hMargin + 8 + hGap) , vMargin;
				add: hMargin, (hMargin + 8 + hGap + 6 + hGap), vMargin;
				yourself.
	sizeX := ((hMargin + 3 + hGap + 5 + hGap + 7 + hMargin) 
					max: (hMargin + 6 + hGap + 4 + hGap + 1 + hMargin))
						max: (hMargin + 4 + hMargin).
	sizeY := hMargin + 8 + hGap + 6 + hGap + 9 + hMargin.
	sizeZ := vMargin + 7 + vMargin.
	expectedBox := sizeX, sizeY, sizeZ.! !

!QuadraticLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testLayoutGlyphsOrigin
	| origin largestSize box biasedPositions actualPositions |
	origin := 3 , 4 , 5.
	self assert: (glyphs notNil and: [glyphs size = dimensions size]).
	self assert: (glyphs collect: [:each | each dimension]) = dimensions.
	box := layout layoutGlyphs: glyphs origin: origin.
	largestSize := layout computeLargestDimensionForGlyphs: glyphs.
	self assert: largestSize = (7 , 9 , 7).
	self assert: (box x - expectedBox x) abs < epsilon.
	self assert: (box y - expectedBox y) abs < epsilon.
	self assert: (box z - expectedBox z) abs < epsilon.
	"arranges row-wise (on x axis)"
	biasedPositions := relativePositions collect: [:each | each + origin].
	actualPositions := glyphs collect: [:each | each position].
	self assert: actualPositions size = biasedPositions size.
	1 to: actualPositions size
		do: 
			[:eachIndex | 
			self 
				assert: ((actualPositions at: eachIndex) - (biasedPositions at: eachIndex)) 
						abs < epsilon]! !

TestCase subclass: #RectanglePackingLayoutTest
	instanceVariableNames: 'glyphs epsilon mooseModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!RectanglePackingLayoutTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
modelSpec
	^
#(Moose.Model (entity
	(FAMIX.Namespace (id: 1) (name 'one::namespace'))
	(FAMIX.Class (id: 2) (name TestClass1) (belongsTo (idref: 1)))
	(FAMIX.Class (id: 3) (name TestClass2) (belongsTo (idref: 1)))
	(FAMIX.Class (id: 4) (name TestClass3) (belongsTo (idref: 1)))
	(FAMIX.Class (id: 5) (name TestClass4) (belongsTo (idref: 1)))
	(FAMIX.Class (id: 6) (name TestClass5) (belongsTo (idref: 1)))
	(FAMIX.Class (id: 7) (name TestClass6) (belongsTo (idref: 1)))

))! !

!RectanglePackingLayoutTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	| sizes |
	sizes := (OrderedCollection new)
				add: 8 , 6 , 1;
				add: 4 , 6 , 1;
				add: 4 , 5 , 1;
				add: 6 , 4 , 1;
				add: 5 , 4 , 1;
				add: 4 , 4 , 1;
				add: 3 , 4 , 1;
				add: 5 , 3 , 1;
				add: 3 , 1 , 1;
				yourself.
	epsilon := 0.001.
	glyphs := sizes 
				collect: [:each | CuboidGlyph new dimension: each].
	mooseModel := (Importer withActiveMetamodel 
				importArray: self modelSpec) name: #TestModel.! !

!RectanglePackingLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testLayoutGlyphs
	| layoutManager expectedPositions containerSize spacing|
	layoutManager := RectanglePackingLayout 
				newHorizontalMargin: 0.7 horizontalGap: 1
				verticalMargin: 0.2 verticalGap: 2.

	spacing := layoutManager horizontalMargin, layoutManager horizontalMargin, layoutManager verticalMargin.
	containerSize := layoutManager layoutGlyphs: glyphs origin: 0 , 0 , 0.
	self assert: (containerSize x - 18.4) abs < epsilon.
	self assert: (containerSize y - 19.4) abs < epsilon.
	self assert: (containerSize z - 1.4) abs < epsilon.
	expectedPositions := (OrderedCollection new)
				add: (0 , 0 , 0) + spacing;
				add: (0 , 7 , 0) + spacing;
				add: (9 , 0 , 0) + spacing;
				add: (5 , 7 , 0) + spacing;
				add: (12 , 7 , 0) + spacing;
				add: (0 , 14 , 0) + spacing;
				add: (5 , 14 , 0) + spacing;
				add: (9 , 14 , 0) + spacing;
				add: (5, 12 , 0) + spacing;
				yourself.
	1 to: glyphs size
		do: 
			[:each | 
			| expected actual |
			expected := expectedPositions at: each.
			actual := (glyphs at: each) position.
			self assert: (expected x - actual x) abs < epsilon.
			self assert: (expected y - actual y) abs < epsilon.
			self assert: (expected z - actual z) abs < epsilon].! !

!RectanglePackingLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testSortGlyphs
	| layoutManager unsortedSizes unsortedGlyphs sortedGlyphs expectedIndexes classes|
	layoutManager := RectanglePackingLayout new.
	unsortedSizes := (OrderedCollection new)
				add: 5 , 3 , 1;
				add: 4 , 5 , 1;
				add: 4 , 5 , 1;
				add: 4 , 6 , 1;
				add: 8 , 6 , 1;
				add: 3 , 1 , 1;
				yourself.
	classes := mooseModel allClasses entities.
	self assert: classes size = unsortedSizes size.
	unsortedGlyphs := OrderedCollection new.
	1 to: classes size do: [:each | |model dimension glyph|
		model := classes at: each.
		dimension := unsortedSizes at: each.
		glyph := CuboidGlyph new model: model; dimension: dimension; yourself.
		unsortedGlyphs add: glyph
	].
	self assert: unsortedGlyphs size = classes size.
	sortedGlyphs := layoutManager sortGlyphs: unsortedGlyphs.
	expectedIndexes := #(5 4 2 3 1 6).
	1 to: sortedGlyphs size do:[:each| |model expectedModel|
		model := (sortedGlyphs at: each) model.
		expectedModel := classes at: (expectedIndexes at: each).
		self assert: model = expectedModel]! !

!RectanglePackingLayoutTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testSymmetricalBug
	| layoutManager glyph1 glyph2 theGlyphs spacing |
	layoutManager := RectanglePackingLayout 
				newHorizontalMargin: 0.7 horizontalGap: 8
				verticalMargin: 0.2 verticalGap: 2.
	spacing := layoutManager horizontalMargin, layoutManager horizontalMargin, layoutManager verticalMargin.
	glyph1 := (CuboidGlyph new model: mooseModel allClasses entities first)
				dimension: 12 , 12 , 1;
				yourself.
	glyph2 := (CuboidGlyph new model: mooseModel allClasses entities last)
				dimension: 5 , 5 , 1;
				yourself.
	theGlyphs := OrderedCollection with: glyph1 with: glyph2.
	layoutManager layoutGlyphs: theGlyphs origin: (0,0,0).
	self assert: glyph1 position = ((0,0,0) + spacing).
	self assert: glyph2 position = ((20,0,0) + spacing). "using detectMin: can get different results on the same data"! !

TestCase subclass: #RectanglePackingNodeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!RectanglePackingNodeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAllFreeLeavesUnderneath
	| nodes numberOfNodes freeLeavesBefore freeLeavesAfter|
	numberOfNodes := 7.
	nodes := (OrderedCollection new).
	numberOfNodes timesRepeat: [nodes add: RectanglePackingNode new].
	(nodes at: 1) firstChild: (nodes at: 2); secondChild: (nodes at: 3).
	(nodes at: 2) firstChild: (nodes at: 4); secondChild: (nodes at: 5).
	(nodes at: 3) firstChild: (nodes at: 6); secondChild: (nodes at: 7).
	freeLeavesBefore := (nodes at: 1) allFreeLeavesUnderneath.
	self assert: freeLeavesBefore size = 4.
	4 to: 7 do: [:each| self assert: (freeLeavesBefore includes: (nodes at: each))].

	(nodes at: 7) glyph: CuboidGlyph new.
	freeLeavesAfter := (nodes at: 1) allFreeLeavesUnderneath.	
	self assert: freeLeavesAfter size = 3.
	4 to: 6 do: [:each| self assert: (freeLeavesAfter includes: (nodes at: each))].
 	self deny: (freeLeavesAfter includes: (nodes at: 7))! !

!RectanglePackingNodeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testIsFitForGlyph
	| smallNode smallRectangle perfectNode perfectRectangle biggerNode biggerRectangle glyph dimension |
	dimension := 3 , 4 , 5.
	glyph := CuboidGlyph new
				dimension: dimension;
				yourself.

	smallNode := RectanglePackingNode new.
	smallRectangle := Rectangle 
				origin: Point zero
				width: dimension x - 1
				height: dimension y - 1.
	smallNode rectangle: smallRectangle.

	perfectNode := RectanglePackingNode new.
	perfectRectangle := Rectangle 
				origin: Point zero
				width: dimension x
				height: dimension y.
	perfectNode rectangle: perfectRectangle.

	biggerNode := RectanglePackingNode new.
	biggerRectangle := Rectangle 
				origin: Point zero
				width: dimension x + 1
				height: dimension y + 1.
	biggerNode rectangle: biggerRectangle.

	self deny: (smallNode isPerfectFitForGlyph: glyph).
	self assert: (perfectNode isPerfectFitForGlyph: glyph).
	self deny: (biggerNode isPerfectFitForGlyph: glyph).

	self deny: (smallNode isAnyFitForGlyph: glyph).
	self assert: (perfectNode isAnyFitForGlyph: glyph).
	self assert: (biggerNode isAnyFitForGlyph: glyph).! !

!RectanglePackingNodeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testIsFree
	|freeNode occupiedNode glyph|
	freeNode := RectanglePackingNode new.
	occupiedNode	:= RectanglePackingNode new.
	glyph := CuboidGlyph new.
	occupiedNode glyph: glyph.
	self assert: freeNode glyph isNil.
	self assert: freeNode isFree.
	self assert: occupiedNode glyph = glyph.
	self deny:  occupiedNode isFree.! !

!RectanglePackingNodeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testIsLeaf
	|singleChildParentNode twinChildrenParentNode firstChildNode secondChildNode thirdChildNode|
	twinChildrenParentNode := RectanglePackingNode new.
	singleChildParentNode := RectanglePackingNode new.
	firstChildNode	:= RectanglePackingNode new.
	secondChildNode	:= RectanglePackingNode new.
	thirdChildNode := RectanglePackingNode new.
	twinChildrenParentNode firstChild: firstChildNode.
	twinChildrenParentNode secondChild: secondChildNode.
	singleChildParentNode firstChild: thirdChildNode.
	self assert: firstChildNode isLeaf.
	self assert: secondChildNode isLeaf.
	self assert: thirdChildNode isLeaf.
	self deny: singleChildParentNode isLeaf.
	self deny: twinChildrenParentNode isLeaf.! !

!RectanglePackingNodeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testSplitForGlyphForHorizontalSplitCase
	|node rectangle glyph gap|
	CityRoot new todo: 'no need to do the gap size push-pop'.
	gap := 0.7.
	node := RectanglePackingNode new.
	rectangle := Rectangle origin: Point zero width: 7 height: 13.
	node rectangle: rectangle.
	node splitGap: gap.
	node firstChild: RectanglePackingNode new.
	node secondChild: RectanglePackingNode  new.
	self assert: node firstChild rectangle isNil.
	self assert: node secondChild rectangle isNil.
	glyph := CuboidGlyph new dimension: (4,11,1); yourself.
	node splitForGlyph: glyph.
	self assert: node firstChild rectangle left = 0.
	self assert: node firstChild rectangle top = 0.
	self assert: node firstChild rectangle right = 4.
	self assert: node firstChild rectangle bottom = 13.

	self assert: node secondChild rectangle left = ((node firstChild rectangle right + gap) min: node secondChild rectangle right).
	self assert: node secondChild rectangle top = 0.
	self assert: node secondChild rectangle right = 7.
	self assert: node secondChild rectangle bottom = 13.! !

!RectanglePackingNodeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testSplitForGlyphForVerticalSplitCase
	| node rectangle glyph gap |
	CityRoot new todo: 'no need to do the gap size push-pop'.
	gap := 0.7.
	node := RectanglePackingNode new.
	rectangle := Rectangle 
				origin: Point zero
				width: 7
				height: 13.
	node rectangle: rectangle.
	node splitGap: gap.
	node firstChild: RectanglePackingNode new.
	node secondChild: RectanglePackingNode new.
	self assert: node firstChild rectangle isNil.
	self assert: node secondChild rectangle isNil.
	glyph := CuboidGlyph new
				dimension: 4 , 5 , 1;
				yourself.
	node splitForGlyph: glyph.
	self assert: node firstChild rectangle left = 0.
	self assert: node firstChild rectangle top = 0.
	self assert: node firstChild rectangle right = 7.
	self assert: node firstChild rectangle bottom = 5.
	self assert: node secondChild rectangle left = 0.
	self assert: node secondChild rectangle top 
				= (node firstChild rectangle bottom + gap 
						min: node secondChild rectangle bottom).
	self assert: node secondChild rectangle right = 7.
	self assert: node secondChild rectangle bottom = 13! !

TestCase subclass: #RectanglePackingTreeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!RectanglePackingTreeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCornerIfGlyphWereInNode
	|root tree tooBigGlyph exactFitGlyph smallGlyph origin|
	origin := 2 @ 3.
	root := RectanglePackingNode new.
	root rectangle: (Rectangle origin: origin width: 13 height: 7).
	tree := RectanglePackingTree newRoot: root.
	tooBigGlyph := CuboidGlyph new dimension: 10, 9, 1; yourself.
	self assert: (tree cornerIfGlyph: tooBigGlyph wereInNode: root) isNil.
	exactFitGlyph := CuboidGlyph new dimension: 13, 7, 1; yourself.
	self assert: (tree cornerIfGlyph: exactFitGlyph wereInNode: root) = ((2 + 13) @ (3 + 7)).
	smallGlyph := CuboidGlyph new dimension: 9, 5, 1; yourself.
	self assert: (tree cornerIfGlyph: smallGlyph wereInNode: root) = ((2 + 9) @ (3 + 5))! !

!RectanglePackingTreeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testGlobalExtentIfGlyphWereInNode
	|root tree unacceptableGlyph extendingGlyph nonExtendingGlyph origin|
	origin := 2 @ 3.
	root := RectanglePackingNode new.
	root rectangle: (Rectangle origin: origin width: 13 height: 7).
	tree := RectanglePackingTree newRoot: root.
	tree extent: 12 @ 9.
	unacceptableGlyph := CuboidGlyph new dimension: 10, 9, 1; yourself.
	self assert: (tree globalExtentIfGlyph: unacceptableGlyph wereInNode: root) isNil.
	extendingGlyph := CuboidGlyph new dimension: 13, 7, 1; yourself.
	self assert: (tree globalExtentIfGlyph: extendingGlyph wereInNode: root) = ((2 + 13) @ (3 + 7)).
	nonExtendingGlyph := CuboidGlyph new dimension: 9, 5, 1; yourself.
	self assert: (tree globalExtentIfGlyph: nonExtendingGlyph wereInNode: root) = tree extent! !

!RectanglePackingTreeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInsertOptimal
	| root tree rectangle sizes glyphs gap rectangles expectedRectangles|
	gap := 4.
	sizes := (OrderedCollection new)
				add: 8 , 6 , 1; add: 5 , 3 , 1; add: 4 , 5 , 1;
				add: 6 , 4 , 1; add: 4 , 6 , 1; add: 3 , 4 , 1;
				add: 4 , 4 , 1; add: 3 , 1 , 1; add: 5 , 4 , 1;
				yourself.
	glyphs := OrderedCollection new.
	sizes do: [:each | glyphs add: (CuboidGlyph new dimension: each)].
	root := RectanglePackingNode new.
	rectangle := Rectangle origin: Point zero width: (3 * 8) + (2 * gap) height: (3 * 6) + (2 * gap).
	root rectangle: rectangle.
	root splitGap: gap.
	tree := RectanglePackingTree newRoot: root.
	rectangles := glyphs collect: [:each|(tree optimalInsertGlyph: each) rectangle].
	expectedRectangles := (OrderedCollection new)
			add: (Rectangle origin: 0@0 corner: 8@6);
			add: (Rectangle origin: 0@10 corner: 5@13);
			add: (Rectangle origin: 12@0 corner: 16@5);
			add: (Rectangle origin: 0@17 corner: 6@21);
			add: (Rectangle origin: 12@9 corner: 16@15);
			add: (Rectangle origin: 20@0 corner: 23@4);
			add: (Rectangle origin: 12@19 corner: 16@23);
			add: (Rectangle origin: 20@9 corner: 23@10);
			add: (Rectangle origin: 20@14 corner: 25@18);
			yourself.
	self assert: (rectangles = expectedRectangles).! !

!RectanglePackingTreeTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testWastedSpaceIfGlyphWereInNode
	|root tree tooBigGlyph exactFitGlyph smallGlyph|
	root := RectanglePackingNode new.
	root rectangle: (Rectangle origin: Point zero width: 13 height: 7).
	tree := RectanglePackingTree newRoot: root.
	tooBigGlyph := CuboidGlyph new dimension: 10, 9, 1; yourself.
	self assert: (tree wastedSpaceIfGlyph: tooBigGlyph wereInNode: root) isNil.
	exactFitGlyph := CuboidGlyph new dimension: 13, 7, 1; yourself.
	self assert: (tree wastedSpaceIfGlyph: exactFitGlyph wereInNode: root) isZero.
	smallGlyph := CuboidGlyph new dimension: 9, 5, 1; yourself.
	self assert: (tree wastedSpaceIfGlyph: smallGlyph wereInNode: root) = 8! !

TestCase subclass: #RoofTopGlyphTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!RoofTopGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildVolume
	| roofTop dimension position volume boundingBox epsilon epsilon3d roofTopThickness expectedBoundingBox|
	epsilon := 0.00000001.
	epsilon3d := epsilon , epsilon , epsilon.
	dimension := 13 , 14 , 15.
	position := 11 , 23 , 47.
	roofTop := RoofTopGlyph new
				basicPosition: position;
				dimension: dimension;
				yourself.
	roofTopThickness := roofTop class defaultRoofTopThickness.
	volume := roofTop buildVolume.
	self assert: volume notNil.
	self assert: (volume isKindOf: JunOpenGL3dObject).
	boundingBox := volume boundingBox.
	expectedBoundingBox := Jun3dBoundingBox 
		origin: position x, position y, (position z + dimension z - roofTopThickness) 
		corner: position + dimension.
	self assert: (boundingBox origin - expectedBoundingBox origin) abs < epsilon3d.
	self assert: (boundingBox corner - expectedBoundingBox corner) abs < epsilon3d! !

!RoofTopGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInnerDimension
	| roofTop position roofTopThickness expectedDimension dimension |
	position := 11 , 23 , 47.
	dimension := 13 , 14 , 15.
	roofTop := RoofTopGlyph new
				basicPosition: position;
				yourself.
	roofTopThickness := roofTop class defaultRoofTopThickness.
	self assert: roofTopThickness > 0.
	"setter"
	expectedDimension := dimension + (0, 0, roofTopThickness).
	roofTop innerDimension: dimension.
	self assert: roofTop dimension = expectedDimension.
	"getter"
	self assert: roofTop innerDimension = dimension! !

!RoofTopGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInnerOrigin
	| roofTop position |
	position := 11 , 23 , 47.
	roofTop := RoofTopGlyph new
				basicPosition: position;
				yourself.
	"getter"
	self assert: roofTop innerOrigin = position.
	"setter"
	roofTop basicInnerOrigin: position.
	self assert: roofTop position = position! !

TestCase subclass: #SolidWallGlyphTest
	instanceVariableNames: 'epsilon3d dimension position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!SolidWallGlyphTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	epsilon3d := 0.00000001 , 0.00000001 , 0.00000001.
	dimension := 13 , 14 , 15.
	position := 11 , 23 , 47.! !

!SolidWallGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildEastWall
	| wallGlyph aWall thickness origin corner height |
	wallGlyph := SolidWallGlyph new
				basicPosition: position;
				dimension: dimension;
				yourself.
	aWall := wallGlyph buildEastWall.
	self assert: (aWall notNil and: [aWall isKindOf: JunOpenGL3dObject]).
	thickness := wallGlyph class defaultWallThickness.
	height := wallGlyph class defaultWallHeight.
	origin := (position x + dimension x - thickness), position y, position z.
	self assert: (aWall boundingBox origin - origin) abs < epsilon3d.
	corner := (position x + dimension x), (position y + dimension y), (position z + height).
	self assert: (aWall boundingBox corner - corner) abs < epsilon3d! !

!SolidWallGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildNorthWall
	| wallGlyph aWall thickness origin corner height |
	wallGlyph := SolidWallGlyph new
				basicPosition: position;
				dimension: dimension;
				yourself.
	aWall := wallGlyph buildNorthWall.
	self assert: (aWall notNil and: [aWall isKindOf: JunOpenGL3dObject]).
	thickness := wallGlyph class defaultWallThickness.
	height := wallGlyph class defaultWallHeight.
	origin := position x, (position y + dimension y - thickness), position z.
	self assert: (aWall boundingBox origin - origin) abs < epsilon3d.
	corner := (position x + dimension x), (position y + dimension y), (position z + height).
	self assert: (aWall boundingBox corner - corner) abs < epsilon3d! !

!SolidWallGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildSouthWall
	| wallGlyph aWall thickness origin corner height |
	wallGlyph := SolidWallGlyph new
				basicPosition: position;
				dimension: dimension;
				yourself.
	aWall := wallGlyph buildSouthWall.
	self assert: (aWall notNil and: [aWall isKindOf: JunOpenGL3dObject]).
	thickness := wallGlyph class defaultWallThickness.
	height := wallGlyph class defaultWallHeight.
	origin := position.
	self assert: (aWall boundingBox origin - origin) abs < epsilon3d.
	corner := (position x + dimension x), (position y + thickness), (position z + height).
	self assert: (aWall boundingBox corner - corner) abs < epsilon3d! !

!SolidWallGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildVolume
	| wall volume boundingBox wallHeight |
	wall := SolidWallGlyph new
				basicPosition: position;
				dimension: dimension;
				yourself.
	wallHeight := wall class defaultWallHeight.
	volume := wall buildVolume.
	self assert: volume notNil.
	self assert: (volume isKindOf: JunOpenGL3dObject).
	boundingBox := volume boundingBox.
	self assert: (boundingBox origin - position) abs < epsilon3d.
	self assert: (boundingBox corner x - dimension x - position x) abs < epsilon3d.
	self assert: (boundingBox corner y - dimension y - position y) abs < epsilon3d.
	self assert: (boundingBox corner z - wallHeight - position z) abs < epsilon3d! !

!SolidWallGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildWestWall
	| wallGlyph aWall thickness origin corner height |
	wallGlyph := SolidWallGlyph new
				basicPosition: position;
				dimension: dimension;
				yourself.
	aWall := wallGlyph buildWestWall.
	self assert: (aWall notNil and: [aWall isKindOf: JunOpenGL3dObject]).
	thickness := wallGlyph class defaultWallThickness.
	height := wallGlyph class defaultWallHeight.
	origin := position.
	self assert: (aWall boundingBox origin - origin) abs < epsilon3d.
	corner := (position x + thickness), (position y + dimension y), (position z + height).
	self assert: (aWall boundingBox corner - corner) abs < epsilon3d! !

!SolidWallGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInnerDimension
	| wall wallThickness |
	wall := SolidWallGlyph new
				basicPosition: position;
				yourself.
	wallThickness := wall class defaultWallThickness.
	"setter"
	wall innerDimension: dimension.
	self assert: wall dimension x = (dimension x + (wallThickness * 2)).
	self assert: wall dimension y = (dimension y + (wallThickness * 2)).
	self assert: wall dimension z = dimension z.
	"getter"
	self assert: wall innerDimension = dimension! !

!SolidWallGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInnerOrigin
	| wall wallThickness innerOrigin|
	wall := SolidWallGlyph new
				basicPosition: position;
				dimension: dimension;
				yourself.
	wallThickness := wall class defaultWallThickness.
	"getter"
	innerOrigin := wall innerOrigin.
	self assert: innerOrigin x = (position x + wallThickness).
	self assert: innerOrigin y = (position y + wallThickness).
	self assert: innerOrigin z = position z.
	"setter"
	wall basicInnerOrigin: position.
	self assert: wall position = (position - (wallThickness, wallThickness, 0))! !

TestCase subclass: #SpaceManagerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!SpaceManagerTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBoundingBoxAroundGlyphs
	|glyph1 glyph2 glyph3 spaceManager boundingBox|
	glyph1 := CuboidGlyph new.
	glyph2 := CuboidGlyph new.
	glyph3 := CuboidGlyph new.
	glyph1 basicPosition: 4,1,0; dimension: 1,2,5.
	glyph2 basicPosition: 1,2,1; dimension: 5,3,3.
	glyph3 basicPosition: 2,4,2; dimension: 1,2,3.
	spaceManager := SpaceManager uniqueInstance.
	self assert: spaceManager isNil not.
	boundingBox := spaceManager boundingBoxAroundGlyphs:
		(Array with: glyph1 with: glyph2 with: glyph3).
	self assert: boundingBox origin = (1,1,0).
	self assert: boundingBox dimension = (5,5,5).! !

TestCase subclass: #StreetJunctionBuilderTest
	instanceVariableNames: 'buildings spacing expectedDict expectedJunctions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!StreetJunctionBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
buildBuildings
	| buildingA buildingB buildingC buildingD buildingE |

	buildingA := self createCuboid basicPosition:  2 ,  2 , 2; dimension:  6 ,  6 , 10.
	buildingB := self createCuboid basicPosition: 10 ,  2 , 6; dimension:  6 , 10 , 2.
	buildingC := self createCuboid basicPosition:  2 , 10 , 8; dimension:  4 ,  4 , 4.
	buildingD := self createCuboid basicPosition:  4 , 16 , 2; dimension:  2 ,  2 , 6.
	buildingE := self createCuboid basicPosition:  8 , 14 , 0; dimension: 12 ,  6 , 20.

	^(OrderedCollection new)
				add: buildingA; add: buildingB; add: buildingC; add: buildingD; add: buildingE;
				yourself! !

!StreetJunctionBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
buildExpectedDictionary
	^(Dictionary new)
		at: 1 @ 1 put: (Set with: 2 + spacing);
		at: 1 @ 5 put: (Set with: 2 + spacing);
		at: 1 @ 9 put: (Set with: 2 + spacing with: 8 + spacing);
		at: 1 @ 12 put: (Set with: 8 + spacing);
		at: 1 @ 15 put: (Set with: 8 + spacing);
		at: 3 @ 15 put: (Set with: 2 + spacing);
		at: 3 @ 17 put: (Set with: 2 + spacing);
		at: 3 @ 19 put: (Set with: 2 + spacing);
		at: 4 @ 9 put: (Set with: 8 + spacing);
		at: 4 @ 15 put: (Set with: 8 + spacing);
		at: 5 @ 1 put: (Set with: 2 + spacing);
		at: 5 @ 9 put: (Set with: 2 + spacing);
		at: 5 @ 15 put: (Set with: 2 + spacing);
		at: 5 @ 19 put: (Set with: 2 + spacing);
		at: 7 @ 9 put: (Set with: 8 + spacing);
		at: 7 @ 12 put: (Set with: 8 + spacing);
		at: 7 @ 13 put: (Set with: 0 + spacing);
		at: 7 @ 15 put: (Set with: 8 + spacing with: 2 + spacing);
		at: 7 @ 17 put: (Set with: 2 + spacing with: 0 + spacing);
		at: 7 @ 19 put: (Set with: 2 + spacing);
		at: 7 @ 21 put: (Set with: 0 + spacing);
		at: 9 @ 1 put: (Set with: 2 + spacing with: 6 + spacing);
		at: 9 @ 5 put: (Set with: 2 + spacing);
		at: 9 @ 7 put: (Set with: 6 + spacing);
		at: 9 @ 9 put: (Set with: 2 + spacing);
		at: 9 @ 13 put: (Set with: 6 + spacing);
		at: 13 @ 1 put: (Set with: 6 + spacing);
		at: 13 @ 13 put: (Set with: 6 + spacing);
		at: 14 @ 13 put: (Set with: 0 + spacing);
		at: 14 @ 21 put: (Set with: 0 + spacing);
		at: 17 @ 1 put: (Set with: 6 + spacing);
		at: 17 @ 7 put: (Set with: 6 + spacing);
		at: 17 @ 13 put: (Set with: 6 + spacing);
		at: 21 @ 13 put: (Set with: 0 + spacing);
		at: 21 @ 17 put: (Set with: 0 + spacing);
		at: 21 @ 21 put: (Set with: 0 + spacing);
		yourself! !

!StreetJunctionBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
buildExpectedJunctions
	^(OrderedCollection new)
		add: 1,1, 3; add: 1, 5, 3; add: 1, 9, 6; add: 1, 12, 9; add: 1, 15, 9;
		add: 3, 15, 3; add: 3, 17, 3; add: 3, 19, 3;
		add: 4, 9, 9; add: 4, 15, 9;
		add: 5, 1, 3; add: 5, 9, 3; add: 5, 15, 3; add: 5, 19, 3;
		add: 7, 9, 9; add: 7, 12, 9; add: 7, 13, 1; add: 7, 15, 6; add: 7, 17, 2; add: 7, 19, 3; add: 7, 21, 1;
		add: 9, 1, 5; add: 9, 5, 3; add: 9, 7, 7; add: 9, 9, 3; add: 9, 13, 7;
		add: 13, 1, 7; add: 13, 13, 7;
		add: 14, 13, 1; add: 14, 21, 1;
		add: 17, 1, 7; add: 17, 7, 7; add: 17, 13, 7;
		add: 21, 13, 1; add: 21, 17, 1; add: 21, 21, 1; 
		yourself! !

!StreetJunctionBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
createCuboid
	^CuboidGlyph new! !

!StreetJunctionBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	buildings := self buildBuildings.
	spacing := 1.
	expectedDict := self buildExpectedDictionary.
	expectedJunctions := self buildExpectedJunctions! !

!StreetJunctionBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAddJunctionToInfrastructure
	|builder|
	builder := StreetJunctionBuilder newBuildings: buildings spacing: spacing.
	builder addJunctionToInfrastructure: 3, 4, 5.
	self assert: builder dic2dPointToHeightSet size = 1.
	self assert: (builder dic2dPointToHeightSet at: 3@4) size = 1.
	self assert: ((builder dic2dPointToHeightSet at: 3@4) includes: 5).
	builder addJunctionToInfrastructure: 3, 4, 7.
	self assert: builder dic2dPointToHeightSet size = 1.
	self assert: (builder dic2dPointToHeightSet at: 3@4) size = 2.
	self assert: ((builder dic2dPointToHeightSet at: 3@4) includes: 5).
	self assert: ((builder dic2dPointToHeightSet at: 3@4) includes: 7).
	builder addJunctionToInfrastructure: 3, 3, 8.
	self assert: builder dic2dPointToHeightSet size = 2.
	self assert: (builder dic2dPointToHeightSet at: 3@4) size = 2.
	self assert: ((builder dic2dPointToHeightSet at: 3@4) includes: 5).
	self assert: ((builder dic2dPointToHeightSet at: 3@4) includes: 7).
	self assert: (builder dic2dPointToHeightSet at: 3@3) size = 1.
	self assert: ((builder dic2dPointToHeightSet at: 3@3) includes: 8)! !

!StreetJunctionBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAddJunctionsToInfrastructure
	| builder junctions |
	builder := StreetJunctionBuilder newBuildings: buildings spacing: spacing.
	junctions := OrderedCollection with: 3, 4, 5 with: 3, 3, 8 with: 3, 4, 7.
	builder addJunctionsToInfrastructure: junctions.
	self assert: builder dic2dPointToHeightSet size = 2.
	self assert: (builder dic2dPointToHeightSet at: 3@4) size = 2.
	self assert: ((builder dic2dPointToHeightSet at: 3@4) includes: 5).
	self assert: ((builder dic2dPointToHeightSet at: 3@4) includes: 7).
	self assert: (builder dic2dPointToHeightSet at: 3@3) size = 1.
	self assert: ((builder dic2dPointToHeightSet at: 3@3) includes: 8)! !

!StreetJunctionBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildJunctionInfrastructure
	| builder dict |
	builder := StreetJunctionBuilder newBuildings: buildings spacing: spacing.
	builder buildJunctionInfrastructure.
	dict := builder dic2dPointToHeightSet.
	self assert: dict keys size = 36.
	expectedDict keys do: 
			[:each | 
			| valueSet expectedValueSet |
			valueSet := dict at: each.
			expectedValueSet := expectedDict at: each.
			self assert: (valueSet equalsTo: expectedValueSet)]! !

!StreetJunctionBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildJunctions
	| builder junctions |
	builder := StreetJunctionBuilder newBuildings: buildings spacing: spacing.
	junctions := builder buildJunctions.
	self assert: junctions size = 36.
	self assert: (junctions equalsTo: expectedJunctions)! !

!StreetJunctionBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCreation
	| builder  |
	builder := StreetJunctionBuilder newBuildings: buildings spacing: spacing.
	self assert: builder buildings = buildings.
	self assert: builder spacing = spacing.
	self assert: builder dic2dPointToHeightSet isEmpty! !

!StreetJunctionBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testJunctionAt
	| builder |
	builder := StreetJunctionBuilder newBuildings: buildings spacing: spacing.
	builder buildJunctionInfrastructure.
	self assert: (builder junctionAt: 1 @ 1) = (1 , 1 , 3).
	self assert: (builder junctionAt: 1 @ 9) 
				= (1 , 9 , ((2 + spacing + (8 + spacing)) / 2)).
	self assert: (builder junctionAt: 7 @ 15) 
				= (7 , 15 , ((8 + spacing + (2 + spacing)) / 2)).
	self assert: (builder junctionAt: 7 @ 17) 
				= (7 , 17 , ((2 + spacing + (0 + spacing)) / 2)).
	self assert: (builder junctionAt: 9 @ 1) 
				= (9 , 1 , ((2 + spacing + (6 + spacing)) / 2))! !

!StreetJunctionBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testJunctionEastOfBuilding
	| builder building expectedJunction |
	building := self createCuboid basicPosition:  5 ,  7 , 9; dimension:  10 ,  16 , 22.
	spacing := 3.
	expectedJunction := 18, 15, 12.
	builder := StreetJunctionBuilder newBuildings: buildings spacing: spacing.
	self assert: (builder junctionEastOfBuilding: building) = expectedJunction! !

!StreetJunctionBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testJunctionNorthEastOfBuilding
	| builder building expectedJunction |
	building := self createCuboid basicPosition:  5 ,  7 , 9; dimension:  10 ,  16 , 22.
	spacing := 3.
	expectedJunction := 18, 4, 12.
	builder := StreetJunctionBuilder newBuildings: buildings spacing: spacing.
	self assert: (builder junctionNorthEastOfBuilding: building) = expectedJunction! !

!StreetJunctionBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testJunctionNorthOfBuilding
	| builder building expectedJunction |
	building := self createCuboid basicPosition:  5 ,  7 , 9; dimension:  10 ,  16 , 22.
	spacing := 3.
	expectedJunction := 10, 4, 12.
	builder := StreetJunctionBuilder newBuildings: buildings spacing: spacing.
	self assert: (builder junctionNorthOfBuilding: building) = expectedJunction! !

!StreetJunctionBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testJunctionNorthWestOfBuilding
	| builder building expectedJunction |
	building := self createCuboid basicPosition:  5 ,  7 , 9; dimension:  10 ,  16 , 22.
	spacing := 3.
	expectedJunction := 2, 4, 12.
	builder := StreetJunctionBuilder newBuildings: buildings spacing: spacing.
	self assert: (builder junctionNorthWestOfBuilding: building) = expectedJunction! !

!StreetJunctionBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testJunctionSouthEastOfBuilding
	| builder building expectedJunction |
	building := self createCuboid basicPosition:  5 ,  7 , 9; dimension:  10 ,  16 , 22.
	spacing := 3.
	expectedJunction := 18, 26, 12.
	builder := StreetJunctionBuilder newBuildings: buildings spacing: spacing.
	self assert: (builder junctionSouthEastOfBuilding: building) = expectedJunction! !

!StreetJunctionBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testJunctionSouthOfBuilding
	| builder building expectedJunction |
	building := self createCuboid basicPosition:  5 ,  7 , 9; dimension:  10 ,  16 , 22.
	spacing := 3.
	expectedJunction := 10, 26, 12.
	builder := StreetJunctionBuilder newBuildings: buildings spacing: spacing.
	self assert: (builder junctionSouthOfBuilding: building) = expectedJunction! !

!StreetJunctionBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testJunctionSouthWestOfBuilding
	| builder building expectedJunction |
	building := self createCuboid basicPosition:  5 ,  7 , 9; dimension:  10 ,  16 , 22.
	spacing := 3.
	expectedJunction := 2, 26, 12.
	builder := StreetJunctionBuilder newBuildings: buildings spacing: spacing.
	self assert: (builder junctionSouthWestOfBuilding: building) = expectedJunction! !

!StreetJunctionBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testJunctionWestOfBuilding
	| builder building expectedJunction |
	building := self createCuboid basicPosition:  5 ,  7 , 9; dimension:  10 ,  16 , 22.
	spacing := 3.
	expectedJunction := 2, 15, 12.
	builder := StreetJunctionBuilder newBuildings: buildings spacing: spacing.
	self assert: (builder junctionWestOfBuilding: building) = expectedJunction! !

!StreetJunctionBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testJunctionsAroundBuilding
	| builder junctionsAroundA junctionsAroundB |
	junctionsAroundA := (OrderedCollection new)
		add: 1, 1, 3;	add: 5, 1, 3;	add: 9, 1, 3;
		add: 1, 5, 3;					add: 9, 5, 3;
		add: 1, 9, 3;	add: 5, 9, 3;	add: 9, 9, 3;
		yourself.
	junctionsAroundB := (OrderedCollection new)
		add: 9, 1, 7;		add: 13, 1, 7;		add: 17, 1, 7;
		add: 9, 7, 7;							add: 17, 7, 7;
		add: 9, 13, 7;		add: 13, 13, 7;	add: 17, 13, 7;
		yourself.
	builder := StreetJunctionBuilder newBuildings: buildings spacing: spacing.
	self assert: (builder junctionsAroundBuilding: buildings first) = junctionsAroundA.
	self assert: (builder junctionsAroundBuilding: buildings second) = junctionsAroundB! !

TestCase subclass: #StreetNetworkBuilderTest
	instanceVariableNames: 'buildings expectedStreetNetwork expectedJunctions horizontalSegments verticalsSegments verticalSegments connectionSegments expectedWallJunctions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!StreetNetworkBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
buildBuildings
	| buildingA buildingB buildingC buildingD buildingE |

	buildingA := self createCuboid basicPosition:  2 ,  2 , 2; dimension:  6 ,  6 , 10.
	buildingB := self createCuboid basicPosition: 10 ,  2 , 6; dimension:  6 , 10 , 2.
	buildingC := self createCuboid basicPosition:  2 , 10 , 8; dimension:  4 ,  4 , 4.
	buildingD := self createCuboid basicPosition:  4 , 16 , 2; dimension:  2 ,  2 , 6.
	buildingE := self createCuboid basicPosition:  8 , 14 , 0; dimension: 12 ,  6 , 20.

	^(OrderedCollection new)
				add: buildingA; add: buildingB; add: buildingC; add: buildingD; add: buildingE;
				yourself! !

!StreetNetworkBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
buildConnectionSegments
	^(OrderedCollection new)
		add: (StreetSegment from: 5, 2, 3 to: 5, 1, 3);				"A"
		add: (StreetSegment from: 8, 5, 3 to: 9, 5, 3);
		add: (StreetSegment from: 5, 8, 3 to: 5, 9, 3);
		add: (StreetSegment from: 2, 5, 3 to: 1, 5, 3);
		add: (StreetSegment from: 13, 2, 7 to: 13, 1, 7);			"B"
		add: (StreetSegment from: 16, 7, 7 to: 17, 7, 7);
		add: (StreetSegment from: 13, 12, 7 to: 13, 13, 7);
		add: (StreetSegment from: 10, 7, 7 to: 9, 7, 7);
		add: (StreetSegment from: 4, 10, 9 to: 4, 9, 9);			"C"
		add: (StreetSegment from: 6, 12, 9 to: 7, 12, 9);
		add: (StreetSegment from: 4, 14, 9 to: 4, 15, 9);
		add: (StreetSegment from: 2, 12, 9 to: 1, 12, 9);
		add: (StreetSegment from: 5, 16, 3 to: 5, 15, 3);			"D"
		add: (StreetSegment from: 6, 17, 3 to: 7, 17, 2);			"common with E, which balances the heght"
		add: (StreetSegment from: 5, 18, 3 to: 5, 19, 3);
		add: (StreetSegment from: 4, 17, 3 to: 3, 17, 3);
		add: (StreetSegment from: 14 , 14 , 1 to: 14 , 13 , 1);		"E"
		add: (StreetSegment from: 20 , 17 , 1 to: 21 , 17 , 1);
		add: (StreetSegment from: 14 , 20 , 1 to: 14 , 21 , 1);
		add: (StreetSegment from: 8 , 17 , 1 to: 7 , 17 , 2); 		"common with D"
		yourself! !

!StreetNetworkBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
buildExpectedJunctions
	|jointsA jointsB jointsC jointsD jointsE|
	jointsA := (OrderedCollection new)
		add: 1 , 1 , 3;					add: 5 , 1 , 3;				add: 9 , 1 , ((3 + 7) / 2);
		add: 1 , 5 , 3;												add: 9 , 5 , 3;
		add: 1 , 9 , ((3 + 9) / 2);	add: 5 , 9 , 3;				add: 9 , 9 , 3;
		yourself.
	jointsB := (OrderedCollection new)
		add: 9 ,  1 , ((3 + 7) / 2); 	add: 13 ,  1 , 7; 			add: 17 ,  1 , 7;
		add: 9 ,  7 , 7;											add: 17 ,  7 , 7;
		add: 9 , 13 , 7;				add: 13 , 13 , 7;			add: 17 , 13 , 7;
		yourself.
	jointsC := (OrderedCollection new)
		add: 1 ,  9 , ((3 + 9) / 2);		add: 4 ,  9 , 9;		add: 7 ,  9 , 9;
		add: 1 , 12 , 9;											add: 7 , 12 , 9;
		add: 1 , 15 , 9;					add: 4 , 15 , 9;		add: 7 , 15 , ((9 + 3) / 2);
		yourself.
	jointsD := (OrderedCollection new)
		add: 3 , 15 , 3; 					add: 5 , 15 , 3; 		add: 7 , 15 , ((9 + 3) / 2);
		add: 3 , 17 , 3;											add: 7 , 17 , ((3 + 1) / 2);
		add: 3 , 19 , 3;					add: 5 , 19 , 3; 		add: 7 , 19 , 3;
		yourself.
	jointsE := (OrderedCollection new)
		add: 7 , 13 , 1;	 				add: 14 , 13 , 1;		add: 21 , 13 , 1;
		add: 7 , 17 , ((3 + 1) / 2);								add: 21 , 17 , 1;
		add: 7 , 21 , 1;					add: 14 , 21 , 1;		add: 21 , 21 , 1;
		yourself.
	^(Set new) 
		addAll: jointsA; addAll: jointsB; addAll: jointsC; addAll: jointsD; addAll: jointsE; 
		yourself.! !

!StreetNetworkBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
buildExpectedSegments
	horizontalSegments := self buildHorizontalSegments.
	horizontalSegments do: [:each | expectedStreetNetwork addBothWaysSegment: each].
	verticalSegments := self buildVerticalSegments.
	verticalSegments do: [:each | expectedStreetNetwork addBothWaysSegment: each]! !

!StreetNetworkBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
buildExpectedWallJunctions
	| wallsA wallsB wallsC wallsD wallsE |
	wallsA := OrderedCollection 
					with: 5 , 2 , 3 with: 8 , 5 , 3 with: 5 , 8 , 3 with: 2 , 5 , 3.
	wallsB := OrderedCollection 
					with: 13 , 2 , 7 with: 16 , 7 , 7 with: 13 , 12 , 7 with: 10 , 7 , 7.
	wallsC := OrderedCollection 
					with: 4 , 10 , 9 with: 6 , 12 , 9 with: 4 , 14 , 9 with: 2 , 12 , 9.
	wallsD := OrderedCollection 
					with: 5 , 16 , 3 with: 6 , 17 , 3 with: 5 , 18 , 3 with: 4 , 17 , 3.
	wallsE := OrderedCollection 
					with: 14 , 14 , 1 with: 20 , 17 , 1 with: 14 , 20 , 1 with: 8 , 17 , 1.
	^(Set new)
		addAll: wallsA; addAll: wallsB; addAll: wallsC; addAll: wallsD; addAll: wallsE;
		yourself! !

!StreetNetworkBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
buildHorizontalSegments
	^(OrderedCollection new)
		add: (StreetSegment from: 1, 1, 3 to: 5, 1, 3);
		add: (StreetSegment from: 5, 1, 3 to: 9, 1, 5);
		add: (StreetSegment from: 9, 1, 5 to: 13, 1, 7);
		add: (StreetSegment from: 13, 1, 7 to: 17, 1, 7);
		add: (StreetSegment from: 1, 9, 6 to: 4, 9, 9);
		add: (StreetSegment from: 4, 9, 9 to: 5, 9, 3);
		add: (StreetSegment from: 5, 9, 3 to: 7, 9, 9);
		add: (StreetSegment from: 7, 9, 9 to: 9, 9, 3);
		add: (StreetSegment from: 7, 13, 1 to: 9, 13, 7);
		add: (StreetSegment from: 9, 13, 7 to: 13, 13, 7);
		add: (StreetSegment from: 13, 13, 7 to: 14, 13, 1);
		add: (StreetSegment from: 14, 13, 1 to: 17, 13, 7);
		add: (StreetSegment from: 17, 13, 7 to:  21, 13, 1);
		add: (StreetSegment from: 1, 15, 9 to: 3, 15, 3);
		add: (StreetSegment from: 3, 15, 3 to: 4, 15, 9);
		add: (StreetSegment from: 4, 15, 9 to: 5, 15, 3);
		add: (StreetSegment from: 5, 15, 3 to: 7, 15, 6);
		add: (StreetSegment from: 3, 19, 3 to: 5, 19, 3);
		add: (StreetSegment from: 5, 19, 3 to: 7, 19, 3);
		add: (StreetSegment from: 7, 21, 1 to: 14, 21, 1);
		add: (StreetSegment from: 14, 21, 1 to: 21, 21, 1);
		yourself! !

!StreetNetworkBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
buildVerticalSegments
	^(OrderedCollection new)
		add: (StreetSegment from: 1, 1, 3 to: 1, 5, 3);
		add: (StreetSegment from: 1, 5, 3 to: 1, 9, 6);
		add: (StreetSegment from: 1, 9, 6 to: 1, 12, 9);
		add: (StreetSegment from: 1, 12, 9 to: 1, 15, 9);
		add: (StreetSegment from: 3, 15, 3 to: 3, 17, 3);
		add: (StreetSegment from: 3, 17, 3 to: 3, 19, 3);
		add: (StreetSegment from: 7, 9, 9 to: 7, 12, 9);
		add: (StreetSegment from: 7, 12, 9 to: 7, 13, 1);
		add: (StreetSegment from: 7, 13, 1 to: 7, 15, 6);
		add: (StreetSegment from: 7, 15, 6 to: 7, 17, 2);
		add: (StreetSegment from: 7, 17, 2 to: 7, 19, 3);
		add: (StreetSegment from: 7, 19, 3 to: 7, 21, 1);
		add: (StreetSegment from: 9, 1, 5 to:  9, 5, 3);
		add: (StreetSegment from: 9, 5, 3 to: 9, 7, 7);
		add: (StreetSegment from: 9, 7, 7 to: 9, 9, 3);
		add: (StreetSegment from: 9, 9, 3 to: 9, 13, 7);
		add: (StreetSegment from: 17, 1, 7 to: 17, 7, 7);
		add: (StreetSegment from: 17, 7, 7 to: 17, 13, 7);
		add: (StreetSegment from: 21, 13, 1 to: 21, 17, 1);
		add: (StreetSegment from: 21, 17, 1 to: 21, 21, 1);
		yourself! !

!StreetNetworkBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
createCuboid
	^CuboidGlyph new! !

!StreetNetworkBuilderTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	buildings := self buildBuildings.
	expectedStreetNetwork := StreetNetwork new.
	expectedJunctions := self buildExpectedJunctions.
	expectedStreetNetwork addJunctions: expectedJunctions.
	self buildExpectedSegments.
	expectedWallJunctions := self buildExpectedWallJunctions.
	expectedStreetNetwork addJunctions: expectedWallJunctions.
	connectionSegments := self buildConnectionSegments.
	connectionSegments do: [:each | expectedStreetNetwork addBothWaysSegment: each]! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAddToObstaclesXAxisClosureForJunctions
	| obstacles junctions builder |
	junctions := OrderedCollection with: 1, 5, 2 with: 9, 5, 2 with: 17, 5, 2.
	obstacles := OrderedCollection with: 5, 5, 2 with: 13, 5, 2.
	builder := StreetNetworkBuilder new.
	builder addToObstacles: obstacles xAxisClosureForJunctions: junctions.
	self assert: obstacles size = 3.
	self assert: obstacles last = (18, 5, 2)! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAddToObstaclesYAxisClosureForJunctions
	| obstacles junctions builder |
	junctions := OrderedCollection with: 5, 1, 2 with: 5, 9, 2 with: 5, 17, 2.
	obstacles := OrderedCollection with: 5, 5, 2 with: 5, 13, 2.
	builder := StreetNetworkBuilder new.
	builder addToObstacles: obstacles yAxisClosureForJunctions: junctions.
	self assert: obstacles size = 3.
	self assert: obstacles last = (5, 18, 2)! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildHorizontalSegmentsOverJunctionsSplitByObstacles
	| junctions obstacles builder segments |
	builder := StreetNetworkBuilder new.
	junctions := (OrderedCollection new) 
					add: 3, 5, 2; add: 4, 5, 2; 
					add: 7, 5, 2; 
					add: 9, 5, 2; add: 12, 5, 2; add: 15, 5, 2;	
					yourself.
	obstacles := OrderedCollection with: 5, 5, 2 with: 8, 5, 2 with: 16, 5, 2.
	segments := builder buildHorizontalSegmentsOverJunctions: junctions splitByObstacles: obstacles.
	self assert: segments size = 3.
	self assert: segments first = (StreetSegment from: junctions first to: junctions second).
	self assert: segments second = (StreetSegment from: (junctions at: 4) to: (junctions at: 5)).
	self assert: segments third = (StreetSegment from: (junctions at: 5) to: junctions last)! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildSegmentsFromJunctionSequence
	|  junctions builder segments expectedSegments |
	junctions := (OrderedCollection new) add: 9, 1, 5; add: 9, 5, 3 ; add: 9, 7, 7; add: 9, 9, 3; add: 9, 13, 7; yourself.
	builder := StreetNetworkBuilder new.
	segments := builder buildSegmentsFromJunctionSequence: junctions.
	self assert: segments size = 4.
	expectedSegments := OrderedCollection
		with: (StreetSegment from: 9, 1, 5 to: 9, 5, 3)
		with: (StreetSegment from: 9, 5, 3 to: 9, 7, 7)
		with: (StreetSegment from: 9, 7, 7 to: 9, 9, 3)
		with: (StreetSegment from: 9, 9, 3 to: 9, 13, 7).
	self assert: segments = expectedSegments! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildStreetNetworkAroundBuildingsGap
	| streetNetwork |
	streetNetwork := StreetNetworkBuilder new 
				   		buildStreetNetworkAroundBuildings: buildings gap: 2.
	self assert: streetNetwork junctions equals: expectedStreetNetwork junctions.
	self assert: streetNetwork segments equals: expectedStreetNetwork segments.! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildVerticalSegmentsOverJunctionsSplitByObstacles
	| junctions obstacles builder segments |
	builder := StreetNetworkBuilder new.
	junctions := (OrderedCollection new) 
					add: 5, 3, 2; add: 5, 4, 2; 
					add: 5, 7, 2; 
					add: 5, 9, 2; add: 5, 12, 2; add: 5, 15, 2;	
					yourself.
	obstacles := OrderedCollection with: 5, 5, 2 with: 5, 8, 2 with: 5, 16, 2.
	segments := builder buildVerticalSegmentsOverJunctions: junctions splitByObstacles: obstacles.
	self assert: segments size = 3.
	self assert: segments first = (StreetSegment from: junctions first to: junctions second).
	self assert: segments second = (StreetSegment from: (junctions at: 4) to: (junctions at: 5)).
	self assert: segments third = (StreetSegment from: (junctions at: 5) to: junctions last).! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConnectBuildingToStreetNetworkWithJunctionBuilder
	| builder streetNetwork junctionBuilder expectedSegments buildingWithSharedConnection |
	builder := StreetNetworkBuilder new.
	streetNetwork := StreetNetwork new.
	streetNetwork addJunctions: expectedJunctions.
	junctionBuilder := StreetJunctionBuilder newBuildings: buildings spacing: 1.
	junctionBuilder buildJunctionInfrastructure.
	self assert: streetNetwork segments isEmpty.
	self assert: streetNetwork junctions size = 36.
	expectedSegments := OrderedCollection 
				with: (StreetSegment from: 14 , 14 , 1 to: 14 , 13 , 1)
				with: (StreetSegment from: 20 , 17 , 1 to: 21 , 17 , 1)
				with: (StreetSegment from: 14 , 20 , 1 to: 14 , 21 , 1)
				with: (StreetSegment from: 7 , 17 , 2 to: 8 , 17 , 1).
	buildingWithSharedConnection := buildings last.
	builder 
		connectBuilding: buildingWithSharedConnection
		toStreetNetwork: streetNetwork
		withJunctionBuilder: junctionBuilder.
	self assert: streetNetwork segments size = 4.
	self assert: streetNetwork junctions size = 40.
	self assert: streetNetwork segments equals: expectedSegments! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConnectBuildingsToStreetNetworkWithJunctionBuilder
	| builder streetNetwork junctionBuilder expectedSegments |
	builder := StreetNetworkBuilder new.
	streetNetwork := StreetNetwork new.
	streetNetwork addJunctions: expectedJunctions.
	junctionBuilder := StreetJunctionBuilder newBuildings: buildings spacing: 1.
	junctionBuilder buildJunctionInfrastructure.
	self assert: streetNetwork segments isEmpty.
	self assert: streetNetwork junctions size = 36.
	expectedSegments := (OrderedCollection new)
		add: (StreetSegment from: 5, 2, 3 to: 5, 1, 3);	"building A's connections"
		add: (StreetSegment from: 8, 5, 3 to: 9, 5, 3);
		add: (StreetSegment from: 5, 8, 3 to: 5, 9, 3);
		add: (StreetSegment from: 2, 5, 3 to: 1, 5, 3);
		add: (StreetSegment from: 13, 2, 7 to: 13, 1, 7);	"building B's connections"
		add: (StreetSegment from: 16, 7, 7 to: 17, 7, 7);
		add: (StreetSegment from: 13, 12, 7 to: 13, 13, 7);
		add: (StreetSegment from: 10, 7, 7 to: 9, 7, 7);
		yourself.		
	builder connectBuildings: (buildings first:	2) toStreetNetwork: streetNetwork withJunctionBuilder: junctionBuilder.
	self assert: streetNetwork segments size = 8.
	self assert: streetNetwork junctions size = 44.
	self assert: streetNetwork segments equals: expectedSegments! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testEastConnectBuildingToStreetNetworkWithJunctionBuilder
	| builder streetNetwork junctionBuilder expectedSegments buildingWithSharedConnection |
	builder := StreetNetworkBuilder new.
	streetNetwork := StreetNetwork new.
	streetNetwork addJunctions: expectedJunctions.
	junctionBuilder := StreetJunctionBuilder newBuildings: buildings spacing: 1.
	junctionBuilder buildJunctionInfrastructure.
	self assert: streetNetwork segments isEmpty.
	self assert: streetNetwork junctions size = 36.
	expectedSegments := OrderedCollection 
				with: (StreetSegment from: 20 , 17 , 1 to: 21 , 17 , 1).
	buildingWithSharedConnection := buildings last.
	builder 
		eastConnectBuilding: buildingWithSharedConnection
		toStreetNetwork: streetNetwork
		withJunctionBuilder: junctionBuilder.
	self assert: streetNetwork segments size = 1.
	self assert: streetNetwork junctions size = 37.
	self assert: streetNetwork segments equals: expectedSegments! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testExtractSortedUniqueXCoordinatesFromJunctions
	| junctions builder expectedCoordinates coordinates |
	junctions := OrderedCollection with: (3, 0, 5) with: (1, 5, 4) with: (9, 2, 1) with: (3, 9, 7).
	expectedCoordinates := SortedCollection with: 1 with: 3 with: 9.
	builder := StreetNetworkBuilder new.
	coordinates := builder extractSortedUniqueXCoordinatesFromJunctions: junctions.
	self assert: coordinates = expectedCoordinates! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testExtractSortedUniqueYCoordinatesFromJunctions
	| junctions builder expectedCoordinates coordinates |
	junctions := OrderedCollection with: (0, 3, 5) with: (5, 1, 4) with: (2, 9, 1) with: (9, 3, 7).
	expectedCoordinates := SortedCollection with: 1 with: 3 with: 9.
	builder := StreetNetworkBuilder new.
	coordinates := builder extractSortedUniqueYCoordinatesFromJunctions: junctions.
	self assert: coordinates = expectedCoordinates! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testGenerateDummyObstaclesForBuildingIntersectingColumns
	| builder columns obstacles |
	builder := StreetNetworkBuilder new.
	columns := OrderedCollection with: 5.
	obstacles := builder generateDummyObstaclesForBuilding: buildings first intersectingColumns: columns.
	self assert: obstacles size = 1.
	self assert: obstacles first = (5, 5, 2)! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testGenerateDummyObstaclesForBuildingIntersectingRows
	| builder rows obstaclesForA obstaclesForB |
	builder := StreetNetworkBuilder new.
	rows := OrderedCollection with: 5.
	obstaclesForA := builder generateDummyObstaclesForBuilding: buildings first intersectingRows: rows.
	self assert: obstaclesForA size = 1.
	self assert: obstaclesForA first = (5, 5, 2).
	obstaclesForB := builder generateDummyObstaclesForBuilding: buildings second intersectingRows: rows.
	self assert: obstaclesForB size = 1.
	self assert: obstaclesForB first = (13, 5, 6)! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testGenerateDummyObstaclesForBuildingsIntersectingColumns
	| builder columns obstaclesForA obstaclesForC |
	builder := StreetNetworkBuilder new.
	columns := OrderedCollection with: 5.
	obstaclesForA := builder generateDummyObstaclesForBuilding: buildings first intersectingColumns: columns.
	self assert: obstaclesForA size = 1.
	self assert: obstaclesForA first = (5, 5, 2).
	obstaclesForC := builder generateDummyObstaclesForBuilding: buildings third intersectingColumns: columns.
	self assert: obstaclesForC size = 1.
	self assert: obstaclesForC first = (5, 12, 8)! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testGenerateDummyObstaclesForBuildingsIntersectingRows
	| builder rows buildingSet obstacles |
	builder := StreetNetworkBuilder new.
	rows := OrderedCollection with: 5.
	buildingSet := OrderedCollection with: buildings first with: buildings second.
	obstacles := builder generateDummyObstaclesForBuildings: buildingSet intersectingRows: rows.
	self assert: obstacles size = 2.
	self assert: obstacles first = (5, 5, 2).
	self assert: obstacles second = (13, 5, 6)! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testGenerateHorizontalSegmentAmongJunctionsAroundBuildings
	| builder segments |
	builder := StreetNetworkBuilder new.
	segments := builder 
				generateHorizontalSegmentsAmongJunctions: expectedJunctions
				aroundBuildings: buildings.
	self assert: (segments allSatisfy: #hasConstantY).
	self deny: (segments anySatisfy: #hasConstantX).
	self assert: segments = horizontalSegments! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testGenerateSegmentsAmongJunctionsAroundBuildings
	| builder segments|
	builder := StreetNetworkBuilder new.
	segments := builder generateSegmentsAmongJunctions: expectedStreetNetwork junctions aroundBuildings: buildings.
	self assert: segments notNil.
	self assert: (segments equalsTo: expectedStreetNetwork segments).! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testGenerateVerticalSegmentAmongJunctionsAroundBuildings
	| builder segments |
	builder := StreetNetworkBuilder new.
	segments := builder 
				generateVerticalSegmentsAmongJunctions: expectedJunctions
				aroundBuildings: buildings.
	self deny: (segments anySatisfy: #hasConstantY).
	self assert: (segments allSatisfy: #hasConstantX).
	self assert: segments = verticalSegments! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testNorthConnectBuildingToStreetNetworkWithJunctionBuilder
	| builder streetNetwork junctionBuilder expectedSegments buildingWithSharedConnection |
	builder := StreetNetworkBuilder new.
	streetNetwork := StreetNetwork new.
	streetNetwork addJunctions: expectedJunctions.
	junctionBuilder := StreetJunctionBuilder newBuildings: buildings spacing: 1.
	junctionBuilder buildJunctionInfrastructure.
	self assert: streetNetwork segments isEmpty.
	self assert: streetNetwork junctions size = 36.
	expectedSegments := OrderedCollection 
				with: (StreetSegment from: 14 , 14 , 1 to: 14 , 13 , 1).
	buildingWithSharedConnection := buildings last.
	builder 
		northConnectBuilding: buildingWithSharedConnection
		toStreetNetwork: streetNetwork
		withJunctionBuilder: junctionBuilder.
	self assert: streetNetwork segments size = 1.
	self assert: streetNetwork junctions size = 37.
	self assert: streetNetwork segments equals: expectedSegments! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testSortedSelectFromCoordinatesWithX
	| aCollectionOfCoordinates sortedSelectedCoordinates builder expected |
	builder := StreetNetworkBuilder new.
	aCollectionOfCoordinates := (OrderedCollection new)
				add: 5 , 19 , 2;
				add: 9 , 7 , 8;
				add: 1 , 1 , 2;
				add: 5 , 9 , 2;
				add: 5 , 1 , 2;
				add: 5 , 15 , 2;
				yourself.
	sortedSelectedCoordinates := builder 
				sortedSelectFrom: aCollectionOfCoordinates
				coordinatesWithX: 5.
	expected := OrderedCollection 
				with: 5 , 1 , 2
				with: 5 , 9 , 2
				with: 5 , 15 , 2
				with: 5 , 19 , 2.
	self assert: (sortedSelectedCoordinates equalsTo: expected)! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testSortedSelectFromCoordinatesWithY
	| aCollectionOfCoordinates sortedSelectedCoordinates builder expected |
	builder := StreetNetworkBuilder new.
	aCollectionOfCoordinates := (OrderedCollection new)
				add: 19 , 5 , 2;
				add: 7 , 9 , 8;
				add: 1 , 1 , 2;
				add: 9 , 5 , 2;
				add: 1 , 5 , 2;
				add: 15 , 5 , 2;
				yourself.
	sortedSelectedCoordinates := builder 
				sortedSelectFrom: aCollectionOfCoordinates
				coordinatesWithY: 5.
	expected := OrderedCollection 
				with: 1 , 5 , 2
				with: 9 , 5 , 2
				with: 15 , 5 , 2
				with: 19 , 5 , 2.
	self assert: (sortedSelectedCoordinates equalsTo: expected)! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testSouthConnectBuildingToStreetNetworkWithJunctionBuilder
	| builder streetNetwork junctionBuilder expectedSegments buildingWithSharedConnection |
	builder := StreetNetworkBuilder new.
	streetNetwork := StreetNetwork new.
	streetNetwork addJunctions: expectedJunctions.
	junctionBuilder := StreetJunctionBuilder newBuildings: buildings spacing: 1.
	junctionBuilder buildJunctionInfrastructure.
	self assert: streetNetwork segments isEmpty.
	self assert: streetNetwork junctions size = 36.
	expectedSegments := OrderedCollection 
				with: (StreetSegment from: 14 , 20 , 1 to: 14 , 21 , 1).
	buildingWithSharedConnection := buildings last.
	builder 
		southConnectBuilding: buildingWithSharedConnection
		toStreetNetwork: streetNetwork
		withJunctionBuilder: junctionBuilder.
	self assert: streetNetwork segments size = 1.
	self assert: streetNetwork junctions size = 37.
	self assert: streetNetwork segments equals: expectedSegments! !

!StreetNetworkBuilderTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testWestConnectBuildingToStreetNetworkWithJunctionBuilder
	| builder streetNetwork junctionBuilder expectedSegments buildingWithSharedConnection |
	builder := StreetNetworkBuilder new.
	streetNetwork := StreetNetwork new.
	streetNetwork addJunctions: expectedJunctions.
	junctionBuilder := StreetJunctionBuilder newBuildings: buildings spacing: 1.
	junctionBuilder buildJunctionInfrastructure.
	self assert: streetNetwork segments isEmpty.
	self assert: streetNetwork junctions size = 36.
	expectedSegments := OrderedCollection 
				with: (StreetSegment from: 7 , 17 , 2 to: 8 , 17 , 1).
	buildingWithSharedConnection := buildings last.
	builder 
		westConnectBuilding: buildingWithSharedConnection
		toStreetNetwork: streetNetwork
		withJunctionBuilder: junctionBuilder.
	self assert: streetNetwork segments size = 1.
	self assert: streetNetwork junctions size = 37.
	self assert: streetNetwork segments equals: expectedSegments! !

TestCase subclass: #StreetNetworkTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!StreetNetworkTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAddJunction
	|streetNetwork|
	streetNetwork := StreetNetwork new.
	streetNetwork addJunction: 1,2,3.
	self assert: streetNetwork junctions size = 1.
	self assert: streetNetwork junctions first = (1,2,3)! !

!StreetNetworkTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAddJunctions
	|streetNetwork junctions|
	streetNetwork := StreetNetwork new.
	junctions := OrderedCollection with: 1,2,3 with: 4,5,6.
	streetNetwork addJunctions: junctions.
	self assert: streetNetwork junctions size = 2.
	self assert: streetNetwork junctions = junctions! !

!StreetNetworkTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testNeighborJunctions
	| streetNetwork junctions segment1to2 segment2to3 |
	streetNetwork := StreetNetwork new.
	junctions := OrderedCollection 
				with: 1 , 2 , 3
				with: 4 , 5 , 6
				with: 7 , 8 , 9.
	streetNetwork addJunctions: junctions.
	segment1to2 := StreetSegment from: junctions first to: junctions second.
	segment2to3 := StreetSegment from: junctions second to: junctions last.
	streetNetwork
		addBothWaysSegment: segment1to2;
		addBothWaysSegment: segment2to3.
	self assert: (streetNetwork neighborJunctionsOf: junctions first) size = 1.
	self assert: ((streetNetwork neighborJunctionsOf: junctions first) includes: junctions second).
	self assert: (streetNetwork neighborJunctionsOf: junctions second) size = 2.
	self assert: ((streetNetwork neighborJunctionsOf: junctions second) includesAll: (OrderedCollection with: junctions first with: junctions last)).
	self assert: (streetNetwork neighborJunctionsOf: junctions last) size = 1.
	self assert: ((streetNetwork neighborJunctionsOf: junctions last) includes: junctions second).! !

!StreetNetworkTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testSegments
	| streetNetwork junctions segment1to2 segment2to3 |
	streetNetwork := StreetNetwork new.
	junctions := OrderedCollection 
				with: 1 , 2 , 3
				with: 4 , 5 , 6
				with: 7 , 8 , 9.
	streetNetwork addJunctions: junctions.
	segment1to2 := StreetSegment from: junctions first to: junctions second.
	segment2to3 := StreetSegment from: junctions second to: junctions last.
	streetNetwork
		addBothWaysSegment: segment1to2;
		addBothWaysSegment: segment2to3.
	self assert: (streetNetwork segmentsStartingIn: junctions first) size = 1.
	self assert: ((streetNetwork segmentsStartingIn: junctions first) includes: segment1to2).
	self assert: (streetNetwork segmentsStartingIn: junctions second) size = 2.
	self assert: ((streetNetwork segmentsStartingIn: junctions second) includesAll: (OrderedCollection with: segment1to2 with: segment2to3)).
	self assert: (streetNetwork segmentsStartingIn: junctions last) size = 1.
	self assert: ((streetNetwork segmentsStartingIn: junctions last) includes: segment2to3).! !

!StreetNetworkTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testWeightOfSegmentBetweenAnd
	| streetNetwork junctions segment1to2 segment2to3 |
	streetNetwork := StreetNetwork new.
	junctions := OrderedCollection 
				with: 1 , 2 , 3
				with: 7 , 5 , 6
				with: 13 , 12 , 9.
	streetNetwork addJunctions: junctions.
	segment1to2 := StreetSegment from: junctions first to: junctions second.
	segment2to3 := StreetSegment from: junctions second to: junctions last.
	streetNetwork
		addBothWaysSegment: segment1to2;
		addBothWaysSegment: segment2to3.
	self assert: (streetNetwork weightOfSegmentBetween: junctions first and: junctions second) = segment1to2 weight.
	self assert: (streetNetwork weightOfSegmentBetween: junctions second and: junctions last) = segment2to3 weight! !

TestCase subclass: #StreetRouterTest
	instanceVariableNames: 'buildings streetNetwork simpleStreetNetwork'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!StreetRouterTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
buildBuildings
	| buildingA buildingB buildingC buildingD buildingE |

	buildingA := self createCuboid basicPosition:  2 ,  2 , 2; dimension:  6 ,  6 , 10.
	buildingB := self createCuboid basicPosition: 10 ,  2 , 6; dimension:  6 , 10 , 2.
	buildingC := self createCuboid basicPosition:  2 , 10 , 8; dimension:  4 ,  4 , 4.
	buildingD := self createCuboid basicPosition:  4 , 16 , 2; dimension:  2 ,  2 , 6.
	buildingE := self createCuboid basicPosition:  8 , 14 , 0; dimension: 12 ,  6 , 20.

	^(OrderedCollection new)
				add: buildingA; add: buildingB; add: buildingC; add: buildingD; add: buildingE;
				yourself! !

!StreetRouterTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
buildSimpleStreetNetwork
	|aStreetNetwork junctions segments |
	aStreetNetwork := StreetNetwork new.
	junctions := OrderedCollection new
		add: 1, 1, 2; 						add: 5, 1, 2; 						add: 9, 1, 2;
											add: 5, 2, 2;
		add: 1, 5, 2;	add: 2, 5, 2;							add: 8, 5, 2;	add: 9, 5, 2;
											add: 5, 8, 2;
		add: 1, 9, 2;						add: 5, 9, 2;						add: 9, 9, 2;
		yourself.
	aStreetNetwork addJunctions: junctions.
	segments := OrderedCollection new
		add: (StreetSegment from: 1, 1, 2 to: 5, 1, 2);
		add: (StreetSegment from: 5, 1, 2 to: 5, 2, 2);
		add: (StreetSegment from: 5, 1, 2 to: 9, 1, 2);
		add: (StreetSegment from: 9, 1, 2 to: 9, 5, 2);
		add: (StreetSegment from: 9, 5, 2 to: 8, 5, 2);
		add: (StreetSegment from: 9, 5, 2 to: 9, 9, 2);
		add: (StreetSegment from: 9, 9, 2 to: 5, 9, 2);
		add: (StreetSegment from: 5, 9, 2 to: 5, 8, 2);
		add: (StreetSegment from: 5, 9, 2 to: 1, 9, 2);
		add: (StreetSegment from: 1, 9, 2 to: 1, 5, 2);
		add: (StreetSegment from: 1, 5, 2 to: 2, 5, 2);
		add: (StreetSegment from: 1, 5, 2 to: 1, 1, 2);
		yourself.
	segments do: [:each|aStreetNetwork addBothWaysSegment: each].
	^aStreetNetwork! !

!StreetRouterTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
createCuboid
	^CuboidGlyph new! !

!StreetRouterTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	| builder |
	buildings := self buildBuildings.
	builder := StreetNetworkBuilder new.
	streetNetwork := builder buildStreetNetworkAroundBuildings: buildings gap: 2.
	simpleStreetNetwork := self buildSimpleStreetNetwork.! !

!StreetRouterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAlgorithmSetup
	| router infiniteSurrogate |
	router := StreetRouter newForStreetNetwork: simpleStreetNetwork.
	router algorithmSetup.
	infiniteSurrogate := 37.

	self assert: (router costFromStartVertex keys equalsTo: simpleStreetNetwork junctions).
	self assert: (router costFromStartVertex values allSatisfy: [:each | each = infiniteSurrogate]).

	self assert: (router previous keys equalsTo: simpleStreetNetwork junctions).
	self assert: (router previous values allSatisfy: #isNil).! !

!StreetRouterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testJunctionSequenceOfShortestPath
	| router expectedSequence routeSequence |
	router := StreetRouter newForStreetNetwork: streetNetwork.
	expectedSequence := (OrderedCollection new)
				add: 1 , 1 , 3;
				add: 5 , 1 , 3;
				add: 9 , 1 , 5;
				add: 13 , 1 , 7;
				add: 17 , 1 , 7;
				add: 17 , 7 , 7;
				add: 17 , 13 , 7;
				add: 14 , 13 , 1;
				add: 14 , 14 , 1;
				yourself.
	routeSequence := router 
				junctionSequenceOfShortestPathFrom: expectedSequence first
				to: expectedSequence last.
	self assert: routeSequence equals: expectedSequence! !

!StreetRouterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testSegmentSequenceOfShortestPath
	| router expectedSequence routeSequence |
	router := StreetRouter newForStreetNetwork: streetNetwork.
	expectedSequence := (OrderedCollection new)
		add: (StreetSegment from: 1, 1, 3 to: 5, 1, 3);
		add: (StreetSegment from: 5, 1, 3 to: 9, 1, 5);
		add: (StreetSegment from: 9, 1, 5 to: 13, 1, 7);
		add: (StreetSegment from: 13, 1, 7 to: 17, 1, 7);
		add: (StreetSegment from: 17, 1, 7 to: 17, 7, 7);
		add: (StreetSegment from: 17, 7, 7 to: 17, 13, 7);
		add: (StreetSegment from: 17, 13, 7 to: 14, 13, 1);
		add: (StreetSegment from: 14, 13, 1 to: 14, 14, 1);
				yourself.
	routeSequence := router 
				segmentSequenceOfShortestPathFrom: expectedSequence first firstEnd
				to: expectedSequence last secondEnd.
	self assert: routeSequence equals: expectedSequence! !

!StreetRouterTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testTotalStreetLength
	| router  |
	router := StreetRouter newForStreetNetwork: simpleStreetNetwork.
	self assert: router totalStreetLength = 36! !

TestCase subclass: #StreetSegmentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!StreetSegmentTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAnyTraversingRelationships
	| segment relationships mooseModel |
	segment := StreetSegment from: 0 , 2 , 3 to: 1 , 7 , 11.
	self assert: segment anyTraversingRelationships not.
	mooseModel := Importer withActiveMetamodel 
				importArray: #('Moose.Model' #(#entity #('FAMIX.Namespace' #(#id: 1) #(#name #X)) #('FAMIX.Class' #(#id: 2) #(#name #A) #(#belongsTo #(#idref: 1))) #('FAMIX.Class' #(#id: 3) #(#name #B) #(#belongsTo #(#idref: 1))) #('FAMIX.InheritanceDefinition' #(#superclass #(#idref: 2)) #(#subclass #(#idref: 3))))).
	relationships := OrderedCollection withAll: mooseModel allInheritanceDefinitions.
	segment addRelationships: relationships.
	self assert: segment anyTraversingRelationships.! !

!StreetSegmentTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testConstructor
	|streetSegment1 streetSegment2|
	streetSegment1 := StreetSegment from: 0,2,3 to: 1,7,11.
	self assert: streetSegment1 notNil.
	self assert: streetSegment1 firstEnd = (0,2,3).
	self assert: streetSegment1 secondEnd = (1,7,11).
	streetSegment2 := StreetSegment from: 1,7,11 to: 0,2,3.
	self assert: streetSegment2 firstEnd = (1,7,11).
	self assert: streetSegment2 secondEnd = (0,2,3).! !

!StreetSegmentTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testHasConstantX
	|parallel notParallel|
	parallel := StreetSegment from: 0,2,3 to: 0,7,11.
	self assert: parallel hasConstantX.
	notParallel := StreetSegment from: 3, 5, 7 to: 4, 5, 7.
	self deny: notParallel hasConstantX! !

!StreetSegmentTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testHasConstantY
	|parallel notParallel|
	parallel := StreetSegment from: 0,2,3 to: 4,2,11.
	self assert: parallel hasConstantY.
	notParallel := StreetSegment from: 3, 5, 7 to: 3, 0, 7.
	self deny: notParallel hasConstantY! !

!StreetSegmentTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testLength
	self assert: (StreetSegment from: 3, 5, 7 to: 3, 1, 4) length = 5.
	self assert: (StreetSegment from: 11,1, 2 to: 5, 1, 10) length = 10.
	self assert: (StreetSegment from: 20, 10, 0 sqrt to: 4, -2, 0) length = 20! !

!StreetSegmentTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testWeight
	|segment|
	segment := StreetSegment from: 0,2,3 to: 1,7,11.
	self assert: segment weight = segment length.! !

TestCase subclass: #TerrainGlyphTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!TerrainGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildVolume
	| terrain dimension position volume boundingBox epsilon epsilon3d terrainThickness|
	epsilon := 0.00000001.
	epsilon3d := epsilon , epsilon , epsilon.
	dimension := 13 , 14 , 15.
	position := 11 , 23 , 47.
	terrain := TerrainGlyph new
				basicPosition: position;
				dimension: dimension;
				yourself.
	terrainThickness := terrain class defaultTerrainThickness.
	volume := terrain buildVolume.
	self assert: volume notNil.
	self assert: (volume isKindOf: JunOpenGL3dObject).
	boundingBox := volume boundingBox.
	self assert: (boundingBox origin - position) abs < epsilon3d.
	self assert: (boundingBox corner x - dimension x - position x) abs < epsilon3d.
	self assert: (boundingBox corner y - dimension y - position y) abs < epsilon3d.
	self assert: (boundingBox corner z - terrainThickness - position z) abs < epsilon3d! !

!TerrainGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInnerDimension
	| terrain position terrainThickness expectedDimension dimension |
	position := 11 , 23 , 47.
	dimension := 13 , 14 , 15.
	terrain := TerrainGlyph new
				basicPosition: position;
				yourself.
	terrainThickness := terrain class defaultTerrainThickness.
	self assert: terrainThickness > 0.
	"setter"
	expectedDimension := dimension + (0, 0, terrainThickness).
	terrain innerDimension: dimension.
	self assert: terrain dimension = expectedDimension.
	"getter"
	self assert: terrain innerDimension = dimension! !

!TerrainGlyphTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInnerOrigin
	| terrain position terrainThickness expectedPosition |
	position := 11 , 23 , 47.
	terrain := TerrainGlyph new
				basicPosition: position;
				yourself.
	terrainThickness := terrain class defaultTerrainThickness.
	expectedPosition := position + (0 , 0 , terrainThickness).
	"getter"
	self assert: terrain innerOrigin = expectedPosition.
	"setter"
	terrain basicInnerOrigin: position.
	self assert: terrain position = (position x, position y, (position z - terrainThickness))! !

TestCase subclass: #ThresholdMapperModelTest
	instanceVariableNames: 'initialMapper glyphProperty modelClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!ThresholdMapperModelTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	glyphProperty := #color.
	modelClass := FAMIXMethod.
	initialMapper := ThresholdMapper new
		function: #numberOfLinesOfCode;
		codomain: #(2 4 6 8 10);
		thresholds: #(100 200 300 400)! !

!ThresholdMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCodomain
	| mapperModel |
	mapperModel := ThresholdMapperModel newMapper: nil forModelClass: modelClass.
	self assert: mapperModel codomain value = nil.
	mapperModel := ThresholdMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel codomain value = initialMapper codomain! !

!ThresholdMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testCreation
	| mapperModel |
	mapperModel := ThresholdMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel mapper = initialMapper! !

!ThresholdMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInitializeCodomain
	| mapperModel |
	mapperModel := ThresholdMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel initializeCodomain = initialMapper codomain! !

!ThresholdMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInitializeModelProperty
	| mapperModel |
	mapperModel := ThresholdMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel initializeModelProperty = initialMapper function! !

!ThresholdMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testInitializeThresholds
	| mapperModel |
	mapperModel := ThresholdMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel initializeThresholds = initialMapper thresholds! !

!ThresholdMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testModelProperty
	| mapperModel |
	mapperModel := ThresholdMapperModel newMapper: nil forModelClass: modelClass.
	self assert: mapperModel modelProperty value = nil.
	mapperModel := ThresholdMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel modelProperty value = initialMapper function! !

!ThresholdMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testModelPropertyList
	| mapperModel |
	mapperModel := ThresholdMapperModel newMapper: initialMapper
				forModelClass: modelClass.
	self assert: mapperModel modelPropertyList
		equals: (modelClass new allDeclaredProperties 
				collect: [:e | e definingMethod selector])! !

!ThresholdMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testOutputMapper
	| mapperModel newCodomain newModelProperty newThresholds resultingMapperModel |
	mapperModel := ThresholdMapperModel newMapper: initialMapper forModelClass: modelClass.
	newCodomain := #(10 9 8 7 5).
	newModelProperty := #numberOfStatements.
	newThresholds := #(3 4 5 6).
	mapperModel modelProperty value: newModelProperty.
	mapperModel codomain value: newCodomain.
	mapperModel thresholds value: newThresholds.
	resultingMapperModel := (initialMapper copy)
				function: newModelProperty;
				codomain: newCodomain;
				thresholds: newThresholds;
				yourself.
	self assert: mapperModel outputMapper = resultingMapperModel! !

!ThresholdMapperModelTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testThresholds
	| mapperModel |
	mapperModel := ThresholdMapperModel newMapper: nil forModelClass: modelClass.
	self assert: mapperModel codomain value = nil.
	mapperModel := ThresholdMapperModel newMapper: initialMapper forModelClass: modelClass.
	self assert: mapperModel thresholds value = initialMapper thresholds! !

TestCase subclass: #ThresholdMapperTest
	instanceVariableNames: 'mooseModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!ThresholdMapperTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
modelSpec
	^
#(Moose.Model (entity
	(FAMIX.Namespace (id: 1) (name TestNamespace))

	(FAMIX.Class (id: 2) (name TestClass1) (belongsTo (idref: 1)))
	(FAMIX.Method (name method1TC1) (signature 'method1TC1()') (belongsTo (idref: 2)) (LOC 2))

	(FAMIX.Class (id: 3) (name TestClass3) (belongsTo (idref: 1)))
	(FAMIX.Method (name method1TC3) (signature 'method1TC3()') (belongsTo (idref: 3)) (LOC 2))
	(FAMIX.Method (name method2TC3) (signature 'method2TC3()') (belongsTo (idref: 3)) (LOC 2))
	(FAMIX.Method (name method3TC3) (signature 'method3TC3()') (belongsTo (idref: 3)) (LOC 2))

	(FAMIX.Class (id: 4) (name TestClass5) (belongsTo (idref: 1)))
	(FAMIX.Method (name method1TC5) (signature 'method1TC5()') (belongsTo (idref: 4)) (LOC 2))
	(FAMIX.Method (name method2TC5) (signature 'method2TC5()') (belongsTo (idref: 4)) (LOC 2))
	(FAMIX.Method (name method3TC5) (signature 'method3TC5()') (belongsTo (idref: 4)) (LOC 2))
	(FAMIX.Method (name method4TC5) (signature 'method4TC5()') (belongsTo (idref: 4)) (LOC 2))
	(FAMIX.Method (name method5TC5) (signature 'method5TC5()') (belongsTo (idref: 4)) (LOC 2))

	(FAMIX.Class (id: 5) (name TestClass7) (belongsTo (idref: 1)))
	(FAMIX.Method (name method1TC7) (signature 'method1TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method2TC7) (signature 'method2TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method3TC7) (signature 'method3TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method4TC7) (signature 'method4TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method5TC7) (signature 'method5TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method6TC7) (signature 'method6TC7()') (belongsTo (idref: 5)) (LOC 2))
	(FAMIX.Method (name method7TC7) (signature 'method7TC7()') (belongsTo (idref: 5)) (LOC 2))

))! !

!ThresholdMapperTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	mooseModel := (Importer withActiveMetamodel importArray: self modelSpec) name: #'TestModel'! !

!ThresholdMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAsScript
	| aFunction instantiatedBlockMapper uninstantiatedBlockMapper population codomain scripted thresholds |
	aFunction := #size.
	population := OrderedCollection with: #(1 2 3) with: #($a $b $c $d) with: 'defgh'.
	codomain := #(1 4 5 6 8 11).
	thresholds := #(2 22 222).
	instantiatedBlockMapper := ThresholdMapper new 
										function: aFunction;
										population: population;
										codomain: codomain;
										thresholds: thresholds.
	self assert: instantiatedBlockMapper 
				= (Compiler evaluate: instantiatedBlockMapper asScript).
	uninstantiatedBlockMapper := ThresholdMapper new.
	self assert: uninstantiatedBlockMapper 
				= (Compiler evaluate: uninstantiatedBlockMapper asScript).
	scripted := Compiler evaluate: instantiatedBlockMapper asScript.
	self assert: scripted population isEmpty! !

!ThresholdMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildConverter
	| mapper values converter |
	mapper := ThresholdMapper new.
	values := #(10 -10 30 0).
	mapper population: values.
	converter := mapper buildConverter.
	self assert: converter class = PercentileConverter.
	self assert: converter minimumInput = -10.
	self assert: converter maximumInput = 30! !

!ThresholdMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testBuildEquidistantThresholds
	| mapper values thresholds expected |
	mapper := ThresholdMapper new.
	values := #(160 121 100 144).
	mapper population: values.
	thresholds := mapper buildEquidistantThresholds.
	expected := #(110 120 130 140 150 160).
	self assert: thresholds equals: expected! !

!ThresholdMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testComputeValueOnForThresholdsBothExtremes
	"values: 1 3 5 7"

	| mapper mappedValues |
	mapper := (ThresholdMapper new)
				population: mooseModel allClasses;
				function: #numberOfMethods;
				codomain: #(2 5 11 23);
				thresholds: #(0 2 6 10);
				yourself.
	mappedValues := mooseModel allClasses 
				collect: [:each | mapper computeValueOn: each].
	self assert: mappedValues equals: #(5 11 11 23)! !

!ThresholdMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testComputeValueOnForThresholdsLowerExtreme
	"values: 1 3 5 7"

	| mapper mappedValues |
	mapper := (ThresholdMapper new)
				population: mooseModel allClasses;
				function: #numberOfMethods;
				codomain: #(2 5 11 23);
				thresholds: #(0 2 6);
				yourself.
	mappedValues := mooseModel allClasses 
				collect: [:each | mapper computeValueOn: each].
	self assert: mappedValues equals: #(5 11 11 23)! !

!ThresholdMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testComputeValueOnForThresholdsNoExtremes
	"values: 1 3 5 7"

	| mapper mappedValues |
	mapper := (ThresholdMapper new)
				population: mooseModel allClasses;
				function: #numberOfMethods;
				codomain: #(2 5 11 23);
				thresholds: #(2 6);
				yourself.
	mappedValues := mooseModel allClasses 
				collect: [:each | mapper computeValueOn: each].
	self assert: mappedValues equals: #(2 5 5 11)! !

!ThresholdMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testComputeValueOnForThresholdsUpperExtreme
	"values: 1 3 5 7"

	| mapper mappedValues |
	mapper := (ThresholdMapper new)
				population: mooseModel allClasses;
				function: #numberOfMethods;
				codomain: #(2 5 11 23);
				thresholds: #(2 4 10);
				yourself.
	mappedValues := mooseModel allClasses 
				collect: [:each | mapper computeValueOn: each].
	self assert: mappedValues equals: #(2 5 11 11)! !

!ThresholdMapperTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testEquality
	| i1 i2 notI stillI1 |
	i1 := ThresholdMapper new
			function:  #size;
			codomain: #(1 2 3 4);
			thresholds: #(5 55 555).
	i2 := i1 copy.
	self assert: i1 = i2.
	#(#function: #codomain: #thresholds:) do: 
			[:each | 
			notI := i1 copy perform: each with: nil.
			self deny: notI = i1].
	"population is not taken into account for the equality test"
	stillI1 := i1 copy perform: #population: with: #(1 2).
	self assert: stillI1 = i1! !

TestCase subclass: #ViewConfigurationTest
	instanceVariableNames: 'configuration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeCityTest'!

!ViewConfigurationTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
buildModelElementSpecificationDictionary
	| dictionary specForClass specForInheritance |
	dictionary := Dictionary new.
	specForClass := (ModelElementSpecification new)
				visible: true;
				glyphClass: CuboidGlyph;
				layout: BricksLayout new;
				addMapper: (ThresholdMapper new
								function: #numberOfMethods;
								codomain: #(0.1 0.4 0.8 1.0);
								thresholds: #(2 4 6))
					onGlyphProperty: #alpha;
				addMapper: (ConstantMapper new 
								constantValue: 13) 
					onGlyphProperty: #height.
	specForInheritance := (ModelElementSpecification new)
				visible: false;
				glyphClass: LineGlyph;
				layout: (QuadraticLayout newHorizontalMargin: 2 horizontalGap: 1 verticalMargin: 3 verticalGap: 5);
				addMapper: (ConstantMapper new constantValue: 0.7) 
					onGlyphProperty: #alpha;
				addMapper: (ConstantMapper new constantValue: ColorValue red) 
					onGlyphProperty: #color.
	^dictionary 
		at: FAMIXClass put: specForClass;
		at: FAMIXInheritanceDefinition put: specForInheritance;
		yourself! !

!ViewConfigurationTest methodsFor: 'setup' stamp: 'StephaneDucasse 02/08/12 6:2'!
setUp
	configuration := ViewConfiguration new 
			name: 'TestConfiguration';
			description: 'Just a test';
			modelElementSpecificationDictionary: self buildModelElementSpecificationDictionary! !

!ViewConfigurationTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAddMapperOnGlyphPropertyForModelClass
	| spec mapper modelClass |
	modelClass := FAMIXClass.
	spec := configuration elementSpecForModelClass: modelClass.
	self assert: spec mapperDictionary size = 2.
	mapper := ConstantMapper new constantValue: 13.
	self deny: (spec mapperDictionary at: #alpha) = mapper.
	configuration addMapper: mapper onGlyphProperty: #alpha forModelClass: modelClass.
	self assert: spec mapperDictionary size = 2.
	self assert: (spec mapperDictionary at: #alpha) = mapper.
	configuration addMapper: mapper onGlyphProperty: #length forModelClass: modelClass.
	self assert: spec mapperDictionary size = 3.
	self assert: (spec mapperDictionary at: #length) = mapper.! !

!ViewConfigurationTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testAsScript
	| configurationFromScript |
	configurationFromScript := Compiler evaluate: configuration asScript.
	self assert: configuration name = configurationFromScript name.
	self assert: configuration description = configurationFromScript description.
	self assert: configuration modelElementSpecificationDictionary
		equals: configurationFromScript modelElementSpecificationDictionary! !

!ViewConfigurationTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testGlyphClassFor
	self assert:	(configuration glyphClassFor: FAMIXClass) = CuboidGlyph.
	self assert: (configuration glyphClassFor: FAMIXInheritanceDefinition) = LineGlyph! !

!ViewConfigurationTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testLayoutFor
	self assert: (configuration layoutFor: FAMIXClass) = BricksLayout new.
	self assert: (configuration layoutFor: FAMIXInheritanceDefinition) 
				= (QuadraticLayout 
						newHorizontalMargin: 2
						horizontalGap: 1
						verticalMargin: 3
						verticalGap: 5)! !

!ViewConfigurationTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testMapperForGlyphProperty
	self assert: (configuration mapperFor: FAMIXClass glyphProperty: #alpha) = 
					((ThresholdMapper new)
						function: #numberOfMethods;
						codomain: #(0.1 0.4 0.8 1.0);
						thresholds: #(2 4 6)).
	self assert: (configuration mapperFor: FAMIXClass glyphProperty: #height)
				= (ConstantMapper new constantValue: 13)! !

!ViewConfigurationTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testMappersFor
	| mapperDict |
	mapperDict := configuration mappersFor: FAMIXClass.
	self assert: mapperDict size = 2.
	self assert: (mapperDict at: #alpha) = ((ThresholdMapper new)
						function: #numberOfMethods;
						codomain: #(0.1 0.4 0.8 1.0);
						thresholds: #(2 4 6)).
	self assert: (mapperDict at: #height) 
				= (ConstantMapper new constantValue: 13)! !

!ViewConfigurationTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testSetGlyphClassForModelClass
	| spec  modelClass glyphClass alphaMapper |
	modelClass := FAMIXClass.
	spec := configuration elementSpecForModelClass: modelClass.
	glyphClass := TerrainGlyph.
	self deny: spec glyphClass = glyphClass.
	alphaMapper := spec mapperDictionary at: #alpha.
	self assert: alphaMapper notNil.
	configuration setGlyphClass: glyphClass forModelClass: modelClass.
	self assert: spec glyphClass = glyphClass.
	"make sure that the common-property mappers have been transfered"
	self assert: (spec mapperDictionary at: #alpha) = alphaMapper.
	self should: [spec mapperDictionary at: #height] raise: KeyNotFoundError! !

!ViewConfigurationTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testSetLayoutForModel
	| spec  modelClass layout |
	modelClass := FAMIXClass.
	spec := configuration elementSpecForModelClass: modelClass.
	layout := ChessboardLayout new.
	self deny: spec layout = layout.
	configuration setLayout: layout forModelClass: modelClass.
	self assert: spec layout = layout.! !

!ViewConfigurationTest methodsFor: 'running' stamp: 'StephaneDucasse 02/08/12 6:2'!
testSetVisibilityForModel
	| spec  modelClass |
	modelClass := FAMIXClass.
	spec := configuration elementSpecForModelClass: modelClass.
	self assert: spec visible.
	configuration setVisibility: false forModelClass: modelClass.
	self deny: spec visible! !
